@@ -46,6 +46,8 @@ Provides:
 """
 
 import bz2
+import itertools
+import src.lib.i18n as i18n
 import os
 
 import wx
@@ -60,13 +62,10 @@ from src.lib.selection import Selection
 from src.actions._main_window_actions import Actions
 from src.actions._grid_cell_actions import CellActions
 
-from src.gui._events import EVT_COMMAND_GRID_ACTION_NEW
-from src.gui._events import EVT_COMMAND_GRID_ACTION_TABLE_SWITCH
-from src.gui._events import EVT_COMMAND_GRID_ACTION_OPEN
-from src.gui._events import EVT_COMMAND_GRID_ACTION_SAVE
-from src.gui._events import post_command_event, SafeModeEntryMsg
-from src.gui._events import SafeModeExitMsg, StatusBarMsg, ContentChangedMsg
-from src.gui._events import ResizeGridMsg, GridActionTableSwitchMsg
+from src.gui._events import post_command_event
+
+#use ugettext instead of getttext to avoid unicode errors
+_ = i18n.language.ugettext
 
 
 class FileActions(Actions):
@@ -77,10 +76,10 @@ class FileActions(Actions):
 
         self.saving = False
 
-        self.main_window.Bind(EVT_COMMAND_GRID_ACTION_OPEN, self.open)
-        self.main_window.Bind(EVT_COMMAND_GRID_ACTION_SAVE, self.save)
+        self.main_window.Bind(self.EVT_CMD_GRID_ACTION_OPEN, self.open)
+        self.main_window.Bind(self.EVT_CMD_GRID_ACTION_SAVE, self.save)
 
-    def _is_aborted(self, cycle, statustext, total_elements=None, freq=1000):
+    def _is_aborted(self, cycle, statustext, total_elements=None, freq=100):
         """Displays progress and returns True if abort
 
         Parameters
@@ -97,17 +96,21 @@ class FileActions(Actions):
 
         """
 
+        if total_elements is None:
+            statustext += _("{nele} elements processed. Press <Esc> to abort.")
+        else:
+            statustext += _("{nele} of {totalele} elements processed. "
+                            "Press <Esc> to abort.")
+
         # Show progress in statusbar each freq (1000) cells
         if cycle % freq == 0:
-            # See if we know how much data comes along
-            if total_elements is None:
-                total_elements_str = ""
-            else:
-                total_elements_str = " of " + str(total_elements)
-
-            statustext = statustext + str(cycle) + total_elements_str + \
-                        " elements processed. Press <Esc> to abort."
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            text = statustext.format(nele=cycle, totalele=total_elements)
+            try:
+                post_command_event(self.main_window, self.StatusBarMsg,
+                                   text=text)
+            except TypeError:
+                # The main window does not exist any more
+                pass
 
             # Now wait for the statusbar update to be written on screen
             wx.Yield()
@@ -141,29 +144,46 @@ class FileActions(Actions):
         self.code_array.safe_mode = True
 
     def leave_safe_mode(self):
-        """Leaves save mode"""
+        """Leaves safe mode"""
 
         self.code_array.safe_mode = False
+
+        # Clear result cache
         self.code_array.result_cache.clear()
-        post_command_event(self.main_window, SafeModeExitMsg)
+
+        # Execute macros
+        self.main_window.actions.execute_macros()
+
+        post_command_event(self.main_window, self.SafeModeExitMsg)
 
     def approve(self, filepath):
         """Sets safe mode if signature missing of invalid"""
 
         if self.validate_signature(filepath):
             self.leave_safe_mode()
-            post_command_event(self.main_window, SafeModeExitMsg)
+            post_command_event(self.main_window, self.SafeModeExitMsg)
 
-            statustext = "Valid signature found. File is trusted."
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            statustext = _("Valid signature found. File is trusted.")
+            post_command_event(self.main_window, self.StatusBarMsg,
+                               text=statustext)
 
         else:
             self.enter_safe_mode()
-            post_command_event(self.main_window, SafeModeEntryMsg)
+            post_command_event(self.main_window, self.SafeModeEntryMsg)
 
-            statustext = "File is not properly signed. Safe mode " + \
-                         "activated. Select File -> Approve to leave safe mode."
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            statustext = _("File is not properly signed. Safe mode "
+                "activated. Select File -> Approve to leave safe mode.")
+            post_command_event(self.main_window, self.StatusBarMsg,
+                               text=statustext)
+
+    def clear_globals_reload_modules(self):
+        """Clears globals and reloads modules"""
+
+        self.code_array.clear_globals()
+        self.code_array.reload_modules()
+
+        # Clear result cache
+        self.code_array.result_cache.clear()
 
     def _get_file_version(self, infile):
         """Returns infile version string."""
@@ -171,7 +191,7 @@ class FileActions(Actions):
         # Determine file version
         for line1 in infile:
             if line1.strip() != "[Pyspread save file version]":
-                raise ValueError, "File format unsupported."
+                raise ValueError(_("File format unsupported."))
             break
 
         for line2 in infile:
@@ -180,8 +200,9 @@ class FileActions(Actions):
     def _abort_open(self, filepath, infile):
         """Aborts file open"""
 
-        statustext = "File loading aborted."
-        post_command_event(self.main_window, StatusBarMsg, text=statustext)
+        statustext = _("File loading aborted.")
+        post_command_event(self.main_window, self.StatusBarMsg,
+                           text=statustext)
 
         infile.close()
 
@@ -192,7 +213,7 @@ class FileActions(Actions):
         """Empties grid and sets shape to shape
 
         Clears all attributes, row heights, column withs and frozen states.
-        Empties undo/redo list and caches.
+        Empties undo/redo list and caches. Empties globals.
 
         Properties
         ----------
@@ -221,6 +242,10 @@ class FileActions(Actions):
         self.code_array.unredo.reset()
         self.code_array.result_cache.clear()
 
+        # Clear globals
+        self.code_array.clear_globals()
+        self.code_array.reload_modules()
+
     def open(self, event):
         """Opens a file that is specified in event.attr
 
@@ -242,8 +267,9 @@ class FileActions(Actions):
             infile = bz2.BZ2File(filepath, "r")
 
         except IOError:
-            statustext = "Error opening file " + filepath + "."
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            statustext = _("Error opening file {}.").format(filepath)
+            post_command_event(self.main_window, self.StatusBarMsg,
+                               text=statustext)
 
             return False
 
@@ -254,23 +280,24 @@ class FileActions(Actions):
         try:
             version = self._get_file_version(infile)
             if version != "0.1":
-                statustext = "File version " + version + \
-                             "unsupported (not 0.1)."
-                post_command_event(self.main_window, StatusBarMsg,
+                statustext = \
+                    _("File version {} unsupported (not 0.1).").format(version)
+                post_command_event(self.main_window, self.StatusBarMsg,
                                    text=statustext)
                 return False
 
         except (IOError, ValueError), errortext:
-            post_command_event(self.main_window, StatusBarMsg, text=errortext)
+            post_command_event(self.main_window, self.StatusBarMsg,
+                               text=errortext)
 
         # Parse content
 
         def parser(*args):
             """Dummy parser. Raises ValueError"""
 
-            raise ValueError, "No section parser present."
+            raise ValueError(_("No section parser present."))
 
-        section_readers = { \
+        section_readers = {
             "[shape]": self.code_array.dict_grid.parse_to_shape,
             "[grid]": self.code_array.dict_grid.parse_to_grid,
             "[attributes]": self.code_array.dict_grid.parse_to_attribute,
@@ -307,8 +334,9 @@ class FileActions(Actions):
                     return False
 
         except IOError:
-            statustext = "Error opening file " + filepath + "."
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            statustext = _("Error opening file {}.").format(filepath)
+            post_command_event(self.main_window, self.StatusBarMsg,
+                               text=statustext)
 
             return False
 
@@ -319,6 +347,9 @@ class FileActions(Actions):
         infile.close()
         self.opening = False
 
+        # Execute macros
+        self.main_window.actions.execute_macros()
+
         # Enable undo again
         self.grid.code_array.unredo.active = False
 
@@ -333,8 +364,14 @@ class FileActions(Actions):
 
         signature = sign(filepath)
         if signature is None:
-            statustext = 'Error signing file. File is not signed.'
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            statustext = _('Error signing file. File is not signed.')
+            try:
+                post_command_event(self.main_window, self.StatusBarMsg,
+                                   text=statustext)
+            except TypeError:
+                # The main window does not exist any more
+                pass
+
             return
 
         signfile = open(filepath + '.sig', 'wb')
@@ -344,17 +381,28 @@ class FileActions(Actions):
         # Statustext differs if a save has occurred
 
         if self.code_array.safe_mode:
-            statustext = 'File saved and signed'
+            statustext = _('File saved and signed')
         else:
-            statustext = 'File signed'
+            statustext = _('File signed')
 
-        post_command_event(self.main_window, StatusBarMsg, text=statustext)
+        try:
+            post_command_event(self.main_window, self.StatusBarMsg,
+                           text=statustext)
+        except TypeError:
+            # The main window does not exist any more
+            pass
 
     def _abort_save(self, filepath, outfile):
         """Aborts file save"""
 
-        statustext = "Save aborted."
-        post_command_event(self.main_window, StatusBarMsg, text=statustext)
+        statustext = _("Save aborted.")
+
+        try:
+            post_command_event(self.main_window, self.StatusBarMsg,
+                               text=statustext)
+        except TypeError:
+            # The main window does not exist any more
+            pass
 
         outfile.close()
         os.remove(filepath)
@@ -379,13 +427,20 @@ class FileActions(Actions):
         self.saving = True
         self.need_abort = False
 
+        io_error_text = _("Error writing to file {}.").format(filepath)
+
         # Save file is compressed
         try:
             outfile = bz2.BZ2File(filepath, "wb")
 
         except IOError:
-            statustext = "Error opening file " + filepath + "."
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            statustext = _("Error opening file {}.").format(filepath)
+            try:
+                post_command_event(self.main_window, self.StatusBarMsg,
+                                   text=statustext)
+            except TypeError:
+                # The main window does not exist any more
+                pass
             return False
 
         # Header
@@ -394,12 +449,17 @@ class FileActions(Actions):
             outfile.write("0.1\n")
 
         except IOError:
-            statustext = "Error writing to file " + filepath + "."
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            try:
+                post_command_event(self.main_window, self.StatusBarMsg,
+                                   text=io_error_text)
+            except TypeError:
+                # The main window does not exist any more
+                pass
+
             return False
 
         # The output generators yield the lines for the outfile
-        output_generators = [ \
+        output_generators = [
             # Grid content
             dict_grid.grid_to_strings(),
             # Cell attributes
@@ -413,7 +473,7 @@ class FileActions(Actions):
         ]
 
         # Options for self._is_aborted
-        abort_options_list = [ \
+        abort_options_list = [
             ["Saving grid... ", len(dict_grid), 100000],
             ["Saving cell attributes... ", len(dict_grid.cell_attributes)],
             ["Saving row heights... ", len(dict_grid.row_heights)],
@@ -429,9 +489,12 @@ class FileActions(Actions):
                     outfile.write(line.encode("utf-8"))
 
                 except IOError:
-                    statustext = "Error writing to file " + filepath + "."
-                    post_command_event(self.main_window, StatusBarMsg,
-                                       text=statustext)
+                    try:
+                        post_command_event(self.main_window, self.StatusBarMsg,
+                                           text=io_error_text)
+                    except TypeError:
+                        # The main window does not exist any more
+                        pass
                     return False
 
                 # Enable abort during long saves
@@ -446,7 +509,12 @@ class FileActions(Actions):
         self.saving = False
 
         # Mark content as unchanged
-        post_command_event(self.main_window, ContentChangedMsg, changed=False)
+        try:
+            post_command_event(self.main_window, self.ContentChangedMsg,
+                               changed=False)
+        except TypeError:
+            # The main window does not exist any more
+            pass
 
         # Sign so that the new file may be retrieved without safe mode
 
@@ -460,7 +528,8 @@ class TableRowActionsMixin(Actions):
         """Sets row height and marks grid as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         tab = self.grid.current_table
 
@@ -475,7 +544,8 @@ class TableRowActionsMixin(Actions):
         """
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         self.code_array.insert(row, no_rows, axis=0)
 
@@ -483,7 +553,8 @@ class TableRowActionsMixin(Actions):
         """Deletes no_rows rows and marks grid as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         self.code_array.delete(row, no_rows, axis=0)
 
@@ -495,7 +566,8 @@ class TableColumnActionsMixin(Actions):
         """Sets column width and marks grid as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         tab = self.grid.current_table
 
@@ -510,7 +582,8 @@ class TableColumnActionsMixin(Actions):
         """
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         self.code_array.insert(col, no_cols, axis=1)
 
@@ -518,7 +591,8 @@ class TableColumnActionsMixin(Actions):
         """Deletes no_cols column and marks grid as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         self.code_array.delete(col, no_cols, axis=1)
 
@@ -534,25 +608,27 @@ class TableTabActionsMixin(Actions):
         """
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         self.code_array.insert(tab, no_tabs, axis=2)
 
         # Update TableChoiceIntCtrl
         shape = self.grid.code_array.shape
-        post_command_event(self.main_window, ResizeGridMsg, shape=shape)
+        post_command_event(self.main_window, self.ResizeGridMsg, shape=shape)
 
     def delete_tabs(self, tab, no_tabs=1):
         """Deletes no_tabs tabs and marks grid as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         self.code_array.delete(tab, no_tabs, axis=2)
 
         # Update TableChoiceIntCtrl
         shape = self.grid.code_array.shape
-        post_command_event(self.main_window, ResizeGridMsg, shape=shape)
+        post_command_event(self.main_window, self.ResizeGridMsg, shape=shape)
 
 
 class TableActions(TableRowActionsMixin, TableColumnActionsMixin,
@@ -583,11 +659,28 @@ class TableActions(TableRowActionsMixin, TableColumnActionsMixin,
 
         event.Skip()
 
+    def _get_full_key(self, key):
+        """Returns full key even if table is omitted"""
+
+        length = len(key)
+
+        if length == 3:
+            return key
+
+        elif length == 2:
+            row, col = key
+            tab = self.grid.current_table
+            return row, col, tab
+
+        else:
+            raise ValueError(_("Key length {}  not in (2, 3)".format(length)))
+
     def _abort_paste(self):
         """Aborts import"""
 
-        statustext = "Paste aborted."
-        post_command_event(self.main_window, StatusBarMsg, text=statustext)
+        statustext = _("Paste aborted.")
+        post_command_event(self.main_window, self.StatusBarMsg,
+                           text=statustext)
 
         self.pasting = False
         self.need_abort = False
@@ -596,21 +689,33 @@ class TableActions(TableRowActionsMixin, TableColumnActionsMixin,
         """Displays overflow message after import in statusbar"""
 
         if row_overflow and col_overflow:
-            overflow_cause = "rows and columns"
+            overflow_cause = _("rows and columns")
         elif row_overflow:
-            overflow_cause = "rows"
+            overflow_cause = _("rows")
         elif col_overflow:
-            overflow_cause = "columns"
+            overflow_cause = _("columns")
         else:
-            raise AssertionError, "Import cell overflow missing"
+            raise AssertionError(_("Import cell overflow missing"))
 
-        statustext = "The imported data did not fit into the grid " + \
-                     overflow_cause + ". It has been truncated. " + \
-                     "Use a larger grid for full import."
-        post_command_event(self.main_window, StatusBarMsg, text=statustext)
+        statustext = _("The imported data did not fit into the grid {cause}. "
+            "It has been truncated. Use a larger grid for full import.").\
+                format(cause=overflow_cause)
+        post_command_event(self.main_window, self.StatusBarMsg,
+                           text=statustext)
 
-    def paste(self, tl_key, data):
-        """Pastes data into grid from top left cell tl_key, marks grid changed
+    def _show_final_paste_message(self, tl_key, no_pasted_cells):
+        """Show actually pasted number of cells"""
+
+        plural = "" if no_pasted_cells == 1 else _("s")
+
+        statustext = _("{ncells} cell{plural} pasted at cell {topleft}").\
+            format(ncells=no_pasted_cells, plural=plural, topleft=tl_key)
+
+        post_command_event(self.main_window, self.StatusBarMsg,
+                           text=statustext)
+
+    def paste_to_current_cell(self, tl_key, data):
+        """Pastes data into grid from top left cell tl_key
 
         Parameters
         ----------
@@ -622,31 +727,23 @@ class TableActions(TableRowActionsMixin, TableColumnActionsMixin,
 
         """
 
-        # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
-
         self.pasting = True
 
-        grid_rows, grid_cols, _ = self.grid.code_array.shape
+        grid_rows, grid_cols, __ = self.grid.code_array.shape
 
         self.need_abort = False
 
-        try:
-            tl_row, tl_col, tl_tab = tl_key
-
-        except ValueError:
-            tl_row, tl_col = tl_key
-            tl_tab = self.grid.current_table
+        tl_row, tl_col, tl_tab = self._get_full_key(tl_key)
 
         row_overflow = False
         col_overflow = False
 
         no_pasted_cells = 0
 
-        for src_row, col_data in enumerate(data):
+        for src_row, row_data in enumerate(data):
             target_row = tl_row + src_row
 
-            if self.grid.actions._is_aborted(src_row, "Pasting cells... "):
+            if self.grid.actions._is_aborted(src_row, _("Pasting cells... ")):
                 self._abort_paste()
                 return False
 
@@ -655,46 +752,113 @@ class TableActions(TableRowActionsMixin, TableColumnActionsMixin,
                 row_overflow = True
                 break
 
-            for src_col, cell_data in enumerate(col_data):
+            for src_col, cell_data in enumerate(row_data):
                 target_col = tl_col + src_col
 
                 if target_col >= grid_cols:
                     col_overflow = True
                     break
 
-                key = target_row, target_col, tl_tab
+                if cell_data is not None:
+                    # Is only None if pasting into selection
+                    key = target_row, target_col, tl_tab
 
-                try:
-                    self.grid.code_array[key] = cell_data
-                    no_pasted_cells += 1
-                except KeyError:
-                    pass
+                    try:
+                        # Set cell but do not mark unredo
+                        # before pasting is finished
+
+                        self.grid.code_array.__setitem__(key, cell_data,
+                                                         mark_unredo=False)
+                        no_pasted_cells += 1
+                    except KeyError:
+                        pass
 
         if row_overflow or col_overflow:
             self._show_final_overflow_message(row_overflow, col_overflow)
 
         else:
-            # Show actually pasted number of cells
+            self._show_final_paste_message(tl_key, no_pasted_cells)
 
-            plural = "s" if no_pasted_cells == 1 else ""
-
-            statustext = str(no_pasted_cells) + " cell" + plural + \
-                         " pasted at cell " + str(tl_key)
-
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+        if no_pasted_cells:
+            # If cells have been pasted mark unredo operation
+            self.grid.code_array.unredo.mark()
 
         self.pasting = False
 
+    def selection_paste_data_gen(self, selection, data):
+        """Generator that yields data for selection paste"""
+
+        (bb_top, bb_left), (bb_bottom, bb_right) = \
+            selection.get_grid_bbox(self.grid.code_array.shape)
+        bbox_height = bb_bottom - bb_top + 1
+        bbox_width = bb_right - bb_left + 1
+
+        for row, row_data in enumerate(itertools.cycle(data)):
+            # Break if row is not in selection bbox
+            if row >= bbox_height:
+                break
+
+            # Duplicate row data if selection is wider than row data
+            duplicated_row_data = row_data * (bbox_width // len(row_data) + 1)
+            duplicated_row_data = duplicated_row_data[:bbox_width]
+
+            for col in xrange(len(duplicated_row_data)):
+                if (bb_top, bb_left + col) not in selection:
+                    duplicated_row_data[col] = None
+
+            yield duplicated_row_data
+
+    def paste_to_selection(self, selection, data):
+        """Pastes data into grid selection"""
+
+        (bb_top, bb_left), (bb_bottom, bb_right) = \
+            selection.get_grid_bbox(self.grid.code_array.shape)
+        adjusted_data = self.selection_paste_data_gen(selection, data)
+        self.paste_to_current_cell((bb_top, bb_left), adjusted_data)
+
+    def paste(self, tl_key, data):
+        """Pastes data into grid, marks grid changed
+
+        If no selection is present, data is pasted starting with current cell
+        If a selection is present, data is pasted fully if the selection is
+        smaller. If the selection is larger then data is duplicated.
+
+        Parameters
+        ----------
+
+        ul_key: Tuple
+        \key of top left cell of paste area
+        data: iterable of iterables where inner iterable returns string
+        \tThe outer iterable represents rows
+
+        """
+
+        # Get selection bounding box
+
+        selection = self.get_selection()
+
+        # Mark content as changed
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
+
+        if selection:
+            # There is a selection.  Paste into it
+            self.paste_to_selection(selection, data)
+        else:
+            # There is no selection.  Paste from top left cell.
+            self.paste_to_current_cell(tl_key, data)
+
     def change_grid_shape(self, shape):
         """Grid shape change event handler, marks content as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         self.grid.code_array.shape = shape
 
         # Update TableChoiceIntCtrl
-        post_command_event(self.main_window, ResizeGridMsg, shape=shape)
+        post_command_event(self.main_window, self.ResizeGridMsg, shape=shape)
 
 
 class UnRedoActions(Actions):
@@ -704,7 +868,8 @@ class UnRedoActions(Actions):
         """Calls undo in model.code_array.unredo, marks content as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         self.grid.code_array.unredo.undo()
 
@@ -712,7 +877,8 @@ class UnRedoActions(Actions):
         """Calls redo in model.code_array.unredo, marks content as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         self.grid.code_array.unredo.redo()
 
@@ -727,8 +893,8 @@ class GridActions(Actions):
 
         self.prev_rowcol = []  # Last mouse over cell
 
-        self.main_window.Bind(EVT_COMMAND_GRID_ACTION_NEW, self.new)
-        self.main_window.Bind(EVT_COMMAND_GRID_ACTION_TABLE_SWITCH,
+        self.main_window.Bind(self.EVT_CMD_GRID_ACTION_NEW, self.new)
+        self.main_window.Bind(self.EVT_CMD_GRID_ACTION_TABLE_SWITCH,
                               self.switch_to_table)
 
     def new(self, event):
@@ -799,9 +965,10 @@ class GridActions(Actions):
         self.grid.ForceRefresh()
 
         if status:
-            statustext = u"Zoomed to {0:.2f}.".format(zoom)
+            statustext = _(u"Zoomed to {0:.2f}.").format(zoom)
 
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            post_command_event(self.main_window, self.StatusBarMsg,
+                               text=statustext)
 
     def zoom_in(self):
         """Zooms in by zoom factor"""
@@ -831,12 +998,15 @@ class GridActions(Actions):
         if (row, col) != self.prev_rowcol and row >= 0 and col >= 0:
             self.prev_rowcol[:] = [row, col]
 
-            hinttext = self.grid.GetTable().GetSource(row, col, tab)
+            max_result_length = int(config["max_result_length"])
+            table = self.grid.GetTable()
+            hinttext = table.GetSource(row, col, tab)[:max_result_length]
 
             if hinttext is None:
                 hinttext = ''
 
-            post_command_event(self.main_window, StatusBarMsg, text=hinttext)
+            post_command_event(self.main_window, self.StatusBarMsg,
+                               text=hinttext)
 
     def get_visible_area(self):
         """Returns visible area
@@ -880,20 +1050,21 @@ class GridActions(Actions):
 
         newtable = event.newtable
 
-        no_tabs = self.grid.code_array.shape[2]
+        no_tabs = self.grid.code_array.shape[2] - 1
 
         if 0 <= newtable <= no_tabs:
             self.grid.current_table = newtable
-            self.main_window.table_choice.SetMax(newtable+1)
+            self.main_window.table_choice.SetMax(newtable + 1)
             self.main_window.table_choice.SetValue(newtable)
 
             # Reset row heights and column widths by zooming
 
             self.zoom()
 
-            statustext = u"Switched to table " + str(newtable)
+            statustext = _("Switched to table {}.").format(newtable)
 
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            post_command_event(self.main_window, self.StatusBarMsg,
+                               text=statustext)
 
     def get_cursor(self):
         """Returns current grid cursor cell (row, col, tab)"""
@@ -916,8 +1087,8 @@ class GridActions(Actions):
             row, col, tab = value
 
             if tab != self.cursor[2]:
-                post_command_event(self.main_window, GridActionTableSwitchMsg,
-                                   newtable=tab)
+                post_command_event(self.main_window,
+                                   self.GridActionTableSwitchMsg, newtable=tab)
         else:
             row, col = value
 
@@ -979,7 +1150,7 @@ class SelectionActions(Actions):
         elif row_slc.stop is None and col_slc.stop is None:
             # A block is selected:
             self.grid.SelectBlock(row_slc.start, col_slc.start,
-                                  row_slc.stop-1, col_slc.stop-1)
+                                  row_slc.stop - 1, col_slc.stop - 1)
         else:
             for row in xrange(row_slc.start, row_slc.stop, row_slc.step):
                 for col in xrange(col_slc.start, col_slc.stop, col_slc.step):
@@ -989,11 +1160,12 @@ class SelectionActions(Actions):
         """Deletes selected cells, marks content as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         selection = self.get_selection()
 
-        del_keys = [key for key in self.grid.code_array \
+        del_keys = [key for key in self.grid.code_array
                         if key[:2] in selection]
 
         for key in del_keys:
@@ -1060,7 +1232,8 @@ class FindActions(Actions):
         """
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         old_code = self.grid.code_array(findpos)
         new_code = old_code.replace(find_string, replace_string)
@@ -1068,10 +1241,11 @@ class FindActions(Actions):
         self.grid.code_array[findpos] = new_code
         self.grid.actions.cursor = findpos
 
-        statustext = "Replaced '" + old_code + "' with '" + new_code + \
-                     "' in cell " + unicode(list(findpos)) + "."
+        statustext = _("Replaced {} with {} in cell {}.").format(\
+                        old_code, new_code, findpos)
 
-        post_command_event(self.main_window, StatusBarMsg, text=statustext)
+        post_command_event(self.main_window, self.StatusBarMsg,
+                           text=statustext)
 
 
 class AllGridActions(FileActions, TableActions, UnRedoActions,
