@@ -21,12 +21,14 @@
 
 import wx
 
+import src.lib.i18n as i18n
 from src.lib.selection import Selection
-
 from src.actions._main_window_actions import Actions
+from src.gui._events import post_command_event
+
+#use ugettext instead of getttext to avoid unicode errors
+_ = i18n.language.ugettext
 
-from src.gui._events import post_command_event, EntryLineMsg, ContentChangedMsg
-from src.gui._events import StatusBarMsg
 
 """
 _grid_cell_actions.py
@@ -57,7 +59,7 @@ class CellActions(Actions):
 
         if not (old_code is None and not code) and code != old_code:
             # Mark content as changed
-            post_command_event(self.main_window, ContentChangedMsg,
+            post_command_event(self.main_window, self.ContentChangedMsg,
                                changed=True)
 
         # Set cell code
@@ -111,12 +113,14 @@ class CellActions(Actions):
                 return u"+" + str(diff_key_ele)
 
             else:
-                raise ValueError, str(diff_key_ele) + "seems to be no Integer"
+                errmsg = _("{} seems to be no Integer").format(diff_key_ele)
+                raise ValueError(errmsg)
 
         key_strings = []
 
         for magic, cursor_ele, ref_key_ele in zip(magics, cursor, ref_key):
-            key_strings.append(magic + get_rel_key_ele(cursor_ele, ref_key_ele))
+            key_strings.append(magic +
+                               get_rel_key_ele(cursor_ele, ref_key_ele))
 
         key_string = u", ".join(key_strings)
 
@@ -145,7 +149,7 @@ class CellActions(Actions):
             code = self._get_relative_reference(key, ref_key)
 
         else:
-            raise ValueError, 'ref_type has to be "absolute" or "relative".'
+            raise ValueError(_('ref_type has to be "absolute" or "relative".'))
 
         old_code = self.grid.code_array(key)
 
@@ -153,12 +157,15 @@ class CellActions(Actions):
             old_code = u""
 
         if "S" in old_code and old_code[-1] == "]":
-            old_code_left, _ = old_code.rsplit("S", 1)
+            old_code_left, __ = old_code.rsplit("S", 1)
             new_code = old_code_left + code
         else:
             new_code = old_code + code
 
-        post_command_event(self.grid.main_window, EntryLineMsg, text=new_code)
+        post_command_event(self.grid.main_window, self.EntryLineMsg,
+                           text=new_code)
+
+        return new_code  # For unit tests
 
     def _set_cell_attr(self, selection, table, attr):
         """Sets cell attr for key cell and mark grid content as changed
@@ -173,15 +180,16 @@ class CellActions(Actions):
         \t"bgcolor", "textfont",
         \t"pointsize", "fontweight", "fontstyle", "textcolor", "underline",
         \t"strikethrough", "angle", "column-width", "row-height",
-        \t"vertical_align", "justification", "frozen"]
+        \t"vertical_align", "justification", "frozen", "merge_area"]
 
         """
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         if selection is not None:
-            self.code_array.cell_attributes.undoable_append( \
+            self.code_array.cell_attributes.undoable_append(
                                             (selection, table, attr))
 
     def set_attr(self, attr, value, selection=None):
@@ -216,6 +224,9 @@ class CellActions(Actions):
         """
 
         selection = self.grid.selection
+        if not selection:
+            selection.cells.append(self.grid.actions.cursor[:2])
+
         # determine selection for core cells and selection for border cells
         # Then apply according to inner and outer
         # A cell is inner iif it is not at the edge of the selection bbox
@@ -239,27 +250,27 @@ class CellActions(Actions):
             # Adjust selection so that only bounding box edge is in selection
             bbox_tl, bbox_lr = selection.get_bbox()
             if "top" in borders:
-                adj_selection = Selection([bbox_tl], [(bbox_tl[0], bbox_lr[1])],
+                adj_selection = Selection([bbox_tl],
+                                          [(bbox_tl[0], bbox_lr[1])],
                                           [], [], []) + (-1, 0)
                 self.set_attr(attr + "_bottom", value, adj_selection)
 
             if "bottom" in borders:
-                adj_selection = Selection([(bbox_lr[0], bbox_tl[1])], [bbox_lr],
-                                          [], [], [])
+                adj_selection = Selection([(bbox_lr[0], bbox_tl[1])],
+                                          [bbox_lr], [], [], [])
                 self.set_attr(attr + "_bottom", value, adj_selection)
 
             if "left" in borders:
-                adj_selection = Selection([bbox_tl], [(bbox_lr[0], bbox_tl[1])],
+                adj_selection = Selection([bbox_tl],
+                                          [(bbox_lr[0], bbox_tl[1])],
                                           [], [], []) + (0, -1)
                 self.set_attr(attr + "_right", value, adj_selection)
 
             if "right" in borders:
-                adj_selection = Selection([(bbox_tl[0], bbox_lr[1])], [bbox_lr],
-                                          [], [], [])
+                adj_selection = Selection([(bbox_tl[0], bbox_lr[1])],
+                                          [bbox_lr], [], [], [])
                 self.set_attr(attr + "_right", value, adj_selection)
 
-
-
     def toggle_attr(self, attr):
         """Toggles an attribute attr for current selection"""
 
@@ -286,8 +297,9 @@ class CellActions(Actions):
         # Selections are not supported
 
         if self.grid.selection:
-            statustext = "Freezing selections is not supported."
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            statustext = _("Freezing selections is not supported.")
+            post_command_event(self.main_window, self.StatusBarMsg,
+                               text=statustext)
 
         cursor = self.grid.actions.cursor
 
@@ -310,7 +322,55 @@ class CellActions(Actions):
         selection = Selection([], [], [], [], [cursor[:2]])
         self.set_attr("frozen", not frozen, selection=selection)
 
-    attr_toggle_values = { \
+    def unmerge(self, unmerge_area, tab):
+        """Unmerges all cells in unmerge_area"""
+
+        top, left, bottom, right = unmerge_area
+        selection = Selection([(top, left)], [(bottom, right)], [], [], [])
+        attr = {"merge_area": None}
+
+        self.grid.actions._set_cell_attr(selection, tab, attr)
+
+    def merge(self, merge_area, tab):
+        """Merges top left cell with all cells until bottom_right"""
+
+        top, left, bottom, right = merge_area
+        selection = Selection([(top, left)], [(bottom, right)], [], [], [])
+        attr = {"merge_area": merge_area}
+
+        self.grid.actions._set_cell_attr(selection, tab, attr)
+
+    def merge_selected_cells(self, selection):
+        """Merges or unmerges cells that are in the selection bounding box
+
+        Parameters
+        ----------
+        selection: Selection object
+        \tSelection for which attr toggle shall be returned
+
+        """
+
+        tab = self.grid.current_table
+
+        # Get the selection bounding box
+        bbox = selection.get_bbox()
+        if bbox is None:
+            row, col, tab = self.grid.actions.cursor
+            (bb_top, bb_left), (bb_bottom, bb_right) = (row, col), (row, col)
+        else:
+            (bb_top, bb_left), (bb_bottom, bb_right) = bbox
+        merge_area = bb_top, bb_left, bb_bottom, bb_right
+
+        # Check if top-left cell is already merged
+        cell_attributes = self.grid.code_array.cell_attributes
+        tl_merge_area = cell_attributes[(bb_top, bb_left, tab)]["merge_area"]
+
+        if tl_merge_area is None:
+            self.merge(merge_area, tab)
+        else:
+            self.unmerge(tl_merge_area, tab)
+
+    attr_toggle_values = {
         "fontweight": [wx.NORMAL, wx.BOLD],
         "fontstyle": [wx.NORMAL, wx.ITALIC],
         "underline": [False, True],
@@ -403,7 +463,8 @@ class CellActions(Actions):
                 skey = attr_selection.cells[0]
                 if skey in selection:
                     key = tuple(list(skey) + [tab])
-                    result = self.grid.code_array._eval_cell(key)
+                    code = self.grid.code_array(key)
+                    result = self.grid.code_array._eval_cell(key, code)
                     self.grid.code_array.frozen_cache[repr(key)] = result
 
         cell_attributes._attr_cache.clear()
\ No newline at end of file
