@@ -40,22 +40,27 @@ Provides:
 
 """
 
+import base64
+import bz2
 import os
 
 import wx
 import wx.html
 
+import lib.i18n as i18n
 from src.sysvars import get_help_path
 
 from src.config import config
 from src.lib.__csv import CsvInterface, TxtGenerator
 from src.gui._printout import PrintCanvas, Printout
 
-from src.gui._events import post_command_event, SafeModeEntryMsg
-from src.gui._events import StatusBarMsg, ContentChangedMsg
+from src.gui._events import post_command_event, EventMixin
+
+#use ugettext instead of getttext to avoid unicode errors
+_ = i18n.language.ugettext
 
 
-class Actions(object):
+class Actions(EventMixin):
     """Actions base class"""
 
     def __init__(self, grid):
@@ -70,18 +75,20 @@ class ExchangeActions(Actions):
     def _import_csv(self, path):
         """CSV import workflow"""
 
+        # If path is not set, do nothing
+        if not path:
+            return
+
         # Get csv info
 
         try:
             dialect, has_header, digest_types = \
                 self.main_window.interfaces.get_csv_import_info(path)
 
-        except TypeError:
-            return
-
         except IOError:
-            statustext = "Error opening file " + path + "."
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            statustext = _("Error opening file {}.").format(path)
+            post_command_event(self.main_window, self.StatusBarMsg,
+                               text=statustext)
             return
 
         return CsvInterface(self.main_window,
@@ -106,7 +113,8 @@ class ExchangeActions(Actions):
         """
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         if filterindex == 0:
             # CSV import option choice
@@ -115,8 +123,8 @@ class ExchangeActions(Actions):
             # TXT import option choice
             return self._import_txt(filepath)
         else:
-            msg = "Unknown import choice" + str(filterindex)
-            short_msg = 'Error reading CSV file'
+            msg = _("Unknown import choice {}.").format(filterindex)
+            short_msg = _('Error reading CSV file')
 
             self.main_window.interfaces.display_warning(msg, short_msg)
 
@@ -144,9 +152,9 @@ class ExchangeActions(Actions):
             csv_interface.write(data)
 
         except IOError, err:
-            msg = 'The file "' + filepath + '" could not be fully written ' + \
-                  '\n \nError message:\n' + str(err)
-            short_msg = 'Error writing CSV file'
+            msg = _("The file {} could not be fully written\n \n"
+                    "Error message:\n{}").format(filepath, err)
+            short_msg = _('Error writing CSV file')
             self.main_window.interfaces.display_warning(msg, short_msg)
 
     def export_file(self, filepath, filterindex, data):
@@ -170,10 +178,10 @@ class PrintActions(Actions):
         preview = wx.PrintPreview(printout, printout2, print_data)
 
         if not preview.Ok():
-            print "Houston, we have a problem...\n"
+            print "Printout preview failed.\n"
             return
 
-        pfrm = wx.PreviewFrame(preview, self.main_window, "Print preview")
+        pfrm = wx.PreviewFrame(preview, self.main_window, _("Print preview"))
 
         pfrm.Initialize()
         pfrm.SetPosition(self.main_window.GetPosition())
@@ -321,12 +329,75 @@ class ClipboardActions(Actions):
         return unicode(self.grid.code_array[row, col, tab])
 
     def copy_result(self, selection):
-        """Returns result strings from selection in a tab separated string"""
+        """Returns result
 
+        If selection consists of one cell only and result is a bitmap then
+        the bitmap is returned.
+        Otherwise the method returns string representations of the result
+        for the given selection in a tab separated string.
+
+        """
+
+        bbox = selection.get_bbox()
+
+        if not bbox:
+            # There is no selection
+            bb_top, bb_left = self.grid.actions.cursor[:2]
+            bb_bottom, bb_right = bb_top, bb_left
+        else:
+            # Thereis a selection
+            (bb_top, bb_left), (bb_bottom, bb_right) = bbox
+
+        if bb_top == bb_bottom and bb_left == bb_right:
+            # We have  a single selection
+
+            tab = self.grid.current_table
+            result = self.grid.code_array[bb_top, bb_left, tab]
+
+            if type(result) is wx._gdi.Bitmap:
+                # the result is a bitmap
+                return result
+
+        # So we have result strings to be returned
         getter = self._get_result_string
 
         return self.copy(selection, getter=getter)
 
+    def bmp2code(self, key, bmp):
+        """Pastes bitmap into single cell"""
+
+        assert type(bmp) is wx._gdi.Bitmap
+
+        row, col, tab = key
+        width = self.grid.GetColSize(col) / self.grid.grid_renderer.zoom
+        height = self.grid.GetRowSize(row) / \
+                    self.grid.grid_renderer.zoom
+
+        code_str = ""
+        code_template = \
+            "wx.BitmapFromImage(wx.ImageFromData(" + \
+            "{width}, {height}, bz2.decompress(base64.b64decode('{data}'))))"
+
+        # Fix length limitation of wx.TextCtrl
+        # Therefore scale down image until text length < max_textctrl_length
+
+        while not code_str or \
+              len(code_str) > int(config["max_textctrl_length"]):
+
+            img = bmp.ConvertToImage()
+
+            img.Rescale(width, height, quality=wx.IMAGE_QUALITY_HIGH)
+
+            data = base64.b64encode(bz2.compress(img.GetData(), 9))
+
+            code_str = code_template.format(width=img.GetWidth(),
+                                            height=img.GetHeight(), data=data)
+
+            width *= 0.9
+            height *= 0.9
+
+        return code_str
+
     def paste(self, key, data):
         """Pastes data into grid
 
@@ -335,11 +406,16 @@ class ClipboardActions(Actions):
 
         key: 2-Tuple of Integer
         \tTop left cell
-        data: String
+        data: String or wx.Bitmap
         \tTab separated string of paste data
+        \tor paste data image
         """
 
-        data_gen = (line.split("\t") for line in data.split("\n"))
+        if type(data) is wx._gdi.Bitmap:
+            code_str = self.bmp2code(key, data)
+            data_gen = [[code_str]]
+        else:
+            data_gen = (line.split("\t") for line in data.split("\n"))
 
         self.grid.actions.paste(key[:2], data_gen)
 
@@ -358,7 +434,8 @@ class MacroActions(Actions):
         """Executes macros and marks grid as changed"""
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         self.grid.code_array.execute_macros()
 
@@ -376,13 +453,15 @@ class MacroActions(Actions):
             macro_infile = open(filepath, "r")
 
         except IOError:
-            statustext = "Error opening file " + filepath + "."
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            statustext = _("Error opening file {}.").format(filepath)
+            post_command_event(self.main_window, self.StatusBarMsg,
+                               text=statustext)
 
             return False
 
         # Mark content as changed
-        post_command_event(self.main_window, ContentChangedMsg, changed=True)
+        post_command_event(self.main_window, self.ContentChangedMsg,
+                           changed=True)
 
         macrocode = macro_infile.read()
         macro_infile.close()
@@ -390,7 +469,7 @@ class MacroActions(Actions):
         self.grid.code_array.macros += "\n" + macrocode.strip("\n")
 
         self.main_window.grid.actions.enter_safe_mode()
-        post_command_event(self.main_window, SafeModeEntryMsg)
+        post_command_event(self.main_window, self.SafeModeEntryMsg)
 
     def save_macros(self, filepath, macros):
         """Saves macros to file
@@ -478,5 +557,4 @@ class AllMainWindowActions(ExchangeActions, PrintActions,
                            ClipboardActions, MacroActions, HelpActions):
     """All main window actions as a bundle"""
 
-    pass
-
+    pass
\ No newline at end of file
