@@ -31,7 +31,7 @@ import wx
 
 from sysvars import get_color, get_font_string
 
-VERSION = "0.2.1"
+VERSION = "0.2.3"
 
 
 class DefaultConfig(object):
@@ -97,8 +97,6 @@ class DefaultConfig(object):
         # --------------
 
         self.gpg_key_uid = repr('')
-        self.gpg_key_passphrase = repr('')
-        self.gpg_key_passphrase_isstored = repr(True)
 
         # CSV parameters for import and export
         # ------------------------------------
@@ -106,6 +104,9 @@ class DefaultConfig(object):
         # Number of bytes for the sniffer (should be larger than 1st+2nd line)
         self.sniff_size = "65536"
 
+        # Maximum number of characters in wx.TextCtrl
+        self.max_textctrl_length = "65534"
+
 
 class Config(object):
     """Configuration class for the application pyspread"""
@@ -157,11 +158,12 @@ class Config(object):
         """Saves configuration file"""
 
         for key in self.defaults.__dict__:
-            if not key == "gpg_key_passphrase" or \
-               self["gpg_key_passphrase_isstored"]:
-                self.cfg_file.Write(key, getattr(self.data, key))
-            else:
-                self.cfg_file.Write(key, "''")
+            data = getattr(self.data, key)
 
+            if key == "gpg_key_passphrase" and \
+               not self["gpg_key_passphrase_isstored"]:
+                data = "''"
+
+            self.cfg_file.Write(key, data)
 
 config = Config()
\ No newline at end of file
