@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright 2011 Martin Manns
+# Copyright Martin Manns
 # Distributed under the terms of the GNU General Public License
 
 # --------------------------------------------------------------------
@@ -43,6 +43,7 @@ Provides:
 import cStringIO
 import csv
 import os
+import string
 import types
 
 import wx
@@ -51,13 +52,18 @@ from wx.lib.wordwrap import wordwrap
 import wx.lib.masked
 import wx.stc as stc
 
-from src.config import config
+import src.lib.i18n as i18n
+from src.config import config, VERSION
 from src.sysvars import get_program_path
 from src.gui._widgets import PythonSTC
-from src.gui._events import *
+from src.gui._events import post_command_event
+from src.gui._events import MainWindowEventMixin, GridEventMixin
 from src.lib.__csv import Digest, sniff, get_first_line
 from src.lib.__csv import csv_digest_gen, cell_key_val_gen
 
+#use ugettext instead of getttext to avoid unicode errors
+_ = i18n.language.ugettext
+
 
 class IntValidator(wx.PyValidator):
     """IntTextCtrl input validation class"""
@@ -74,7 +80,7 @@ class IntValidator(wx.PyValidator):
             return True
 
     def Clone(self):
-        return Validator()
+        return wx.Validator()
 
     def Validate(self, win):
         """Returns True if Value in digits, False otherwise"""
@@ -142,8 +148,8 @@ class ChoiceRenderer(wx.grid.PyGridCellRenderer):
         button_y = rect.y
         button_width = self.iconwidth
         button_height = rect.height
-        render.DrawComboBoxDropButton(grid, dc, \
-            (button_x, button_y, button_width, button_height), \
+        render.DrawComboBoxDropButton(grid, dc,
+            (button_x, button_y, button_width, button_height),
             wx.CONTROL_CURRENT)
 
 
@@ -163,62 +169,63 @@ class CsvParameterWidgets(object):
     """
 
     csv_params = \
-        [["dialects", types.TupleType, "Dialect", \
-            "To make it easier to specify the format of input and output " \
-            "records, specific formatting parameters are grouped together " \
-            "into dialects.\n'excel': Defines the usual properties of an " \
-            "Excel-generated CSV file.\n'sniffer': Deduces the format of a " \
-            "CSV file\n'excel-tab': Defines the usual " \
-            "properties of an Excel-generated TAB-delimited file."], \
-         ["delimiter", types.StringType, "Delimiter", \
-            "A one-character string used to separate fields."], \
-         ["doublequote", types.BooleanType, "Doublequote", \
-            "Controls how instances of quotechar appearing inside a " \
-            "field should be themselves be quoted. When True, the character " \
-            "is doubled. When False, the escapechar is used as a prefix to " \
-            "the quotechar."], \
-         ["escapechar", types.StringType, "Escape character", \
-            "A one-character string used by " \
-            "the writer to escape the delimiter if quoting is set to " \
-            "QUOTE_NONE and the quotechar if doublequote is False. On " \
-            "reading, the escapechar removes any special meaning from the " \
-            "following character."], \
-         ["quotechar", types.StringType, "Quote character", \
-            "A one-character string used to quote fields containing special " \
-            "characters, such as the delimiter or quotechar, or which " \
-            "contain new-line characters."], \
-         ["quoting", types.TupleType, "Quoting style", \
-            "Controls when quotes should be recognised."], \
-         ["self.has_header", types.BooleanType, "Header present", \
-            "Analyze the CSV file and treat the first row as strings if it " \
-            "appears to be a series of column headers."], \
-         ["skipinitialspace", types.BooleanType, "Skip initial space", \
-            "When True, whitespace immediately following the delimiter is " \
-            "ignored."], \
+        [["dialects", types.TupleType, _("Dialect"),
+          _("To make it easier to specify the format of input and output "
+            "records, specific formatting parameters are grouped together "
+            "into dialects.\n'excel': Defines the usual properties of an "
+            "Excel-generated CSV file.\n'sniffer': Deduces the format of a "
+            "CSV file\n'excel-tab': Defines the usual "
+            "properties of an Excel-generated TAB-delimited file.")],
+         ["delimiter", types.StringType, _("Delimiter"),
+          _("A one-character string used to separate fields.")],
+         ["doublequote", types.BooleanType, _("Doublequote"),
+          _("Controls how instances of quotechar appearing inside a "
+            "field should be themselves be quoted. When True, the character "
+            "is doubled. When False, the escapechar is used as a prefix to "
+            "the quotechar.")],
+         ["escapechar", types.StringType, _("Escape character"),
+          _("A one-character string used by "
+            "the writer to escape the delimiter if quoting is set to "
+            "QUOTE_NONE and the quotechar if doublequote is False. On "
+            "reading, the escapechar removes any special meaning from the "
+            "following character.")],
+         ["quotechar", types.StringType, _("Quote character"),
+          _("A one-character string used to quote fields containing special "
+            "characters, such as the delimiter or quotechar, or which "
+            "contain new-line characters.")],
+         ["quoting", types.IntType, _("Quoting style"),
+          _("Controls when quotes should be recognised.")],
+         ["self.has_header", types.BooleanType, _("Header present"),
+          _("Analyze the CSV file and treat the first row as strings if it "
+            "appears to be a series of column headers.")],
+         ["skipinitialspace", types.BooleanType, _("Skip initial space"),
+          _("When True, whitespace immediately following the delimiter is "
+            "ignored.")],
         ]
 
-    type2widget = { \
-        types.StringType: wx.TextCtrl, \
-        types.BooleanType: wx.CheckBox, \
-        types.TupleType: wx.Choice, \
+    type2widget = {
+        types.StringType: wx.TextCtrl,
+        types.BooleanType: wx.CheckBox,
+        types.TupleType: wx.Choice,
+        types.IntType: wx.Choice,
     }
 
     # All tuple types from csv_params have choice boxes
-    choices = { \
-        'dialects': tuple(["sniffer"] + csv.list_dialects() + ["user"]), \
-        'quoting': ("QUOTE_ALL", "QUOTE_MINIMAL", \
-                    "QUOTE_NONNUMERIC", "QUOTE_NONE"), \
+    choices = {
+        'dialects': tuple(["sniffer"] + csv.list_dialects() + ["user"]),
+        'quoting': ("QUOTE_ALL", "QUOTE_MINIMAL",
+                    "QUOTE_NONNUMERIC", "QUOTE_NONE"),
     }
 
-    widget_handlers = { \
-        'dialects': "OnDialectChoice", \
-        'quoting': "OnWidget", \
-        'delimiter': "OnWidget", \
-        'escapechar': "OnWidget", \
-        'quotechar': "OnWidget", \
-        'doublequote': "OnWidget", \
-        'self.has_header': "OnWidget", \
-        'skipinitialspace': "OnWidget", \
+    widget_handlers = {
+        'dialects': "OnDialectChoice",
+        'quoting': "OnWidget",
+        'delimiter': "OnWidget",
+        'escapechar': "OnWidget",
+        'quotechar': "OnWidget",
+        'doublequote': "OnWidget",
+        'self.has_header': "OnWidget",
+        'skipinitialspace': "OnWidget",
     }
 
     def __init__(self, parent, csvfilepath):
@@ -255,12 +262,13 @@ class CsvParameterWidgets(object):
                 widget.SetValue = widget.Select
 
             # Bind event handler to widget
-            if ptype is types.StringType:
+            if isinstance(ptype, types.StringType):
                 event_type = wx.EVT_TEXT
-            elif ptype is types.BooleanType:
+            elif isinstance(ptype, types.BooleanType):
                 event_type = wx.EVT_CHECKBOX
-            elif ptype is types.TupleType:
+            else:
                 event_type = wx.EVT_CHOICE
+
             handler = getattr(self, self.widget_handlers[pname])
             self.parent.Bind(event_type, handler, widget)
 
@@ -310,7 +318,7 @@ class CsvParameterWidgets(object):
 
             widget = self._widget_from_p(pname, ptype)
 
-            if ptype is types.TupleType:
+            if isinstance(ptype, types.TupleType):
                 ptype = types.ObjectType
 
             digest = Digest(acceptable_types=[ptype])
@@ -320,7 +328,6 @@ class CsvParameterWidgets(object):
                     widget.SetValue(digest(self.has_header))
             else:
                 value = getattr(dialect, pname)
-
                 widget.SetValue(digest(value))
 
     def _widget_from_p(self, pname, ptype):
@@ -373,7 +380,7 @@ class CsvParameterWidgets(object):
                 choice = self.choices['quoting'][widget.GetSelection()]
                 parameters[pname] = getattr(csv, choice)
             else:
-                raise TypeError, ptype + "unknown"
+                raise TypeError(_("{} unknown.").format(ptype))
 
         has_header = parameters.pop("self.has_header")
 
@@ -381,11 +388,12 @@ class CsvParameterWidgets(object):
             csv.register_dialect('user', **parameters)
 
         except TypeError, err:
-            msg = 'The dialect is invalid. \n \nError message:\n' + str(err)
+            msg = _("The dialect is invalid. \n "
+                    "\nError message:\n{}").format(err)
             dlg = wx.MessageDialog(self.parent, msg, style=wx.ID_CANCEL)
             dlg.ShowModal()
             dlg.Destroy()
-            raise TypeError, err
+            raise TypeError(err)
 
         return csv.get_dialect('user'), has_header
 
@@ -395,13 +403,13 @@ class CSVPreviewGrid(wx.grid.Grid):
 
     shape = [10, 10]
 
-    digest_types = { \
-        'String': types.StringType, \
-        'Unicode': types.UnicodeType, \
-        'Integer': types.IntType, \
-        'Float': types.FloatType, \
-        'Boolean': types.BooleanType, \
-        'Object': types.ObjectType, \
+    digest_types = {
+        'String': types.StringType,
+        'Unicode': types.UnicodeType,
+        'Integer': types.IntType,
+        'Float': types.FloatType,
+        'Boolean': types.BooleanType,
+        'Object': types.ObjectType,
     }
 
     # Only add date and time if dateutil is installed
@@ -494,7 +502,7 @@ class CSVPreviewGrid(wx.grid.Grid):
         # Add Choices
         for col in xrange(self.shape[1]):
             choice_renderer = ChoiceRenderer(self)
-            choice_editor = wx.grid.GridCellChoiceEditor( \
+            choice_editor = wx.grid.GridCellChoiceEditor(
                                 self.digest_types.keys(), False)
             self.SetCellRenderer(has_header, col, choice_renderer)
             self.SetCellEditor(has_header, col, choice_editor)
@@ -530,7 +538,7 @@ class CSVPreviewGrid(wx.grid.Grid):
     def get_digest_types(self):
         """Returns a list of the target types"""
 
-        return [self.digest_types[digest_key] \
+        return [self.digest_types[digest_key]
                     for digest_key in self.get_digest_keys()]
 
 
@@ -585,10 +593,11 @@ class CsvImportDialog(wx.Dialog):
         wx.Dialog.__init__(self, *args, **kwds)
 
         self.csvwidgets = CsvParameterWidgets(self, self.csvfilepath)
+
         dialect, self.has_header = sniff(self.csvfilepath)
 
-        self.grid = CSVPreviewGrid(self, -1, \
-                                   has_header=self.has_header, \
+        self.grid = CSVPreviewGrid(self, -1,
+                                   has_header=self.has_header,
                                    csvfilepath=self.csvfilepath)
 
         self.button_cancel = wx.Button(self, wx.ID_CANCEL, "")
@@ -602,14 +611,14 @@ class CsvImportDialog(wx.Dialog):
     def _set_properties(self):
         """Sets dialog title and size limitations of the widgets"""
 
-        self.SetTitle(" ".join(["CSV Import:", self.csvfilename]))
+        self.SetTitle(_("CSV Import: {}").format(self.csvfilename))
         self.SetSize((600, 600))
 
         for button in [self.button_cancel, self.button_ok]:
             button.SetMinSize((80, 28))
 
     def _do_layout(self):
-        """Sizer hell"""
+        """Set sizers"""
 
         sizer_dialog = wx.FlexGridSizer(3, 1, 0, 0)
 
@@ -625,7 +634,7 @@ class CsvImportDialog(wx.Dialog):
             sizer_buttons.AddGrowableCol(col)
 
         # Adding main components
-        sizer_dialog.Add(self.csvwidgets.sizer_csvoptions,  \
+        sizer_dialog.Add(self.csvwidgets.sizer_csvoptions,
                          0, wx.ALL | wx.EXPAND, 5)
         sizer_dialog.Add(self.grid,  1, wx.ALL | wx.EXPAND, 0)
         sizer_dialog.Add(sizer_buttons,  0, wx.ALL | wx.EXPAND, 5)
@@ -665,7 +674,7 @@ class CsvExportDialog(wx.Dialog):
         dialect = csv.get_dialect(csv.list_dialects()[0])
         self.has_header = False
 
-        self.preview_textctrl = CSVPreviewTextCtrl(self, -1, \
+        self.preview_textctrl = CSVPreviewTextCtrl(self, -1,
             style=wx.TE_MULTILINE | wx.TE_READONLY | wx.HSCROLL)
 
         self.button_cancel = wx.Button(self, wx.ID_CANCEL, "")
@@ -689,7 +698,7 @@ class CsvExportDialog(wx.Dialog):
             button.SetMinSize((80, 28))
 
     def _do_layout(self):
-        """Sizer hell"""
+        """Set sizers"""
 
         sizer_dialog = wx.FlexGridSizer(3, 1, 0, 0)
 
@@ -705,7 +714,7 @@ class CsvExportDialog(wx.Dialog):
             sizer_buttons.AddGrowableCol(col)
 
         # Adding main components
-        sizer_dialog.Add(self.csvwidgets.sizer_csvoptions,  \
+        sizer_dialog.Add(self.csvwidgets.sizer_csvoptions,
                          0, wx.ALL | wx.EXPAND, 5)
         sizer_dialog.Add(self.preview_textctrl,  1, wx.ALL | wx.EXPAND, 0)
         sizer_dialog.Add(sizer_buttons,  0, wx.ALL | wx.EXPAND, 5)
@@ -734,7 +743,7 @@ class CsvExportDialog(wx.Dialog):
 # end of class CsvImportDialog
 
 
-class MacroDialog(wx.Frame):
+class MacroDialog(wx.Frame, MainWindowEventMixin):
     """Macro management dialog"""
 
     def __init__(self, parent, macros, *args, **kwds):
@@ -754,11 +763,8 @@ class MacroDialog(wx.Frame):
         self.upper_panel = wx.Panel(self.splitter, -1)
         self.lower_panel = wx.Panel(self.splitter, -1)
 
-        self.codetext_ctrl = PythonSTC(self.upper_panel, -1,
-                                       style=wx.TE_PROCESS_ENTER | \
-                                       wx.TE_PROCESS_TAB | \
-                                       wx.TE_MULTILINE | \
-                                       wx.EXPAND)
+        self.codetext_ctrl = PythonSTC(self.upper_panel, -1, style=wx.EXPAND |
+            wx.TE_PROCESS_ENTER | wx.TE_PROCESS_TAB | wx.TE_MULTILINE)
 
         self.result_ctrl = wx.TextCtrl(self.lower_panel, -1,
           style=wx.TE_MULTILINE | wx.TE_READONLY)
@@ -808,12 +814,12 @@ class MacroDialog(wx.Frame):
     def _set_properties(self):
         """Setup title, size and tooltips"""
 
-        self.SetTitle("Macro list")
+        self.SetTitle(_("Macro list"))
         self.SetSize((800, 600))
-        self.codetext_ctrl.SetToolTipString("Enter python code here.")
-        self.ok_button.SetToolTipString("Accept all changes")
-        self.apply_button.SetToolTipString("Apply changes to current macro")
-        self.cancel_button.SetToolTipString("Remove current macro")
+        self.codetext_ctrl.SetToolTipString(_("Enter python code here."))
+        self.ok_button.SetToolTipString(_("Accept all changes"))
+        self.apply_button.SetToolTipString(_("Apply changes to current macro"))
+        self.cancel_button.SetToolTipString(_("Remove current macro"))
         self.splitter.SetBackgroundStyle(wx.BG_STYLE_COLOUR)
         self.result_ctrl.SetMinSize((10, 10))
 
@@ -832,8 +838,9 @@ class MacroDialog(wx.Frame):
     def OnApply(self, event):
         """Event handler for Apply button"""
 
-        post_command_event(self.parent, MacroReplaceMsg, macros=self.macros)
-        post_command_event(self.parent, MacroExecuteMsg)
+        post_command_event(self.parent, self.MacroReplaceMsg,
+                           macros=self.macros)
+        post_command_event(self.parent, self.MacroExecuteMsg)
 
         event.Skip()
 
@@ -854,13 +861,13 @@ class DimensionsEntryDialog(wx.Dialog):
             wx.DEFAULT_DIALOG_STYLE | wx.MINIMIZE_BOX | wx.STAY_ON_TOP
         wx.Dialog.__init__(self, parent, *args, **kwds)
 
-        self.Rows_Label = wx.StaticText(self, -1, "Rows",
+        self.Rows_Label = wx.StaticText(self, -1, _("Rows"),
                                         style=wx.ALIGN_CENTRE)
         self.X_DimensionsEntry = wx.TextCtrl(self, -1, "")
-        self.Columns_Label = wx.StaticText(self, -1, "Columns", \
+        self.Columns_Label = wx.StaticText(self, -1, _("Columns"),
                                            style=wx.ALIGN_CENTRE)
         self.Y_DimensionsEntry = wx.TextCtrl(self, -1, "")
-        self.Tabs_Label = wx.StaticText(self, -1, "Tables",
+        self.Tabs_Label = wx.StaticText(self, -1, _("Tables"),
                                         style=wx.ALIGN_CENTRE)
         self.Z_DimensionsEntry = wx.TextCtrl(self, -1, "")
 
@@ -890,21 +897,21 @@ class DimensionsEntryDialog(wx.Dialog):
         """Layout sizers"""
 
         grid_sizer_1 = wx.GridSizer(4, 2, 3, 3)
-        grid_sizer_1.Add(self.Rows_Label, 0, \
+        grid_sizer_1.Add(self.Rows_Label, 0,
                          wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 3)
         grid_sizer_1.Add(self.X_DimensionsEntry, 0, wx.EXPAND, 0)
-        grid_sizer_1.Add(self.Columns_Label, 0, \
+        grid_sizer_1.Add(self.Columns_Label, 0,
                          wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 3)
         grid_sizer_1.Add(self.Y_DimensionsEntry, 0, wx.EXPAND, 0)
-        grid_sizer_1.Add(self.Tabs_Label, 0, \
+        grid_sizer_1.Add(self.Tabs_Label, 0,
                          wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 3)
         grid_sizer_1.Add(self.Z_DimensionsEntry, 0, wx.EXPAND, 0)
-        grid_sizer_1.Add(self.ok_button, 0, \
-                        wx.ALL | wx.EXPAND | wx.ALIGN_CENTER_HORIZONTAL | \
-                        wx.ALIGN_CENTER_VERTICAL | wx.FIXED_MINSIZE, 3)
-        grid_sizer_1.Add(self.cancel_button, 0, \
-                        wx.ALL | wx.EXPAND | wx.ALIGN_CENTER_HORIZONTAL | \
-                        wx.ALIGN_CENTER_VERTICAL | wx.FIXED_MINSIZE, 3)
+        grid_sizer_1.Add(self.ok_button, 0, wx.ALL | wx.EXPAND |
+            wx.ALIGN_CENTER_HORIZONTAL | wx.ALIGN_CENTER_VERTICAL |
+            wx.FIXED_MINSIZE, 3)
+        grid_sizer_1.Add(self.cancel_button, 0, wx.ALL | wx.EXPAND |
+            wx.ALIGN_CENTER_HORIZONTAL | wx.ALIGN_CENTER_VERTICAL |
+            wx.FIXED_MINSIZE, 3)
         self.SetSizer(grid_sizer_1)
         grid_sizer_1.Fit(self)
         self.Layout()
@@ -961,7 +968,7 @@ class DimensionsEntryDialog(wx.Dialog):
 # end of class DimensionsEntryDialog
 
 
-class CellEntryDialog(wx.Dialog):
+class CellEntryDialog(wx.Dialog, GridEventMixin):
     """Allows entring three digits"""
 
     def __init__(self, parent):
@@ -974,12 +981,12 @@ class CellEntryDialog(wx.Dialog):
 
         fgs = wx.FlexGridSizer(0, 2)
 
-        fgs.Add(wx.StaticText(self, -1, "Goto cell:"))
+        fgs.Add(wx.StaticText(self, -1, _("Goto cell:")))
         fgs.Add((1, 1))
         fgs.Add((1, VSPACE))
         fgs.Add((1, VSPACE))
 
-        label = wx.StaticText(self, -1, "Row: ")
+        label = wx.StaticText(self, -1, _("Row: "))
         fgs.Add(label, 0, wx.ALIGN_RIGHT | wx.CENTER)
         self.row_textctrl = \
             wx.TextCtrl(self, -1, "", validator=IntValidator())
@@ -988,7 +995,7 @@ class CellEntryDialog(wx.Dialog):
         fgs.Add((1, VSPACE))
         fgs.Add((1, VSPACE))
 
-        label = wx.StaticText(self, -1, "Column: ")
+        label = wx.StaticText(self, -1, _("Column: "))
         fgs.Add(label, 0, wx.ALIGN_RIGHT | wx.CENTER)
         self.col_textctrl = \
             wx.TextCtrl(self, -1, "", validator=IntValidator())
@@ -996,7 +1003,7 @@ class CellEntryDialog(wx.Dialog):
         fgs.Add(self.col_textctrl)
         fgs.Add((1, VSPACE))
         fgs.Add((1, VSPACE))
-        label = wx.StaticText(self, -1, "Table: ")
+        label = wx.StaticText(self, -1, _("Table: "))
         fgs.Add(label, 0, wx.ALIGN_RIGHT | wx.CENTER)
         self.tab_textctrl = \
             wx.TextCtrl(self, -1, "", validator=IntValidator())
@@ -1004,10 +1011,10 @@ class CellEntryDialog(wx.Dialog):
         fgs.Add(self.tab_textctrl)
 
         buttons = wx.StdDialogButtonSizer()  # wx.BoxSizer(wx.HORIZONTAL)
-        b = wx.Button(self, wx.ID_OK, "OK")
+        b = wx.Button(self, wx.ID_OK, _("OK"))
         b.SetDefault()
         buttons.AddButton(b)
-        buttons.AddButton(wx.Button(self, wx.ID_CANCEL, "Cancel"))
+        buttons.AddButton(wx.Button(self, wx.ID_CANCEL, _("Cancel")))
         buttons.Realize()
 
         border = wx.BoxSizer(wx.VERTICAL)
@@ -1038,7 +1045,7 @@ class CellEntryDialog(wx.Dialog):
 
         # Post event
 
-        post_command_event(self.parent, GotoCellMsg, key=tuple(key))
+        post_command_event(self.parent, self.GotoCellMsg, key=tuple(key))
 
 
 class AboutDialog(object):
@@ -1051,15 +1058,17 @@ class AboutDialog(object):
         info = wx.AboutDialogInfo()
         info.Name = "pyspread"
         info.Version = config["version"]
-        info.Copyright = "(C) Martin Manns 2008-2012"
+        info.Copyright = "(C) Martin Manns"
         info.Description = wordwrap(
-            "A non-traditional Python spreadsheet application.\nPyspread is "
-            "based on and written in the programming language Python.",
+          _("A non-traditional Python spreadsheet application.\nPyspread is "
+            "based on and written in the programming language Python."),
             350, wx.ClientDC(parent))
-        info.WebSite = ("http://pyspread.sourceforge.net",
-                        "Pyspread Web site")
+        info.WebSite = ("http://manns.github.com/pyspread/",
+                        _("Pyspread Web site"))
         info.Developers = ["Martin Manns"]
         info.DocWriters = ["Martin Manns", "Bosko Markovic"]
+        info.Translators = ["Joe Hansen", "Mark Haanen", "Yuri Chornoivan",
+                            "Mario Blättermann"]
 
         license_file = open(get_program_path() + "/COPYING", "r")
         license_text = license_file.read()
@@ -1073,23 +1082,24 @@ class AboutDialog(object):
     def _set_properties(self):
         """Setup title and label"""
 
-        self.SetTitle("About pyspread")
+        self.SetTitle(_("About pyspread"))
 
-        self.about_label.SetLabel("pyspread " + VERSION + \
-                                  "\nCopyright Martin Manns 2008-2012")
+        label = _("pyspread {}\nCopyright Martin Manns").format(VERSION)
+
+        self.about_label.SetLabel(label)
 
     def _do_layout(self):
         """Layout sizers"""
 
         sizer_v = wx.BoxSizer(wx.VERTICAL)
         sizer_h = wx.BoxSizer(wx.HORIZONTAL)
-        sizer_h.Add(self.logo_pyspread, 0, \
+        sizer_h.Add(self.logo_pyspread, 0,
             wx.ALL | wx.ALIGN_CENTER_HORIZONTAL | wx.ALIGN_CENTER_VERTICAL, 10)
-        sizer_h.Add(self.about_label, 0, \
+        sizer_h.Add(self.about_label, 0,
             wx.ALL | wx.ALIGN_CENTER_HORIZONTAL | wx.ALIGN_CENTER_VERTICAL, 10)
         sizer_v.Add(sizer_h)
         self.SetSizer(sizer_v)
-        sizer_v.Add(self.button_close, 0, \
+        sizer_v.Add(self.button_close, 0,
             wx.ALL | wx.ALIGN_CENTER_HORIZONTAL | wx.ALIGN_CENTER_VERTICAL, 10)
         sizer_v.Fit(self)
         self.Layout()
@@ -1123,69 +1133,53 @@ class CheckBoxCtrl(wx.CheckBox):
 class PreferencesDialog(wx.Dialog):
     """Dialog for changing pyspread's configuration preferences"""
 
-    parameters = ( \
-        ("max_unredo", { \
-            "label": u"Max. undo steps",
-            "tooltip": u"Maximum number of undo steps",
+    parameters = (
+        ("max_unredo", {
+            "label": _(u"Max. undo steps"),
+            "tooltip": _(u"Maximum number of undo steps"),
             "widget": wx.lib.intctrl.IntCtrl,
             "widget_params": {"min": 0, "allow_long": True},
             "prepocessor": int,
         }),
-        ("grid_rows", { \
-            "label": u"Grid rows",
-            "tooltip": u"Initial number of grid rows when starting pyspread",
+        ("grid_rows", {
+            "label": _(u"Grid rows"),
+            "tooltip": _(u"Number of grid rows when starting pyspread"),
             "widget": wx.lib.intctrl.IntCtrl,
             "widget_params": {"min": 0, "allow_long": True},
             "prepocessor": int,
         }),
-        ("grid_columns", { \
-            "label": u"Grid columns",
-            "tooltip": u"Initial number of grid columns when starting " \
-                       u"pyspread",
+        ("grid_columns", {
+            "label": _(u"Grid columns"),
+            "tooltip": _(u"Number of grid columns when starting pyspread"),
             "widget": wx.lib.intctrl.IntCtrl,
             "widget_params": {"min": 0, "allow_long": True},
             "prepocessor": int,
         }),
-        ("grid_tables", { \
-            "label": u"Grid tables",
-            "tooltip": u"Initial number of grid tables when starting pyspread",
+        ("grid_tables", {
+            "label": _(u"Grid tables"),
+            "tooltip": _(u"Number of grid tables when starting pyspread"),
             "widget": wx.lib.intctrl.IntCtrl,
             "widget_params": {"min": 0, "allow_long": True},
             "prepocessor": int,
         }),
-        ("max_result_length", { \
-            "label": u"Max. result length",
-            "tooltip": u"Maximum length of cell result string",
+        ("max_result_length", {
+            "label": _(u"Max. result length"),
+            "tooltip": _(u"Maximum length of cell result string"),
             "widget": wx.lib.intctrl.IntCtrl,
             "widget_params": {"min": 0, "allow_long": True},
             "prepocessor": int,
         }),
-        ("gpg_key_uid", { \
-            "label": u"GPG key name",
-            "tooltip": u"Name of the GPG key that is used for signing files",
+        ("gpg_key_uid", {
+            "label": _(u"GPG key name"),
+            "tooltip": _(u"Name of the GPG key for signing files"),
             "widget": wx.TextCtrl,
             "widget_params": {},
             "prepocessor": unicode,
         }),
-        ("gpg_key_passphrase", { \
-            "label": u"GPG key passphrase",
-            "tooltip": \
-                u"Passphrase of the GPG key that is used for signing files",
-            "widget": wx.TextCtrl,
-            "widget_params": {"style": wx.TE_PASSWORD | wx.OK},
-            "prepocessor": unicode,
-        }),
-        ("gpg_key_passphrase_isstored", { \
-            "label": u"Store passphrase in config file",
-            "tooltip": u"If False then the passprase is not stored on exit",
-            "widget": CheckBoxCtrl,
-            "widget_params": {},
-            "prepocessor": bool,
-        }),
     )
 
     def __init__(self, *args, **kwargs):
-        kwargs["title"] = u"Preferences"
+        kwargs["title"] = _(u"Preferences")
         kwargs["style"] = \
             wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER | wx.THICK_FRAME
         wx.Dialog.__init__(self, *args, **kwargs)
@@ -1248,39 +1242,29 @@ class GPGParamsDialog(wx.Dialog):
 
         sizer = wx.FlexGridSizer(len(params), 2, 5, 5)
 
-        label = wx.StaticText(self, -1, "GPG key data")
+        label = wx.StaticText(self, -1, _("GPG key data"))
         sizer.Add(label, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
         sizer.Add(wx.Panel(self, -1), 0, wx.ALIGN_CENTRE | wx.ALL, 5)
 
         self.textctrls = []
 
-        for labeltext, _, pwd in params:
+        for labeltext, __ in params:
             label = wx.StaticText(self, -1, labeltext)
             sizer.Add(label, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
 
-            if pwd:
-                textctrl = wx.TextCtrl(self, -1, "", size=(80, -1),
-                                       style=wx.TE_PASSWORD)
-            else:
-                textctrl = wx.TextCtrl(self, -1, "", size=(80, -1))
+            textctrl = wx.TextCtrl(self, -1, "", size=(80, -1))
 
             self.textctrls.append(textctrl)
 
             sizer.Add(textctrl, 1, wx.ALIGN_CENTRE | wx.ALL, 5)
 
-        label = wx.StaticText(self, -1, "Store passphrase")
-        sizer.Add(label)
-        store_passwd_checkbox = wx.CheckBox(self, True, "")
-        store_passwd_checkbox.SetValue(True)
-        sizer.Add(store_passwd_checkbox)
-
         ok_button = wx.Button(self, wx.ID_OK)
-        ok_button.SetToolTipString("Starts key generation.")
+        ok_button.SetToolTipString(_("Starts key generation."))
         ok_button.SetDefault()
         sizer.Add(ok_button)
 
         cancel_button = wx.Button(self, wx.ID_CANCEL)
-        cancel_button.SetToolTipString("Exits pyspread.")
+        cancel_button.SetToolTipString(_("Exits pyspread."))
         sizer.Add(cancel_button)
 
         self.SetSizer(sizer)
