@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright 2011 Martin Manns
+# Copyright Martin Manns
 # Distributed under the terms of the GNU General Public License
 
 # --------------------------------------------------------------------
@@ -32,20 +32,28 @@ Provides
 
 import wx.grid
 
-from _events import *
+from _events import post_command_event, EventMixin
 
 from _grid_table import GridTable
 from _grid_renderer import GridRenderer
 from _gui_interfaces import GuiInterfaces
 from _menubars import ContextMenu
+from _chart_dialog import ChartDialog
+
+from src.config import config
+
+import src.lib.i18n as i18n
 
 import src.lib.xrect as xrect
 from src.model.model import CodeArray
 
 from src.actions._grid_actions import AllGridActions
 
+#use ugettext instead of getttext to avoid unicode errors
+_ = i18n.language.ugettext
 
-class Grid(wx.grid.Grid):
+
+class Grid(wx.grid.Grid, EventMixin):
     """Pyspread's main grid"""
 
     def __init__(self, main_window, *args, **kwargs):
@@ -61,7 +69,7 @@ class Grid(wx.grid.Grid):
 
         # Create new grid
         self.code_array = CodeArray(dimensions)
-        post_command_event(self, GridActionNewMsg, shape=dimensions)
+        post_command_event(self, self.GridActionNewMsg, shape=dimensions)
 
         _grid_table = GridTable(self, self.code_array)
         self.SetTable(_grid_table, True)
@@ -81,7 +89,6 @@ class Grid(wx.grid.Grid):
         self.actions = AllGridActions(self)
 
         # Layout and bindings
-
         self._layout()
         self._bind()
 
@@ -141,30 +148,39 @@ class Grid(wx.grid.Grid):
 
         # Cell code events
 
-        main_window.Bind(EVT_COMMAND_CODE_ENTRY, c_handlers.OnCellText)
+        main_window.Bind(self.EVT_CMD_CODE_ENTRY, c_handlers.OnCellText)
+
+        main_window.Bind(self.EVT_CMD_INSERT_BMP, c_handlers.OnInsertBitmap)
+        main_window.Bind(self.EVT_CMD_LINK_BMP, c_handlers.OnLinkBitmap)
+        main_window.Bind(self.EVT_CMD_INSERT_CHART,
+                         c_handlers.OnInsertChartDialog)
 
         # Cell attribute events
 
-        main_window.Bind(EVT_COMMAND_FONT, c_handlers.OnCellFont)
-        main_window.Bind(EVT_COMMAND_FONTSIZE, c_handlers.OnCellFontSize)
-        main_window.Bind(EVT_COMMAND_FONTBOLD, c_handlers.OnCellFontBold)
-        main_window.Bind(EVT_COMMAND_FONTITALICS, c_handlers.OnCellFontItalics)
-        main_window.Bind(EVT_COMMAND_FONTUNDERLINE,
+        main_window.Bind(self.EVT_CMD_FONT, c_handlers.OnCellFont)
+        main_window.Bind(self.EVT_CMD_FONTSIZE, c_handlers.OnCellFontSize)
+        main_window.Bind(self.EVT_CMD_FONTBOLD, c_handlers.OnCellFontBold)
+        main_window.Bind(self.EVT_CMD_FONTITALICS,
+                    c_handlers.OnCellFontItalics)
+        main_window.Bind(self.EVT_CMD_FONTUNDERLINE,
                     c_handlers.OnCellFontUnderline)
-        main_window.Bind(EVT_COMMAND_FONTSTRIKETHROUGH,
+        main_window.Bind(self.EVT_CMD_FONTSTRIKETHROUGH,
                     c_handlers.OnCellFontStrikethrough)
-        main_window.Bind(EVT_COMMAND_FROZEN, c_handlers.OnCellFrozen)
-        main_window.Bind(EVT_COMMAND_JUSTIFICATION,
+        main_window.Bind(self.EVT_CMD_FROZEN, c_handlers.OnCellFrozen)
+        main_window.Bind(self.EVT_CMD_MERGE, c_handlers.OnMerge)
+        main_window.Bind(self.EVT_CMD_JUSTIFICATION,
                     c_handlers.OnCellJustification)
-        main_window.Bind(EVT_COMMAND_ALIGNMENT, c_handlers.OnCellAlignment)
-        main_window.Bind(EVT_COMMAND_BORDERWIDTH, c_handlers.OnCellBorderWidth)
-        main_window.Bind(EVT_COMMAND_BORDERCOLOR, c_handlers.OnCellBorderColor)
-        main_window.Bind(EVT_COMMAND_BACKGROUNDCOLOR,
+        main_window.Bind(self.EVT_CMD_ALIGNMENT, c_handlers.OnCellAlignment)
+        main_window.Bind(self.EVT_CMD_BORDERWIDTH,
+                    c_handlers.OnCellBorderWidth)
+        main_window.Bind(self.EVT_CMD_BORDERCOLOR,
+                    c_handlers.OnCellBorderColor)
+        main_window.Bind(self.EVT_CMD_BACKGROUNDCOLOR,
                     c_handlers.OnCellBackgroundColor)
-        main_window.Bind(EVT_COMMAND_TEXTCOLOR, c_handlers.OnCellTextColor)
-        main_window.Bind(EVT_COMMAND_ROTATIONDIALOG,
+        main_window.Bind(self.EVT_CMD_TEXTCOLOR, c_handlers.OnCellTextColor)
+        main_window.Bind(self.EVT_CMD_ROTATIONDIALOG,
                     c_handlers.OnTextRotationDialog)
-        main_window.Bind(EVT_COMMAND_TEXTROTATATION,
+        main_window.Bind(self.EVT_CMD_TEXTROTATATION,
                     c_handlers.OnCellTextRotation)
 
         # Cell selection events
@@ -173,18 +189,18 @@ class Grid(wx.grid.Grid):
 
         # Grid view events
 
-        main_window.Bind(EVT_COMMAND_REFRESH_SELECTION,
+        main_window.Bind(self.EVT_CMD_REFRESH_SELECTION,
                     handlers.OnRefreshSelectedCells)
-        main_window.Bind(EVT_COMMAND_DISPLAY_GOTO_CELL_DIALOG,
+        main_window.Bind(self.EVT_CMD_DISPLAY_GOTO_CELL_DIALOG,
                     handlers.OnDisplayGoToCellDialog)
-        main_window.Bind(EVT_COMMAND_GOTO_CELL, handlers.OnGoToCell)
-        main_window.Bind(EVT_COMMAND_ZOOM_IN, handlers.OnZoomIn)
-        main_window.Bind(EVT_COMMAND_ZOOM_OUT, handlers.OnZoomOut)
-        main_window.Bind(EVT_COMMAND_ZOOM_STANDARD, handlers.OnZoomStandard)
+        main_window.Bind(self.EVT_CMD_GOTO_CELL, handlers.OnGoToCell)
+        main_window.Bind(self.EVT_CMD_ZOOM_IN, handlers.OnZoomIn)
+        main_window.Bind(self.EVT_CMD_ZOOM_OUT, handlers.OnZoomOut)
+        main_window.Bind(self.EVT_CMD_ZOOM_STANDARD, handlers.OnZoomStandard)
 
         # Find events
-        main_window.Bind(EVT_COMMAND_FIND, handlers.OnFind)
-        main_window.Bind(EVT_COMMAND_REPLACE, handlers.OnShowFindReplace)
+        main_window.Bind(self.EVT_CMD_FIND, handlers.OnFind)
+        main_window.Bind(self.EVT_CMD_REPLACE, handlers.OnShowFindReplace)
         main_window.Bind(wx.EVT_FIND, handlers.OnReplaceFind)
         main_window.Bind(wx.EVT_FIND_NEXT, handlers.OnReplaceFind)
         main_window.Bind(wx.EVT_FIND_REPLACE, handlers.OnReplace)
@@ -193,15 +209,15 @@ class Grid(wx.grid.Grid):
 
         # Grid change events
 
-        main_window.Bind(EVT_COMMAND_INSERT_ROWS, handlers.OnInsertRows)
-        main_window.Bind(EVT_COMMAND_INSERT_COLS, handlers.OnInsertCols)
-        main_window.Bind(EVT_COMMAND_INSERT_TABS, handlers.OnInsertTabs)
+        main_window.Bind(self.EVT_CMD_INSERT_ROWS, handlers.OnInsertRows)
+        main_window.Bind(self.EVT_CMD_INSERT_COLS, handlers.OnInsertCols)
+        main_window.Bind(self.EVT_CMD_INSERT_TABS, handlers.OnInsertTabs)
 
-        main_window.Bind(EVT_COMMAND_DELETE_ROWS, handlers.OnDeleteRows)
-        main_window.Bind(EVT_COMMAND_DELETE_COLS, handlers.OnDeleteCols)
-        main_window.Bind(EVT_COMMAND_DELETE_TABS, handlers.OnDeleteTabs)
+        main_window.Bind(self.EVT_CMD_DELETE_ROWS, handlers.OnDeleteRows)
+        main_window.Bind(self.EVT_CMD_DELETE_COLS, handlers.OnDeleteCols)
+        main_window.Bind(self.EVT_CMD_DELETE_TABS, handlers.OnDeleteTabs)
 
-        main_window.Bind(EVT_COMMAND_SHOW_RESIZE_GRID_DIALOG,
+        main_window.Bind(self.EVT_CMD_SHOW_RESIZE_GRID_DIALOG,
                                                   handlers.OnResizeGridDialog)
 
         main_window.Bind(wx.grid.EVT_GRID_ROW_SIZE, handlers.OnRowSize)
@@ -209,8 +225,8 @@ class Grid(wx.grid.Grid):
 
         # Undo/Redo events
 
-        main_window.Bind(EVT_COMMAND_UNDO, handlers.OnUndo)
-        main_window.Bind(EVT_COMMAND_REDO, handlers.OnRedo)
+        main_window.Bind(self.EVT_CMD_UNDO, handlers.OnUndo)
+        main_window.Bind(self.EVT_CMD_REDO, handlers.OnRedo)
 
     _get_selection = lambda self: self.actions.get_selection()
     selection = property(_get_selection, doc="Grid selection")
@@ -231,7 +247,7 @@ class Grid(wx.grid.Grid):
         while self.IsVisible(topleft[0], col, wholeCellVisible=False):
             col += 1
         lowerright = (row, col)  # This cell is *not* visible
-        return (slice(topleft[0], lowerright[0]), \
+        return (slice(topleft[0], lowerright[0]),
                 slice(topleft[1], lowerright[1]),
                 slice(self.current_table, self.current_table + 1))
 
@@ -332,6 +348,61 @@ class GridCellEventHandlers(object):
 
         event.Skip()
 
+    def OnInsertBitmap(self, event):
+        """Insert bitmap event handler"""
+
+        # Get file name
+        wildcard = "*"
+        message = _("Select bitmap for current cell")
+        style = wx.OPEN | wx.CHANGE_DIR
+        filepath, __ = self.grid.interfaces.get_filepath_findex_from_user(
+                                                    wildcard, message, style)
+
+        bmp = wx.Bitmap(filepath)
+        if bmp.Size == (-1, -1):
+            # Bitmap could not be read
+            return
+
+        key = self.grid.actions.cursor
+        code = self.grid.main_window.actions.bmp2code(key, bmp)
+
+        self.grid.actions.set_code(key, code)
+
+    def OnLinkBitmap(self, event):
+        """Link bitmap event handler"""
+
+        # Get file name
+        wildcard = "*"
+        message = _("Select bitmap for current cell")
+        style = wx.OPEN | wx.CHANGE_DIR
+        filepath, __ = self.grid.interfaces.get_filepath_findex_from_user(
+                                                    wildcard, message, style)
+
+        bmp = wx.Bitmap(filepath)
+        if bmp.Size == (-1, -1):
+            # Bitmap could not be read
+            return
+
+        code = "wx.Bitmap('{}')".format(filepath)
+
+        key = self.grid.actions.cursor
+        self.grid.actions.set_code(key, code)
+
+    def OnInsertChartDialog(self, event):
+        """Chart dialog event handler"""
+
+        cell_code = self.grid.code_array(self.grid.actions.cursor)
+
+        if cell_code is None:
+            cell_code = u""
+
+        chart_dialog = ChartDialog(self.grid, cell_code)
+
+        if chart_dialog.ShowModal() == wx.ID_OK:
+            code = chart_dialog.get_code()
+            key = self.grid.actions.cursor
+            self.grid.actions.set_code(key, code)
+
     # Cell attribute events
 
     def OnCellFont(self, event):
@@ -361,7 +432,7 @@ class GridCellEventHandlers(object):
             elif event.weight == "wxBOLD":
                 weight = wx.BOLD
             else:
-                raise ValueError, "Weight " + str(event.weight) + "unknown"
+                raise ValueError(_("Weight {} unknown").format(event.weight))
 
             self.grid.actions.set_attr("fontweight", weight)
 
@@ -381,7 +452,7 @@ class GridCellEventHandlers(object):
             elif event.style == "wxITALIC":
                 style = wx.ITALIC
             else:
-                raise ValueError, "Style " + str(event.style) + "unknown"
+                raise ValueError(_("Style {} unknown").format(event.style))
 
             self.grid.actions.set_attr("fontstyle", style)
 
@@ -419,6 +490,13 @@ class GridCellEventHandlers(object):
 
         event.Skip()
 
+    def OnMerge(self, event):
+        """Merge cells event handler"""
+
+        self.grid.actions.merge_selected_cells(self.grid.selection)
+
+        self.grid.ForceRefresh()
+
     def OnCellJustification(self, event):
         """Horizontal cell justification event handler"""
 
@@ -475,12 +553,12 @@ class GridCellEventHandlers(object):
         """Text rotation dialog event handler"""
 
         cond_func = lambda i: 0 <= i <= 359
-        angle = self.grid.interfaces.get_int_from_user( \
-            "Enter text angle in degrees.", cond_func)
+        get_int = self.grid.interfaces.get_int_from_user
+        angle = get_int(_("Enter text angle in degrees."), cond_func)
 
         if angle is not None:
-            post_command_event(self.grid.main_window, TextRotationMsg,
-                                angle=angle)
+            post_command_event(self.grid.main_window,
+                               self.grid.TextRotationMsg, angle=angle)
 
     def OnCellTextRotation(self, event):
         """Cell text rotation event handler"""
@@ -495,20 +573,52 @@ class GridCellEventHandlers(object):
         """Updates the entry line"""
 
         cell_code = self.grid.code_array(key)
-        post_command_event(self.grid, EntryLineMsg, text=cell_code)
+
+        # Do not display anything if there is no cell code, i. e. it is None
+        # Also do not display anything if there is too much code because
+        # wx.TextCtrl can only handle up to max_textctrl_length characters
+
+        if cell_code is None or \
+           len(cell_code) < int(config["max_textctrl_length"]):
+
+            post_command_event(self.grid, self.grid.EntryLineMsg,
+                               text=cell_code)
 
     def _update_attribute_toolbar(self, key):
         """Updates the attribute toolbar"""
 
-        post_command_event(self.grid, ToolbarUpdateMsg, key=key,
+        post_command_event(self.grid, self.grid.ToolbarUpdateMsg, key=key,
                            attr=self.grid.code_array.cell_attributes[key])
 
     # Cell selection event handlers
 
+    def set_cursor(self, row, col, tab):
+        """Sets grid cursor to key"""
+
+        self.grid.SetGridCursor(row, col)
+        self._last_selected_cell = row, col, tab
+        return
+
     def OnCellSelected(self, event):
         """Cell selection event handler"""
 
-        key = event.Row, event.Col, self.grid.current_table
+        key = row, col, tab = event.Row, event.Col, self.grid.current_table
+
+        # Is the cell merged then go to merging cell
+        merge_area = self.grid.code_array.cell_attributes[key]["merge_area"]
+
+        if merge_area is not None:
+            top, left, bottom, right = merge_area
+            if self._last_selected_cell == (top, left, tab):
+                if row == top + 1:
+                    self.set_cursor(bottom + 1, left, tab)
+                    return
+                elif col == left + 1:
+                    self.set_cursor(top, right + 1, tab)
+                    return
+            elif (row, col) != (top, left):
+                self.set_cursor(top, left, tab)
+                return
 
         # Redraw cursor
         self.grid.ForceRefresh()
@@ -519,6 +629,8 @@ class GridCellEventHandlers(object):
         # Update attribute toolbar
         self._update_attribute_toolbar(key)
 
+        self._last_selected_cell = key
+
         event.Skip()
 
 
@@ -541,6 +653,13 @@ class GridEventHandlers(object):
         col = grid.XToCol(pos_x)
         tab = grid.current_table
 
+        key = row, col, tab
+
+        merge_area = self.grid.code_array.cell_attributes[key]["merge_area"]
+        if merge_area is not None:
+            top, left, bottom, right = merge_area
+            row, col = top, left
+
         grid.actions.on_mouse_over((row, col, tab))
 
         event.Skip()
@@ -564,7 +683,6 @@ class GridEventHandlers(object):
             self.grid.main_window.entry_line.SetFocus()
             self.grid.main_window.entry_line.SetInsertionPointEnd()
 
-
         elif event.ControlDown() and event.ShiftDown() and not event.AltDown():
             # Add click position as absolute reference
             self.grid.actions.append_reference_code(cursor, click_key,
@@ -589,11 +707,26 @@ class GridEventHandlers(object):
         if event.ControlDown():
             if keycode == 388:
                 # Ctrl + + pressed
-                post_command_event(self.grid, ZoomInMsg)
+                post_command_event(self.grid, self.grid.ZoomInMsg)
 
             elif keycode == 390:
                 # Ctrl + - pressed
-                post_command_event(self.grid, ZoomOutMsg)
+                post_command_event(self.grid, self.grid.ZoomOutMsg)
+
+            elif keycode == 13:
+            # <Ctrl> + <Enter>
+                grid = self.grid
+                grid.DisableCellEditControl()
+
+                row = self.grid.GetGridCursorRow()
+                col = self.grid.GetGridCursorCol()
+                tab = grid.current_table
+                key = row, col, tab
+
+                val = grid.code_array(key)
+                grid.actions.set_code(key, '"' + val + '"')
+
+                grid.MoveCursorDown(False)
 
         else:
             # No Ctrl pressed
@@ -609,7 +742,6 @@ class GridEventHandlers(object):
                 self.grid.actions.delete_selection()
 
                 # Update grid
-
                 self.grid.ForceRefresh()
 
                 # Do not enter cell
@@ -690,9 +822,9 @@ class GridEventHandlers(object):
 
         if event.ControlDown():
             if event.WheelRotation > 0:
-                post_command_event(self.grid, ZoomInMsg)
+                post_command_event(self.grid, self.grid.ZoomInMsg)
             else:
-                post_command_event(self.grid, ZoomOutMsg)
+                post_command_event(self.grid, self.grid.ZoomOutMsg)
         else:
             event.Skip()
 
@@ -709,17 +841,16 @@ class GridEventHandlers(object):
         if findpos is None:
             # If nothing is found mention it in the statusbar and return
 
-            statustext = "'" + text + "' not found."
+            statustext = _("'{}' not found.").format(text)
 
         else:
             # Otherwise select cell with next occurrence if successful
             self.grid.actions.cursor = findpos
 
             # Update statusbar
-            statustext = "Found '" + text + "' in cell " + \
-                         unicode(list(findpos)) + "."
+            statustext = _(u"Found '{}' in cell {}.").format(text, findpos)
 
-        post_command_event(self.grid.main_window, StatusBarMsg,
+        post_command_event(self.grid.main_window, self.grid.StatusBarMsg,
                            text=statustext)
 
         event.Skip()
@@ -738,7 +869,7 @@ class GridEventHandlers(object):
     def _wxflag2flag(self, wxflag):
         """Converts wxPython integer flag to pyspread flag list"""
 
-        wx_flags = { \
+        wx_flags = {
             0: ["UP", ],
             1: ["DOWN"],
             2: ["UP", "WHOLE_WORD"],
@@ -857,7 +988,7 @@ class GridEventHandlers(object):
         event.Skip()
 
     def OnDeleteCols(self, event):
-        """Deletes columnss from all tables of the grid"""
+        """Deletes columns from all tables of the grid"""
 
         _, no_cols = self._get_no_rowscols(self.grid.selection.get_bbox())
 
@@ -890,8 +1021,8 @@ class GridEventHandlers(object):
 
         self.grid.GetTable().ResetView()
 
-        statustext = "Grid dimensions changed to " + str(new_shape) + "."
-        post_command_event(self.grid.main_window, StatusBarMsg,
+        statustext = _("Grid dimensions changed to {}.").format(new_shape)
+        post_command_event(self.grid.main_window, self.grid.StatusBarMsg,
                            text=statustext)
 
         event.Skip()
