@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright 2011 Martin Manns
+# Copyright Martin Manns
 # Distributed under the terms of the GNU General Public License
 
 # --------------------------------------------------------------------
@@ -37,12 +37,15 @@ import types
 import wx
 import wx.lib.agw.genericmessagedialog as GMD
 
-from config import config
+import src.lib.i18n as i18n
 
 from _dialogs import MacroDialog, DimensionsEntryDialog, AboutDialog
 from _dialogs import CsvImportDialog, CellEntryDialog, CsvExportDialog
 from _dialogs import PreferencesDialog, GPGParamsDialog
 
+#use ugettext instead of getttext to avoid unicode errors
+_ = i18n.language.ugettext
+
 
 class ModalDialogInterfaceMixin(object):
     """Main window interfaces to modal dialogs"""
@@ -60,8 +63,8 @@ class ModalDialogInterfaceMixin(object):
         # Grid dimension dialog
 
         if no_dim != 3:
-            raise NotImplementedError, \
-                  "Currently, only 3D grids are supported."
+            raise NotImplementedError(
+                _("Currently, only 3D grids are supported."))
 
         dim_dialog = DimensionsEntryDialog(self.main_window)
 
@@ -94,10 +97,10 @@ class ModalDialogInterfaceMixin(object):
     def get_save_request_from_user(self):
         """Queries user if grid should be saved"""
 
-        msg = "There are unsaved changes.\nDo you want to save?"
+        msg = _("There are unsaved changes.\nDo you want to save?")
 
         dlg = GMD.GenericMessageDialog(self.main_window, msg,
-            "Unsaved changes", wx.YES_NO | wx.ICON_QUESTION | wx.CANCEL)
+            _("Unsaved changes"), wx.YES_NO | wx.ICON_QUESTION | wx.CANCEL)
 
         save_choice = dlg.ShowModal()
 
@@ -196,9 +199,9 @@ class ModalDialogInterfaceMixin(object):
         except csv.Error, err:
             # Display modal warning dialog
 
-            msg = csvfilename + '" does not seem to be a valid CSV file.' + \
-                '\n \nOpening it yielded the error:\n' + str(err)
-            short_msg = 'Error reading CSV file'
+            msg = _("'{}' does not seem to be a valid CSV file.\n \nOpening it"
+                    " yielded the error:\n{}").format(csvfilename, err)
+            short_msg = _('Error reading CSV file')
 
             self.display_warning(msg, short_msg)
 
@@ -247,7 +250,8 @@ class ModalDialogInterfaceMixin(object):
 
         return dialect, has_header, digest_types
 
-    def get_int_from_user(self, title="Enter integer value", cond_func=lambda i: i is not None):
+    def get_int_from_user(self, title="Enter integer value",
+                          cond_func=lambda i: i is not None):
         """Opens an integer entry dialog and returns integer
 
         Parameters
@@ -262,7 +266,7 @@ class ModalDialogInterfaceMixin(object):
 
         is_integer = False
 
-        while not is_integer :
+        while not is_integer:
             dlg = wx.TextEntryDialog(None, title, title)
 
             if dlg.ShowModal() == wx.ID_OK:
@@ -283,6 +287,7 @@ class ModalDialogInterfaceMixin(object):
 
         return integer
 
+
 class DialogInterfaceMixin(object):
     """Main window interfaces to dialogs that are not modal"""
 
@@ -305,7 +310,7 @@ class DialogInterfaceMixin(object):
     def display_about(self, parent):
         """Displays About dialog"""
 
-        about_dialog = AboutDialog(parent)
+        AboutDialog(parent)
 
 
 class GuiInterfaces(DialogInterfaceMixin, ModalDialogInterfaceMixin):
@@ -314,25 +319,20 @@ class GuiInterfaces(DialogInterfaceMixin, ModalDialogInterfaceMixin):
     def __init__(self, main_window):
         self.main_window = main_window
 
-def get_key_params_from_user():
-    """Displays parameter entry dialog and returns parameter string"""
 
-    gpg_key_parameters = [ \
-        ('Key-Type', 'DSA'),
-        ('Key-Length', '2048'),
-        ('Subkey-Type', 'ELG-E'),
-        ('Subkey-Length', '2048'),
-        ('Expire-Date', '0'),
+def get_key_params_from_user():
+    """Displays parameter entry dialog and returns parameter dict"""
+
+    gpg_key_parameters = [
+        ('key_type', 'DSA'),
+        ('key_length', '2048'),
+        ('subkey_type', 'ELG-E'),
+        ('subkey_length', '2048'),
+        ('expire_date', '0'),
     ]
 
-    PASSWD = True
-    NO_PASSWD = False
-
-    params = [ \
-        ['Real name', 'Name-Real', NO_PASSWD],
-        ['Passphrase', 'Passphrase', PASSWD],
-        ['E-mail', 'Name-Email', NO_PASSWD],
-        ['Comment', 'Name-Comment', NO_PASSWD],
+    params = [
+        [_('Real name'), 'name_real'],
     ]
 
     vals = [""] * len(params)
@@ -348,47 +348,18 @@ def get_key_params_from_user():
             sys.exit()
 
         vals = [textctrl.Value for textctrl in dlg.textctrls]
-        config["gpg_key_passphrase_isstored"] = \
-            repr(dlg.store_passwd_checkbox.Value)
 
         dlg.Destroy()
 
         if "" in vals:
-            msg = "Please enter a value in each field."
+            msg = _("Please enter a value in each field.")
 
-            dlg = GMD.GenericMessageDialog(None, msg, "Missing value",
+            dlg = GMD.GenericMessageDialog(None, msg, _("Missing value"),
                                            wx.OK | wx.ICON_ERROR)
             dlg.ShowModal()
             dlg.Destroy()
 
-    for (_, key, _), val in zip(params, vals):
+    for (__, key), val in zip(params, vals):
         gpg_key_parameters.insert(-2, (key, val))
 
-    return gpg_key_parameters
-
-
-def get_gpg_passwd_from_user(stored=True):
-    """Opens a dialog for a GPG password and returns the password or None
-
-    Parameters
-    ----------
-
-    stored: Bool
-    \tIf True then a message is displayed that the password is stored on disk
-
-    """
-
-    if stored:
-        stored_msg = \
-            '\nNote that it will be stored as clear text in .pyspreadrc'
-    else:
-        stored_msg = ''
-
-    dlg = wx.TextEntryDialog(None, 'Please enter your GPG key passphrase.' + \
-            stored_msg, 'GPG key passphrase', '', style=wx.TE_PASSWORD | wx.OK)
-
-    if dlg.ShowModal() == wx.ID_OK:
-        dlg.Destroy()
-        return dlg.GetValue()
-
-    dlg.Destroy()
\ No newline at end of file
+    return dict(gpg_key_parameters)
\ No newline at end of file
