@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright 2011 Martin Manns
+# Copyright Martin Manns
 # Distributed under the terms of the GNU General Public License
 
 # --------------------------------------------------------------------
@@ -31,15 +31,15 @@ Provides:
 
 import ast
 import os
-import gettext
 
 import wx
 import wx.aui
 
+import src.lib.i18n as i18n
 from src.config import config
 
 from _menubars import MainMenu
-from _toolbars import MainToolbar, FindToolbar, AttributesToolbar
+from _toolbars import MainToolbar, MacroToolbar, FindToolbar, AttributesToolbar
 from _widgets import EntryLine, StatusBar, TableChoiceIntCtrl
 
 from src.lib.clipboard import Clipboard
@@ -49,14 +49,15 @@ from src.gui.icons import icons
 
 from _grid import Grid
 
-from _events import *
+from _events import post_command_event, EventMixin
 
 from src.actions._main_window_actions import AllMainWindowActions
 
-_ = gettext.gettext
+#use ugettext instead of getttext to avoid unicode errors
+_ = i18n.language.ugettext
 
 
-class MainWindow(wx.Frame):
+class MainWindow(wx.Frame, EventMixin):
     """Main window of pyspread"""
 
     def __init__(self, parent, *args, **kwargs):
@@ -84,17 +85,18 @@ class MainWindow(wx.Frame):
         self.SetMenuBar(self.menubar)
 
         # Disable menu item for leaving safe mode
-        post_command_event(self, SafeModeExitMsg)
+        post_command_event(self, self.SafeModeExitMsg)
 
         # Status bar
         statusbar = StatusBar(self)
         self.SetStatusBar(statusbar)
 
-        welcome_text = "Welcome to pyspread."
-        post_command_event(self, StatusBarMsg, text=welcome_text)
+        welcome_text = _("Welcome to pyspread.")
+        post_command_event(self, self.StatusBarMsg, text=welcome_text)
 
         # Toolbars
         self.main_toolbar = MainToolbar(self, -1)
+        self.macro_toolbar = MacroToolbar(self, -1)
         self.find_toolbar = FindToolbar(self, -1)
         self.attributes_toolbar = AttributesToolbar(self, -1)
 
@@ -103,7 +105,7 @@ class MainWindow(wx.Frame):
 
         # Main grid
 
-        dimensions = ( \
+        dimensions = (
             config["grid_rows"],
             config["grid_columns"],
             config["grid_tables"],
@@ -153,13 +155,14 @@ class MainWindow(wx.Frame):
         self.SetMinSize((2, 2))
 
         # Leave save mode
-        post_command_event(self, SafeModeExitMsg)
+        post_command_event(self, self.SafeModeExitMsg)
 
     def _set_menu_toggles(self):
         """Enable menu bar view item checkmarks"""
 
-        toggles = [ \
+        toggles = [
             (self.main_toolbar, "main_window_toolbar", _("Main toolbar")),
+            (self.macro_toolbar, "macro_toolbar", _("Macro toolbar")),
             (self.attributes_toolbar, "attributes_toolbar",
                                                        _("Format toolbar")),
             (self.find_toolbar, "find_toolbar", _("Find toolbar")),
@@ -172,7 +175,8 @@ class MainWindow(wx.Frame):
             pane = self._mgr.GetPane(pane_name)
 
             # Get menu item to toggle
-            toggle_id = self.menubar.FindMenuItem("View", toggle_label)
+            toggle_id = self.menubar.FindMenuItem(_("View"), toggle_label)
+
             toggle_item = self.menubar.FindItemById(toggle_id)
 
             # Adjust toggle to pane visibility
@@ -182,6 +186,12 @@ class MainWindow(wx.Frame):
         """Adds widgets to the wx.aui manager and controls the layout"""
 
         # Add the toolbars to the manager
+
+        self._mgr.AddPane(self.macro_toolbar, wx.aui.AuiPaneInfo().
+            Name("macro_toolbar").Caption("Macro Toolbar").
+            ToolbarPane().Top().Row(0).CloseButton(False).
+            LeftDockable(False).RightDockable(False))
+
         self._mgr.AddPane(self.main_toolbar, wx.aui.AuiPaneInfo().
             Name("main_window_toolbar").Caption("Main Toolbar").
             ToolbarPane().Top().Row(0).CloseButton(False).
@@ -233,77 +243,80 @@ class MainWindow(wx.Frame):
         self.Bind(wx.EVT_SIZE, handlers.OnSize)
 
         # Content changed event, adjusts title bar with star
-        self.Bind(EVT_CONTENT_CHANGED, handlers.OnContentChanged)
+        self.Bind(self.EVT_CONTENT_CHANGED, handlers.OnContentChanged)
 
         # Program state events
 
-        self.Bind(EVT_COMMAND_TITLE, handlers.OnTitle)
-        self.Bind(EVT_COMMAND_SAFE_MODE_ENTRY, handlers.OnSafeModeEntry)
-        self.Bind(EVT_COMMAND_SAFE_MODE_EXIT, handlers.OnSafeModeExit)
+        self.Bind(self.EVT_CMD_TITLE, handlers.OnTitle)
+        self.Bind(self.EVT_CMD_SAFE_MODE_ENTRY, handlers.OnSafeModeEntry)
+        self.Bind(self.EVT_CMD_SAFE_MODE_EXIT, handlers.OnSafeModeExit)
         self.Bind(wx.EVT_CLOSE, handlers.OnClose)
-        self.Bind(EVT_COMMAND_CLOSE, handlers.OnClose)
+        self.Bind(self.EVT_CMD_CLOSE, handlers.OnClose)
 
         # Preferences events
 
-        self.Bind(EVT_COMMAND_PREFERENCES, handlers.OnPreferences)
+        self.Bind(self.EVT_CMD_PREFERENCES, handlers.OnPreferences)
 
         # Toolbar toggle events
 
-        self.Bind(EVT_COMMAND_MAINTOOLBAR_TOGGLE,
+        self.Bind(self.EVT_CMD_MAINTOOLBAR_TOGGLE,
                   handlers.OnMainToolbarToggle)
-        self.Bind(EVT_COMMAND_ATTRIBUTESTOOLBAR_TOGGLE,
+        self.Bind(self.EVT_CMD_MACROTOOLBAR_TOGGLE,
+                  handlers.OnMacroToolbarToggle)
+        self.Bind(self.EVT_CMD_ATTRIBUTESTOOLBAR_TOGGLE,
                   handlers.OnAttributesToolbarToggle)
-        self.Bind(EVT_COMMAND_FIND_TOOLBAR_TOGGLE,
+        self.Bind(self.EVT_CMD_FIND_TOOLBAR_TOGGLE,
                   handlers.OnFindToolbarToggle)
-        self.Bind(EVT_COMMAND_ENTRYLINE_TOGGLE,
+        self.Bind(self.EVT_CMD_ENTRYLINE_TOGGLE,
                   handlers.OnEntryLineToggle)
-        self.Bind(EVT_COMMAND_TABLECHOICE_TOGGLE,
+        self.Bind(self.EVT_CMD_TABLECHOICE_TOGGLE,
                   handlers.OnTableChoiceToggle)
 
         # File events
 
-        self.Bind(EVT_COMMAND_NEW, handlers.OnNew)
-        self.Bind(EVT_COMMAND_OPEN, handlers.OnOpen)
-        self.Bind(EVT_COMMAND_SAVE, handlers.OnSave)
-        self.Bind(EVT_COMMAND_SAVEAS, handlers.OnSaveAs)
-        self.Bind(EVT_COMMAND_IMPORT, handlers.OnImport)
-        self.Bind(EVT_COMMAND_EXPORT, handlers.OnExport)
-        self.Bind(EVT_COMMAND_APPROVE, handlers.OnApprove)
+        self.Bind(self.EVT_CMD_NEW, handlers.OnNew)
+        self.Bind(self.EVT_CMD_OPEN, handlers.OnOpen)
+        self.Bind(self.EVT_CMD_SAVE, handlers.OnSave)
+        self.Bind(self.EVT_CMD_SAVEAS, handlers.OnSaveAs)
+        self.Bind(self.EVT_CMD_IMPORT, handlers.OnImport)
+        self.Bind(self.EVT_CMD_EXPORT, handlers.OnExport)
+        self.Bind(self.EVT_CMD_APPROVE, handlers.OnApprove)
+        self.Bind(self.EVT_CMD_CLEAR_GLOBALS, handlers.OnClearGlobals)
 
         # Find events
-        self.Bind(EVT_COMMAND_FOCUSFIND, handlers.OnFocusFind)
+        self.Bind(self.EVT_CMD_FOCUSFIND, handlers.OnFocusFind)
 
         # Format events
-        self.Bind(EVT_COMMAND_FONTDIALOG, handlers.OnFontDialog)
-        self.Bind(EVT_COMMAND_TEXTCOLORDIALOG, handlers.OnTextColorDialog)
-        self.Bind(EVT_COMMAND_BGCOLORDIALOG, handlers.OnBgColorDialog)
+        self.Bind(self.EVT_CMD_FONTDIALOG, handlers.OnFontDialog)
+        self.Bind(self.EVT_CMD_TEXTCOLORDIALOG, handlers.OnTextColorDialog)
+        self.Bind(self.EVT_CMD_BGCOLORDIALOG, handlers.OnBgColorDialog)
 
         # Print events
 
-        self.Bind(EVT_COMMAND_PAGE_SETUP, handlers.OnPageSetup)
-        self.Bind(EVT_COMMAND_PRINT_PREVIEW, handlers.OnPrintPreview)
-        self.Bind(EVT_COMMAND_PRINT, handlers.OnPrint)
+        self.Bind(self.EVT_CMD_PAGE_SETUP, handlers.OnPageSetup)
+        self.Bind(self.EVT_CMD_PRINT_PREVIEW, handlers.OnPrintPreview)
+        self.Bind(self.EVT_CMD_PRINT, handlers.OnPrint)
 
         # Clipboard events
 
-        self.Bind(EVT_COMMAND_CUT, handlers.OnCut)
-        self.Bind(EVT_COMMAND_COPY, handlers.OnCopy)
-        self.Bind(EVT_COMMAND_COPY_RESULT, handlers.OnCopyResult)
-        self.Bind(EVT_COMMAND_PASTE, handlers.OnPaste)
+        self.Bind(self.EVT_CMD_CUT, handlers.OnCut)
+        self.Bind(self.EVT_CMD_COPY, handlers.OnCopy)
+        self.Bind(self.EVT_CMD_COPY_RESULT, handlers.OnCopyResult)
+        self.Bind(self.EVT_CMD_PASTE, handlers.OnPaste)
 
         # Help events
 
-        self.Bind(EVT_COMMAND_MANUAL, handlers.OnManual)
-        self.Bind(EVT_COMMAND_TUTORIAL, handlers.OnTutorial)
-        self.Bind(EVT_COMMAND_FAQ, handlers.OnFaq)
-        self.Bind(EVT_COMMAND_PYTHON_TURORIAL, handlers.OnPythonTutorial)
-        self.Bind(EVT_COMMAND_ABOUT, handlers.OnAbout)
+        self.Bind(self.EVT_CMD_MANUAL, handlers.OnManual)
+        self.Bind(self.EVT_CMD_TUTORIAL, handlers.OnTutorial)
+        self.Bind(self.EVT_CMD_FAQ, handlers.OnFaq)
+        self.Bind(self.EVT_CMD_PYTHON_TURORIAL, handlers.OnPythonTutorial)
+        self.Bind(self.EVT_CMD_ABOUT, handlers.OnAbout)
 
-        self.Bind(EVT_COMMAND_MACROLIST, handlers.OnMacroList)
-        self.Bind(EVT_COMMAND_MACROREPLACE, handlers.OnMacroReplace)
-        self.Bind(EVT_COMMAND_MACROEXECUTE, handlers.OnMacroExecute)
-        self.Bind(EVT_COMMAND_MACROLOAD, handlers.OnMacroListLoad)
-        self.Bind(EVT_COMMAND_MACROSAVE, handlers.OnMacroListSave)
+        self.Bind(self.EVT_CMD_MACROLIST, handlers.OnMacroList)
+        self.Bind(self.EVT_CMD_MACROREPLACE, handlers.OnMacroReplace)
+        self.Bind(self.EVT_CMD_MACROEXECUTE, handlers.OnMacroExecute)
+        self.Bind(self.EVT_CMD_MACROLOAD, handlers.OnMacroListLoad)
+        self.Bind(self.EVT_CMD_MACROSAVE, handlers.OnMacroListSave)
 
     def set_icon(self, bmp):
         """Sets main window icon to given wx.Bitmap"""
@@ -360,12 +373,14 @@ class MainWindowEventHandlers(object):
             # Put * in front of title
             if title[:2] != "* ":
                 new_title = "* " + title
-                post_command_event(self.main_window, TitleMsg, text=new_title)
+                post_command_event(self.main_window, self.main_window.TitleMsg,
+                                   text=new_title)
 
         elif title[:2] == "* ":
             # Remove * in front of title
             new_title = title[2:]
-            post_command_event(self.main_window, TitleMsg, text=new_title)
+            post_command_event(self.main_window, self.main_window.TitleMsg,
+                               text=new_title)
 
     def OnTitle(self, event):
         """Title change event handler"""
@@ -408,7 +423,7 @@ class MainWindowEventHandlers(object):
 
             elif save_choice:
                 # User wants to save content
-                post_command_event(self.main_window, SaveMsg)
+                post_command_event(self.main_window, self.main_window.SaveMsg)
 
         # Save the AUI state
 
@@ -470,7 +485,7 @@ class MainWindowEventHandlers(object):
         self.main_window._mgr.Update()
 
     def OnMainToolbarToggle(self, event):
-        """Standard toolbar toggle event handler"""
+        """Main window toolbar toggle event handler"""
 
         main_toolbar = self.main_window._mgr.GetPane("main_window_toolbar")
 
@@ -478,6 +493,15 @@ class MainWindowEventHandlers(object):
 
         event.Skip()
 
+    def OnMacroToolbarToggle(self, event):
+        """Macro toolbar toggle event handler"""
+
+        macro_toolbar = self.main_window._mgr.GetPane("macro_toolbar")
+
+        self._toggle_pane(macro_toolbar)
+
+        event.Skip()
+
     def OnAttributesToolbarToggle(self, event):
         """Format toolbar toggle event handler"""
 
@@ -531,7 +555,7 @@ class MainWindowEventHandlers(object):
 
             elif save_choice:
                 # User wants to save content
-                post_command_event(self.main_window, SaveMsg)
+                post_command_event(self.main_window, self.main_window.SaveMsg)
 
         # Get grid dimensions
 
@@ -545,20 +569,27 @@ class MainWindowEventHandlers(object):
         # Set new filepath and post it to the title bar
 
         self.main_window.filepath = None
-        post_command_event(self.main_window, TitleMsg, text="pyspread")
+        post_command_event(self.main_window, self.main_window.TitleMsg,
+                           text="pyspread")
+
+        # Clear globals
+        self.main_window.grid.actions.clear_globals_reload_modules()
 
         # Create new grid
-        post_command_event(self.main_window, GridActionNewMsg, shape=shape)
+        post_command_event(self.main_window, self.main_window.GridActionNewMsg,
+                           shape=shape)
 
         # Update TableChoiceIntCtrl
-        post_command_event(self.main_window, ResizeGridMsg, shape=shape)
+        post_command_event(self.main_window, self.main_window.ResizeGridMsg,
+                           shape=shape)
 
         self.main_window.grid.GetTable().ResetView()
         self.main_window.grid.ForceRefresh()
 
         # Display grid creation in status bar
-        statustext = "New grid with dimensions " + str(shape) + " created."
-        post_command_event(self.main_window, StatusBarMsg, text=statustext)
+        statustext = _("New grid with dimensions {} created.").format(shape)
+        post_command_event(self.main_window, self.main_window.StatusBarMsg,
+                           text=statustext)
 
         self.main_window.grid.ForceRefresh()
 
@@ -576,17 +607,15 @@ class MainWindowEventHandlers(object):
 
             elif save_choice:
                 # User wants to save content
-                post_command_event(self.main_window, SaveMsg)
+                post_command_event(self.main_window, self.main_window.SaveMsg)
 
         # Get filepath from user
 
-        wildcard = "Pyspread file (*.pys)|*.pys|" \
-                   "All files (*.*)|*.*"
-        message = "Choose pyspread file to open."
+        wildcard = _("Pyspread file (*.pys)|*.pys|All files (*.*)|*.*")
+        message = _("Choose pyspread file to open.")
         style = wx.OPEN | wx.CHANGE_DIR
-        filepath, filterindex = \
-            self.interfaces.get_filepath_findex_from_user(wildcard,
-                                                          message, style)
+        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(
+                                    wildcard, message, style)
 
         if filepath is None:
             return
@@ -596,18 +625,15 @@ class MainWindowEventHandlers(object):
         self.main_window.filepath = filepath
 
         # Load file into grid
-        post_command_event(self.main_window, GridActionOpenMsg,
+        post_command_event(self.main_window,
+                           self.main_window.GridActionOpenMsg,
                            attr={"filepath": filepath})
 
         # Set Window title to new filepath
 
         title_text = filepath.split("/")[-1] + " - pyspread"
-        post_command_event(self.main_window, TitleMsg, text=title_text)
-
-        # Display file load in status bar
-
-        #statustext = "File " + filepath + " loaded."
-        #post_command_event(self.main_window, StatusBarMsg, text=statustext)
+        post_command_event(self.main_window,
+                           self.main_window.TitleMsg, text=title_text)
 
         self.main_window.grid.ForceRefresh()
 
@@ -617,82 +643,84 @@ class MainWindowEventHandlers(object):
         # If there is no filepath then jump to save as
 
         if self.main_window.filepath is None:
-            post_command_event(self.main_window, SaveAsMsg)
+            post_command_event(self.main_window,
+                               self.main_window.SaveAsMsg)
             return
 
         # Save the grid
 
-        post_command_event(self.main_window, GridActionSaveMsg,
+        post_command_event(self.main_window,
+                           self.main_window.GridActionSaveMsg,
                            attr={"filepath": self.main_window.filepath})
 
         # Display file save in status bar
 
         statustext = self.main_window.filepath.split("/")[-1] + " saved."
-        post_command_event(self.main_window, StatusBarMsg, text=statustext)
+        post_command_event(self.main_window,
+                           self.main_window.StatusBarMsg,
+                           text=statustext)
 
     def OnSaveAs(self, event):
         """File save as event handler"""
 
         # Get filepath from user
 
-        wildcard = "Pyspread file (*.pys)|*.pys|" \
-                   "All files (*.*)|*.*"
-        message = "Choose filename for saving."
+        wildcard = _("Pyspread file (*.pys)|*.pys|All files (*.*)|*.*")
+        message = _("Choose filename for saving.")
         style = wx.SAVE | wx.CHANGE_DIR
-        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(\
+        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(
                                     wildcard, message, style)
 
-        if filepath is not None:
+        if filepath is None:
+            return 0
 
-            # Look if path is already present
-            if os.path.exists(filepath):
-                if os.path.isfile(filepath):
-                    # There is a file with the same path
-                    message = "The file " + filepath + \
-                              " is already present.\nOverwrite?"
-                    short_message = "File collison"
-                    if not self.main_window.interfaces.get_warning_choice( \
-                                message, short_message):
+        # Look if path is already present
+        if os.path.exists(filepath):
+            if not os.path.isfile(filepath):
+                # There is a directory with the same path
+                statustext = _("Directory present. Save aborted.")
+                post_command_event(self.main_window,
+                                   self.main_window.StatusBarMsg,
+                                   text=statustext)
+                return 0
 
-                        statustext = "File present. Save aborted by user."
-                        post_command_event(self.main_window, StatusBarMsg,
-                                       text=statustext)
-                        return 0
-                else:
-                    # There is a directory with the same path
-                    statustext = "Directory present. Save aborted."
-                    post_command_event(self.main_window, StatusBarMsg,
-                                       text=statustext)
-                    return 0
+            # There is a file with the same path
+            message = _("The file {filepath} is already present.\nOverwrite?")\
+                                  .format(filepath=filepath)
+            short_message = _("File collision")
+            if not self.main_window.interfaces.get_warning_choice(
+                        message, short_message):
 
-            # Put pys suffix if wildcard choice is 0
+                statustext = _("File present. Save aborted by user.")
+                post_command_event(self.main_window,
+                                   self.main_window.StatusBarMsg,
+                                   text=statustext)
+                return 0
 
-            if filterindex == 0 and filepath[-4:] != ".pys":
-                filepath += ".pys"
+        # Put pys suffix if wildcard choice is 0
+        if filterindex == 0 and filepath[-4:] != ".pys":
+            filepath += ".pys"
 
-            # Set the filepath state
+        # Set the filepath state
+        self.main_window.filepath = filepath
 
-            self.main_window.filepath = filepath
+        # Set Window title to new filepath
+        title_text = filepath.split("/")[-1] + " - pyspread"
+        post_command_event(self.main_window, self.main_window.TitleMsg,
+                           text=title_text)
 
-            # Set Window title to new filepath
-
-            title_text = filepath.split("/")[-1] + " - pyspread"
-            post_command_event(self.main_window, TitleMsg, text=title_text)
-
-            # Now jump to save
-
-            post_command_event(self.main_window, SaveMsg)
+        # Now jump to save
+        post_command_event(self.main_window, self.main_window.SaveMsg)
 
     def OnImport(self, event):
         """File import event handler"""
 
         # Get filepath from user
 
-        wildcard = wildcard="Csv file (*.*)|*.*|" \
-                            "Tab delimited text file (*.*)|*.*"
-        message = "Choose file to import."
+        wildcard = _("Csv file (*.*)|*.*|Tab delimited text file (*.*)|*.*")
+        message = _("Choose file to import.")
         style = wx.OPEN | wx.CHANGE_DIR
-        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(\
+        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(
                                     wildcard, message, style)
 
         if filepath is None:
@@ -742,10 +770,10 @@ class MainWindowEventHandlers(object):
 
         # Get target filepath from user
 
-        wildcard = wildcard = " CSV file (*.*)|*.*"
-        message = "Choose filename for export."
+        wildcard = _("CSV file (*.*)|*.*")
+        message = _("Choose filename for export.")
         style = wx.OPEN | wx.CHANGE_DIR
-        path, filterindex = self.interfaces.get_filepath_findex_from_user( \
+        path, filterindex = self.interfaces.get_filepath_findex_from_user(
                                     wildcard, message, style)
 
         # Export file
@@ -759,18 +787,18 @@ class MainWindowEventHandlers(object):
         if not self.main_window.safe_mode:
             return
 
-        msg = u"You are going to approve and trust a file that\n" + \
-              u"you have received from an untrusted source.\n" + \
-              u"After proceeding, the file is executed.\n" + \
-              u"It can harm your system as any program can.\n" + \
-              u"Unless you took precautions, it can delete your\n" + \
-              u"files or send them away over the Internet.\n" + \
-              u"CHECK EACH CELL BEFORE PROCEEDING.\n \n" + \
-              u"Do not forget cells outside the visible range.\n" + \
-              u"You have been warned.\n \n" + \
-              u"Proceed and sign this file as trusted?"
+        msg = _(u"You are going to approve and trust a file that\n"
+                u"you have received from an untrusted source.\n"
+                u"After proceeding, the file is executed.\n"
+                u"It can harm your system as any program can.\n"
+                u"Unless you took precautions, it can delete your\n"
+                u"files or send them away over the Internet.\n"
+                u"CHECK EACH CELL BEFORE PROCEEDING.\n \n"
+                u"Do not forget cells outside the visible range.\n"
+                u"You have been warned.\n \n"
+                u"Proceed and sign this file as trusted?")
 
-        short_msg = "Security warning"
+        short_msg = _("Security warning")
 
         if self.main_window.interfaces.get_warning_choice(msg, short_msg):
             # Leave safe mode
@@ -778,8 +806,25 @@ class MainWindowEventHandlers(object):
 
             # Display safe mode end in status bar
 
-            statustext = "Safe mode deactivated."
-            post_command_event(self.main_window, StatusBarMsg, text=statustext)
+            statustext = _("Safe mode deactivated.")
+            post_command_event(self.main_window, self.main_window.StatusBarMsg,
+                               text=statustext)
+
+    def OnClearGlobals(self, event):
+        """Clear globals event handler"""
+
+        msg = _("Deleting globals and reloading modules cannot be undone."
+                    " Proceed?")
+        short_msg = _("Really delete globals and modules?")
+
+        choice = self.main_window.interfaces.get_warning_choice(msg, short_msg)
+
+        if choice:
+            self.main_window.grid.actions.clear_globals_reload_modules()
+
+            statustext = _("Globals cleared and base modules reloaded.")
+            post_command_event(self.main_window, self.main_window.StatusBarMsg,
+                               text=statustext)
 
     # Print events
 
@@ -835,36 +880,55 @@ class MainWindowEventHandlers(object):
     def OnCopy(self, event):
         """Clipboard copy event handler"""
 
-        data = self.main_window.actions.copy(self.main_window.grid.selection)
-        self.main_window.clipboard.set_clipboard(data)
+        focus = self.main_window.FindFocus()
+
+        if isinstance(focus, wx.TextCtrl):
+            # Copy selection from TextCtrl if in focus
+            focus.Copy()
+
+        else:
+            data = self.main_window.actions.copy(
+                       self.main_window.grid.selection)
+            self.main_window.clipboard.set_clipboard(data)
 
         event.Skip()
 
     def OnCopyResult(self, event):
         """Clipboard copy results event handler"""
 
-        data = self.main_window.actions.copy_result( \
+        data = self.main_window.actions.copy_result(
                             self.main_window.grid.selection)
-        self.main_window.clipboard.set_clipboard(data)
+
+        # Check if result is a bitmap
+        if type(data) is wx._gdi.Bitmap:
+            # Copy bitmap to clipboard
+            self.main_window.clipboard.set_clipboard(data, datatype="bitmap")
+
+        else:
+            # Copy string representation of result to clipboard
+
+            self.main_window.clipboard.set_clipboard(data, datatype="text")
 
         event.Skip()
 
     def OnPaste(self, event):
         """Clipboard paste event handler"""
 
+        data = self.main_window.clipboard.get_clipboard()
+
         focus = self.main_window.FindFocus()
 
         if isinstance(focus, wx.TextCtrl):
-            pass
+            # Paste into TextCtrl if in focus
+            focus.WriteText(data)
 
-        else:  # We got a grid selection
+        else:
+            # We got a grid selection
             grid = self.main_window.grid
-            key = (grid.GetGridCursorRow(), \
-                   grid.GetGridCursorCol(), \
+            key = (grid.GetGridCursorRow(),
+                   grid.GetGridCursorCol(),
                    grid.current_table)
 
-            data = self.main_window.clipboard.get_clipboard()
-
             self.main_window.actions.paste(key, data)
 
         self.main_window.grid.ForceRefresh()
@@ -887,9 +951,9 @@ class MainWindowEventHandlers(object):
         cursor = self.main_window.grid.actions.cursor
         attr = self.main_window.grid.code_array.cell_attributes[cursor]
 
-        current_font = wx.Font( \
-            int(attr["pointsize"]), -1,
-            attr["fontstyle"], attr["fontweight"], 0, attr["textfont"])
+        size, style, weight, font = [attr[name] for name in
+            ["pointsize", "fontstyle", "fontweight", "textfont"]]
+        current_font = wx.Font(int(size), -1, style, weight, 0, font)
 
         # Get Font from dialog
 
@@ -903,13 +967,15 @@ class MainWindowEventHandlers(object):
             fontdata = dlg.GetFontData()
             font = fontdata.GetChosenFont()
 
-            post_command_event(self.main_window, FontMsg, font=font.FaceName)
-            post_command_event(self.main_window, FontSizeMsg,
+            post_command_event(self.main_window, self.main_window.FontMsg,
+                               font=font.FaceName)
+            post_command_event(self.main_window, self.main_window.FontSizeMsg,
                                size=font.GetPointSize())
 
-            post_command_event(self.main_window, FontBoldMsg,
+            post_command_event(self.main_window, self.main_window.FontBoldMsg,
                                weight=font.GetWeightString())
-            post_command_event(self.main_window, FontItalicsMsg,
+            post_command_event(self.main_window,
+                               self.main_window.FontItalicsMsg,
                                style=font.GetStyleString())
 
     def OnTextColorDialog(self, event):
@@ -927,7 +993,8 @@ class MainWindowEventHandlers(object):
             data = dlg.GetColourData()
             color = data.GetColour().GetRGB()
 
-            post_command_event(self.main_window, TextColorMsg, color=color)
+            post_command_event(self.main_window, self.main_window.TextColorMsg,
+                               color=color)
 
         dlg.Destroy()
 
@@ -946,7 +1013,8 @@ class MainWindowEventHandlers(object):
             data = dlg.GetColourData()
             color = data.GetColour().GetRGB()
 
-            post_command_event(self.main_window, BackgroundColorMsg,
+            post_command_event(self.main_window,
+                               self.main_window.BackgroundColorMsg,
                                color=color)
 
         dlg.Destroy()
@@ -975,12 +1043,11 @@ class MainWindowEventHandlers(object):
 
         # Get filepath from user
 
-        wildcard = "Macro file (*.py)|*.py|" \
-                   "All files (*.*)|*.*"
-        message = "Choose macro file."
+        wildcard = _("Macro file (*.py)|*.py|All files (*.*)|*.*")
+        message = _("Choose macro file.")
 
         style = wx.OPEN | wx.CHANGE_DIR
-        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(\
+        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(
                                     wildcard, message, style)
 
         if filepath is None:
@@ -988,7 +1055,7 @@ class MainWindowEventHandlers(object):
 
         # Enter safe mode because macro file could be harmful
 
-        post_command_event(self.main_window, SafeModeEntryMsg)
+        post_command_event(self.main_window, self.main_window.SafeModeEntryMsg)
 
         # Load macros from file
 
@@ -1001,12 +1068,11 @@ class MainWindowEventHandlers(object):
 
         # Get filepath from user
 
-        wildcard = "Macro file (*.py)|*.py|" \
-                   "All files (*.*)|*.*"
-        message = "Choose macro file."
+        wildcard = _("Macro file (*.py)|*.py|All files (*.*)|*.*")
+        message = _("Choose macro file.")
 
         style = wx.SAVE | wx.CHANGE_DIR
-        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(\
+        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(
                                     wildcard, message, style)
 
         # Save macros to file
