@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright 2011 Martin Manns
+# Copyright Martin Manns
 # Distributed under the terms of the GNU General Public License
 
 # --------------------------------------------------------------------
@@ -32,10 +32,14 @@ Provides:
   3. AttributesToolbar: Toolbar for editing cell attributes
 
 """
+
 import wx
 import wx.lib.colourselect as csel
+import wx.lib.agw.aui as aui
 
-from _events import *
+from _events import post_command_event, EventMixin
+
+import src.lib.i18n as i18n
 
 from src.config import config
 from src.sysvars import get_default_font, get_font_list
@@ -43,81 +47,81 @@ from icons import icons
 
 import _widgets
 
-class MainToolbar(wx.ToolBar):
-    """Main application toolbar, built from attribute toolbardata
+#use ugettext instead of getttext to avoid unicode errors
+_ = i18n.language.ugettext
 
-    toolbardata has the following structure:
-    [[toolobject, "Methodname", "Label",
-                  "Iconname", "Tooltip", "Help string"] , \
-    ...
-    ["Separator"] ,\
-    ...
-    ]
+
+class ToolbarBase(aui.AuiToolBar, EventMixin):
+    """Base class for toolbars, requires self.toolbardata set in init
+
+    Toolbardata has the following structure:
+    [["tool type", "method name", "label", "tool tip"], ... ]
+
+    Tool types are:
+
+    "T": Simple tool
+    "S": Separator
+    "C": Control
+    "O": Check tool / option button
 
     """
 
-    tool = wx.ITEM_NORMAL
+    def __init__(self, parent, *args, **kwargs):
 
-    toolbardata = [
-    [tool, NewMsg, "New", "FileNew", "New spreadsheet",
-        "Create a new, empty spreadsheet"], \
-    [tool, OpenMsg, "Open", "FileOpen", "Open spreadsheet",
-        "Open spreadsheet from file"], \
-    [tool, SaveMsg, "Save", "FileSave", "Save spreadsheet",
-        "Save spreadsheet to file"], \
-    ["Separator"] , \
-    [tool, UndoMsg, "Undo", "Undo", "Undo", "Undo last operation"], \
-    [tool, RedoMsg, "Redo", "Redo", "Redo", "Redo next operation"], \
-    ["Separator"] , \
-    [tool, FindFocusMsg, "Find", "Find", "Find", "Find cell by content"], \
-    [tool, ReplaceMsg, "Replace", "FindReplace", "Replace",
-        "Replace strings in cells"], \
-    ["Separator"] , \
-    [tool, CutMsg, "Cut", "EditCut", "Cut", "Cut cells to clipboard"], \
-    [tool, CopyMsg, "Copy", "EditCopy", "Copy",
-        "Copy the input strings of the cells to clipboard"], \
-    [tool, CopyResultMsg, "Copy Results", "EditCopyRes", "Copy Results",
-        "Copy the result strings of the cells to the clipboard"], \
-    [tool, PasteMsg, "Paste", "EditPaste", "Paste",
-        "Paste cell from clipboard"], \
-    ["Separator"] , \
-    [tool, PrintMsg, "Print", "FilePrint", "Print current spreadsheet",
-        "Print current spreadsheet"], \
-    ]
-
-    def __init__(self, *args, **kwargs):
-        wx.ToolBar.__init__(self, *args, **kwargs)
+        aui.AuiToolBar.__init__(self, parent, *args, **kwargs)
 
         self.SetToolBitmapSize(icons.icon_size)
 
         self.ids_msgs = {}
+        self.label2id = {}
 
-        self.parent = args[0]
-        self._add_tools()
+        self.parent = parent
 
-    def _add_tools(self):
+    def add_tools(self):
         """Adds tools from self.toolbardata to self"""
 
-        for tool in self.toolbardata:
-            obj = tool[0]
-            if obj == "Separator":
+        for data in self.toolbardata:
+            # tool type is in data[0]
+
+            if data[0] == "T":
+                # Simple tool
+
+                _, msg_type, label, tool_tip = data
+                icon = icons[label]
+
+                self.label2id[label] = tool_id = wx.NewId()
+
+                self.AddSimpleTool(tool_id, label, icon,
+                                   short_help_string=tool_tip)
+
+                self.ids_msgs[tool_id] = msg_type
+
+                self.parent.Bind(wx.EVT_TOOL, self.OnTool, id=tool_id)
+
+            elif data[0] == "S":
+                # Separator
+
                 self.AddSeparator()
-            elif obj == self.tool:
-                msgtype = tool[1]
-                label = tool[2]
-                icon = icons[tool[3]]
-                icon2 = wx.NullBitmap
-                tooltip = tool[4]
-                helpstring = tool[5]
-                toolid = wx.NewId()
-                self.AddLabelTool(toolid, label, icon, icon2, obj,
-                                  tooltip, helpstring)\
 
-                self.ids_msgs[toolid] = msgtype
+            elif data[0] == "C":
+                # Control
+
+                _, control, tool_tip = data
+
+                self.AddControl(control, label=tool_tip)
+
+            elif data[0] == "O":
+                # Check tool / option button
+
+                _, label, tool_tip = data
+                icon = icons[label]
+
+                self.label2id[label] = tool_id = wx.NewId()
+
+                self.AddCheckTool(tool_id, label, icon, icon, tool_tip)
 
-                self.parent.Bind(wx.EVT_TOOL, self.OnTool, id=toolid)
             else:
-                raise TypeError, "Toolbar item unknown"
+                raise ValueError("Unknown tooltype " + str(data[0]))
 
     def OnTool(self, event):
         """Toolbar event handler"""
@@ -125,92 +129,116 @@ class MainToolbar(wx.ToolBar):
         msgtype = self.ids_msgs[event.GetId()]
         post_command_event(self, msgtype)
 
+
+class MainToolbar(ToolbarBase):
+    """Main application toolbar, built from attribute toolbardata"""
+
+    def __init__(self, parent, *args, **kwargs):
+
+        ToolbarBase.__init__(self, parent, *args, **kwargs)
+
+        self.toolbardata = [
+            ["T", self.NewMsg, "FileNew", _("New")],
+            ["T", self.OpenMsg, "FileOpen", _("Open")],
+            ["T", self.SaveMsg, "FileSave", _("Save")],
+            ["S"],
+            ["T", self.UndoMsg, "Undo", _("Undo")],
+            ["T", self.RedoMsg, "Redo", _("Redo")],
+            ["S"],
+            ["T", self.FindFocusMsg, "Find", _("Find")],
+            ["T", self.ReplaceMsg, "FindReplace", _("Replace")],
+            ["S"],
+            ["T", self.CutMsg, "EditCut", _("Cut")],
+            ["T", self.CopyMsg, "EditCopy", _("Copy")],
+            ["T", self.CopyResultMsg, "EditCopyRes", _("Copy Results")],
+            ["T", self.PasteMsg, "EditPaste", _("Paste")],
+            ["S"],
+            ["T", self.PrintMsg, "FilePrint", _("Print")],
+        ]
+
+        self.add_tools()
+
+        self.Realize()
+
 # end of class MainToolbar
 
 
-class FindToolbar(wx.ToolBar):
-    """Toolbar for find operations (replaces wxFindReplaceDialog)"""
-
-    # Search flag buttons
-    search_options_buttons = { \
-      "matchcase_tb": { \
-        "ID": wx.NewId(),
-        "iconname": "SearchCaseSensitive",
-        "shorthelp": "Case sensitive",
-        "longhelp": "Case sensitive search",
-        "flag": "MATCH_CASE",
-      },
-      "regexp_tb": {
-        "ID": wx.NewId(),
-        "iconname": "SearchRegexp",
-        "shorthelp": "Regular expression",
-        "longhelp": "Treat search string as regular expression",
-        "flag": "REG_EXP",
-      },
-      "wholeword_tb": { \
-        "ID": wx.NewId(),
-        "iconname": "SearchWholeword",
-        "shorthelp": "Whole word",
-        "longhelp": "Search string is surronted by whitespace characters",
-        "flag": "WHOLE_WORD",
-      },
-    }
+class MacroToolbar(ToolbarBase):
+    """Macro toolbar, built from attribute toolbardata"""
 
     def __init__(self, parent, *args, **kwargs):
-        kwargs["style"] = wx.TB_FLAT | wx.TB_NODIVIDER
-        wx.ToolBar.__init__(self, parent, *args, **kwargs)
 
-        self.SetToolBitmapSize(icons.icon_size)
+        ToolbarBase.__init__(self, parent, *args, **kwargs)
 
-        self.parent = parent
+        self.toolbardata = [
+            ["T", self.InsertBitmapMsg, "InsertBitmap", _("Insert bitmap")],
+            ["T", self.LinkBitmapMsg, "LinkBitmap", _("Link bitmap")],
+            ["T", self.InsertChartMsg, "InsertChart", _("Insert chart")],
+        ]
+
+        self.add_tools()
+
+        self.Realize()
+
+# end of class MainToolbar
+
+
+class FindToolbar(ToolbarBase):
+    """Toolbar for find operations (replaces wxFindReplaceDialog)"""
+
+    def __init__(self, parent, *args, **kwargs):
+
+        ToolbarBase.__init__(self, parent, *args, **kwargs)
+
+        self.search_history = []
+        self.search_options = ["DOWN"]
+        self.search_options_buttons = ["MATCH_CASE", "REG_EXP", "WHOLE_WORD"]
+
+        # Controls
+        # --------
 
         # Search entry control
-        self.search_history = []
-        self.search = wx.SearchCtrl(self, size=(150, -1), \
+        search_tooltip = _("Find in code and results")
+        self.search = wx.SearchCtrl(self, size=(140, -1),
                         style=wx.TE_PROCESS_ENTER | wx.NO_BORDER)
-        self.search.SetToolTip(wx.ToolTip("Enter search string for " + \
-                                "searching in the grid cell source code"))
+        self.search.SetToolTip(wx.ToolTip(search_tooltip))
         self.menu = self.make_menu()
         self.search.SetMenu(self.menu)
-        self.AddControl(self.search)
 
-        # Search direction toggle button
-        self.search_options = ["DOWN"]
-        self.setup_searchdirection_togglebutton()
+        # Search direction togglebutton
+        direction_tooltip = _("Search direction")
+        iconnames = ["GoDown", "GoUp"]
+        bmplist = [icons[iconname] for iconname in iconnames]
+        self.search_direction_tb = _widgets.BitmapToggleButton(self, bmplist)
+        self.search_direction_tb.SetToolTip(wx.ToolTip(direction_tooltip))
 
-        # Search flags buttons
-        sfbs = self.search_options_buttons
-        for name in sfbs:
-            iconname = sfbs[name]["iconname"]
-            __id = sfbs[name]["ID"]
-            shorthelp = sfbs[name]["shorthelp"]
-            longhelp = sfbs[name]["longhelp"]
+        # Toolbar data
+        # ------------
 
-            bmp = icons[iconname]
-            self.AddCheckLabelTool(__id, name, bmp,
-                shortHelp=shorthelp, longHelp=longhelp)
+        self.toolbardata = [
+            ["C", self.search, search_tooltip],
+            ["C", self.search_direction_tb, direction_tooltip],
+            ["O", "MATCH_CASE", _("Case sensitive")],
+            ["O", "REG_EXP", _("Regular expression")],
+            ["O", "WHOLE_WORD", _("Surrounded by whitespace")],
+        ]
 
-        # Event bindings
+        self.add_tools()
+
+        # Bindings and polish
+        # -------------------
+
+        self._bindings()
+
+        self.Realize()
+
+    def _bindings(self):
         self.Bind(wx.EVT_SEARCHCTRL_SEARCH_BTN, self.OnSearch, self.search)
         self.Bind(wx.EVT_TEXT_ENTER, self.OnSearch, self.search)
         self.Bind(wx.EVT_MENU_RANGE, self.OnSearchFlag)
         self.Bind(wx.EVT_BUTTON, self.OnSearchDirectionButton,
                                  self.search_direction_tb)
         self.Bind(wx.EVT_MENU, self.OnMenu)
-        self.Realize()
-
-    def setup_searchdirection_togglebutton(self):
-        """Setup of search direction toggle button for searching up and down"""
-
-        iconnames = ["SearchDirectionDown", "SearchDirectionUp"]
-        bmplist = [icons[iconname] for iconname in iconnames]
-        self.search_direction_tb = _widgets.BitmapToggleButton(self, bmplist)
-
-        self.search_direction_tb.SetInitialSize()
-        self.search_direction_tb.SetToolTip( \
-            wx.ToolTip("Search direction"))
-        self.AddControl(self.search_direction_tb)
-
 
     def make_menu(self):
         """Creates the search menu"""
@@ -250,8 +278,8 @@ class FindToolbar(wx.ToolBar):
 
         search_flags = self.search_options + ["FIND_NEXT"]
 
-        post_command_event(self, FindMsg, text=search_string,
-                                          flags=search_flags)
+        post_command_event(self, self.FindMsg, text=search_string,
+                           flags=search_flags)
 
         self.search.SetFocus()
 
@@ -265,26 +293,28 @@ class FindToolbar(wx.ToolBar):
             flag_index = self.search_options.index("UP")
             self.search_options[flag_index] = "DOWN"
         else:
-            raise AttributeError, "Neither UP nor DOWN in search_flags"
+            raise AttributeError(_("Neither UP nor DOWN in search_flags"))
+
         event.Skip()
 
     def OnSearchFlag(self, event):
         """Event handler for search flag toggle buttons"""
 
-        sfbs = self.search_options_buttons
-        for name in sfbs:
-            if sfbs[name]["ID"] == event.GetId():
+        for label in self.search_options_buttons:
+            button_id = self.label2id[label]
+            if button_id == event.GetId():
                 if event.IsChecked():
-                    self.search_options.append(sfbs[name]["flag"])
+                    self.search_options.append(label)
                 else:
-                    flag_index = self.search_options.index(sfbs[name]["flag"])
+                    flag_index = self.search_options.index(label)
                     self.search_options.pop(flag_index)
+
         event.Skip()
 
 # end of class FindToolbar
 
 
-class AttributesToolbar(wx.ToolBar):
+class AttributesToolbar(aui.AuiToolBar, EventMixin):
     """Toolbar for editing cell attributes
 
     Class attributes
@@ -296,7 +326,7 @@ class AttributesToolbar(wx.ToolBar):
 
     """
 
-    border_toggles = [ \
+    border_toggles = [
         ("AllBorders",       (1, 1, 1, 1, 1, 1)),
         ("LeftBorders",      (0, 0, 1, 0, 1, 1)),
         ("RightBorders",     (0, 0, 0, 1, 1, 1)),
@@ -307,20 +337,20 @@ class AttributesToolbar(wx.ToolBar):
         ("TopBottomBorders", (1, 1, 0, 0, 0, 1)),
     ]
 
-    bordermap = { \
-        "AllBorders":      ("top", "bottom", "left", "right", "inner"),
-        "LeftBorders":     ("left"),
-        "RightBorders":    ("right"),
-        "TopBorders":      ("top"),
-        "BottomBorders":   ("bottom"),
-        ##"InsideBorders":   ("inner"),
-        "OutsideBorders":  ("top", "bottom", "left", "right"),
-        "TopBottomBorders":("top", "bottom"),
+    bordermap = {
+        "AllBorders":       ("top", "bottom", "left", "right", "inner"),
+        "LeftBorders":      ("left"),
+        "RightBorders":     ("right"),
+        "TopBorders":       ("top"),
+        "BottomBorders":    ("bottom"),
+        ##"InsideBorders":    ("inner"),
+        "OutsideBorders":   ("top", "bottom", "left", "right"),
+        "TopBottomBorders": ("top", "bottom"),
     }
 
     def __init__(self, parent, *args, **kwargs):
         kwargs["style"] = wx.TB_FLAT | wx.TB_NODIVIDER
-        wx.ToolBar.__init__(self, parent, *args, **kwargs)
+        aui.AuiToolBar.__init__(self, parent, *args, **kwargs)
 
         self.parent = parent
 
@@ -334,6 +364,7 @@ class AttributesToolbar(wx.ToolBar):
         self._create_borderchoice_combo()
         self._create_penwidth_combo()
         self._create_color_buttons()
+        self._create_merge_button()
         self._create_textrotation_spinctrl()
 
         self.Realize()
@@ -345,13 +376,13 @@ class AttributesToolbar(wx.ToolBar):
         """Creates font choice combo box"""
 
         self.fonts = get_font_list()
-        self.font_choice_combo = _widgets.FontChoiceCombobox(self, \
+        self.font_choice_combo = _widgets.FontChoiceCombobox(self,
                                     choices=self.fonts, style=wx.CB_READONLY,
                                     size=(125, -1))
         self.AddControl(self.font_choice_combo)
 
         self.Bind(wx.EVT_COMBOBOX, self.OnTextFont, self.font_choice_combo)
-        self.parent.Bind(EVT_COMMAND_TOOLBAR_UPDATE, self.OnUpdate)
+        self.parent.Bind(self.EVT_CMD_TOOLBAR_UPDATE, self.OnUpdate)
 
     def _create_font_size_combo(self):
         """Creates font size combo box"""
@@ -369,18 +400,20 @@ class AttributesToolbar(wx.ToolBar):
         """Creates font face buttons"""
 
         font_face_buttons = [
-            (wx.FONTFLAG_BOLD, "OnBold", "FormatTextBold", "Bold"),
-            (wx.FONTFLAG_ITALIC, "OnItalics", "FormatTextItalic", "Italics"),
+            (wx.FONTFLAG_BOLD, "OnBold", "FormatTextBold", _("Bold")),
+            (wx.FONTFLAG_ITALIC, "OnItalics", "FormatTextItalic",
+                 _("Italics")),
             (wx.FONTFLAG_UNDERLINED, "OnUnderline", "FormatTextUnderline",
-                "Underline"),
+                _("Underline")),
             (wx.FONTFLAG_STRIKETHROUGH, "OnStrikethrough",
-                "FormatTextStrikethrough", "Strikethrough"),
-            (wx.FONTFLAG_MASK, "OnFreeze", "Freeze", "Freeze"),
-            ]
+                "FormatTextStrikethrough", _("Strikethrough")),
+            (wx.FONTFLAG_MASK, "OnFreeze", "Freeze", _("Freeze")),
+        ]
 
         for __id, method, iconname, helpstring in font_face_buttons:
             bmp = icons[iconname]
-            self.AddCheckLabelTool(__id, "", bmp, shortHelp=helpstring)
+            self.AddCheckTool(__id, iconname, bmp, bmp,
+                              short_help_string=helpstring)
             self.Bind(wx.EVT_TOOL, getattr(self, method), id=__id)
 
     def _create_justification_button(self):
@@ -405,28 +438,29 @@ class AttributesToolbar(wx.ToolBar):
     def _create_borderchoice_combo(self):
         """Create border choice combo box"""
 
+        choices = [c[0] for c in self.border_toggles]
         self.borderchoice_combo = _widgets.BorderEditChoice(self,
-                                choices=[c[0] for c in self.border_toggles], \
-                                style=wx.CB_READONLY, size=(50, -1))
+                 choices=choices, style=wx.CB_READONLY, size=(50, -1))
 
         self.borderstate = self.border_toggles[0][0]
 
         self.AddControl(self.borderchoice_combo)
 
-        self.Bind(wx.EVT_COMBOBOX, self.OnBorderChoice, self.borderchoice_combo)
+        self.Bind(wx.EVT_COMBOBOX, self.OnBorderChoice,
+                  self.borderchoice_combo)
 
         self.borderchoice_combo.SetValue("AllBorders")
 
     def _create_penwidth_combo(self):
         """Create pen width combo box"""
 
+        choices = map(unicode, xrange(12))
         self.pen_width_combo = _widgets.PenWidthComboBox(self,
-                                choices=map(unicode, xrange(12)), \
-                                style=wx.CB_READONLY, size=(50, -1))
+                choices=choices, style=wx.CB_READONLY, size=(50, -1))
+
         self.AddControl(self.pen_width_combo)
         self.Bind(wx.EVT_COMBOBOX, self.OnLineWidth, self.pen_width_combo)
 
-
     def _create_color_buttons(self):
         """Create color choice buttons"""
 
@@ -458,6 +492,15 @@ class AttributesToolbar(wx.ToolBar):
         self.bgcolor_choice.Bind(csel.EVT_COLOURSELECT, self.OnBGColor)
         self.textcolor_choice.Bind(csel.EVT_COLOURSELECT, self.OnTextColor)
 
+    def _create_merge_button(self):
+        """Create merge button"""
+
+        bmp = icons["Merge"]
+        self.mergetool_id = wx.NewId()
+        self.AddCheckTool(self.mergetool_id, "Merge", bmp, bmp,
+                               short_help_string=_("Merge cells"))
+        self.Bind(wx.EVT_TOOL, self.OnMerge, id=self.mergetool_id)
+
     def _create_textrotation_spinctrl(self):
         """Create text rotation spin control"""
 
@@ -629,6 +672,11 @@ class AttributesToolbar(wx.ToolBar):
 
         self.textcolor_choice.SetColour(textcolor)
 
+    def _update_merge(self, merged):
+        """Updates cell merge toggle control"""
+
+        self.ToggleTool(self.mergetool_id, merged)
+
     def _update_textrotation(self, angle):
         """Updates text rotation spin control"""
 
@@ -655,7 +703,6 @@ class AttributesToolbar(wx.ToolBar):
 
         self.pen_width_combo.SetSelection(borderwidth)
 
-
     # Attributes toolbar event handlers
     # ---------------------------------
 
@@ -676,12 +723,12 @@ class AttributesToolbar(wx.ToolBar):
         self._update_justification(attributes["justification"])
         self._update_alignment(attributes["vertical_align"])
         self._update_fontcolor(attributes["textcolor"])
+        self._update_merge(attributes["merge_area"] is not None)
         self._update_textrotation(attributes["angle"])
         self._update_bgbrush(attributes["bgcolor"])
         self._update_bordercolor(attributes["bordercolor_bottom"])
         self._update_borderwidth(attributes["borderwidth_bottom"])
 
-
     def OnBorderChoice(self, event):
         """Change the borders that are affected by color and width changes"""
 
@@ -694,31 +741,33 @@ class AttributesToolbar(wx.ToolBar):
         color = event.GetValue().GetRGB()
         borders = self.bordermap[self.borderstate]
 
-        post_command_event(self, BorderColorMsg, color=color, borders=borders)
+        post_command_event(self, self.BorderColorMsg, color=color,
+                           borders=borders)
 
     def OnLineWidth(self, event):
         """Line width choice event handler"""
 
         linewidth_combobox = event.GetEventObject()
         idx = event.GetInt()
-        width  = int(linewidth_combobox.GetString(idx))
+        width = int(linewidth_combobox.GetString(idx))
         borders = self.bordermap[self.borderstate]
 
-        post_command_event(self, BorderWidthMsg, width=width, borders=borders)
+        post_command_event(self, self.BorderWidthMsg, width=width,
+                           borders=borders)
 
     def OnBGColor(self, event):
         """Background color choice event handler"""
 
         color = event.GetValue().GetRGB()
 
-        post_command_event(self, BackgroundColorMsg, color=color)
+        post_command_event(self, self.BackgroundColorMsg, color=color)
 
     def OnTextColor(self, event):
         """Text color choice event handler"""
 
         color = event.GetValue().GetRGB()
 
-        post_command_event(self, TextColorMsg, color=color)
+        post_command_event(self, self.TextColorMsg, color=color)
 
     def OnTextFont(self, event):
         """Text font choice event handler"""
@@ -727,11 +776,11 @@ class AttributesToolbar(wx.ToolBar):
         idx = event.GetInt()
 
         try:
-            font_string  = fontchoice_combobox.GetString(idx)
+            font_string = fontchoice_combobox.GetString(idx)
         except AttributeError:
             font_string = event.GetString()
 
-        post_command_event(self, FontMsg, font=font_string)
+        post_command_event(self, self.FontMsg, font=font_string)
 
     def OnTextSize(self, event):
         """Text size combo text event handler"""
@@ -742,48 +791,53 @@ class AttributesToolbar(wx.ToolBar):
         except Exception:
             size = get_default_font().GetPointSize()
 
-        post_command_event(self, FontSizeMsg, size=size)
+        post_command_event(self, self.FontSizeMsg, size=size)
 
     def OnBold(self, event):
         """Bold toggle button event handler"""
 
-        post_command_event(self, FontBoldMsg)
+        post_command_event(self, self.FontBoldMsg)
 
     def OnItalics(self, event):
         """Bold toggle button event handler"""
 
-        post_command_event(self, FontItalicsMsg)
+        post_command_event(self, self.FontItalicsMsg)
 
     def OnUnderline(self, event):
         """Bold toggle button event handler"""
 
-        post_command_event(self, FontUnderlineMsg)
+        post_command_event(self, self.FontUnderlineMsg)
 
     def OnStrikethrough(self, event):
         """Bold toggle button event handler"""
 
-        post_command_event(self, FontStrikethroughMsg)
+        post_command_event(self, self.FontStrikethroughMsg)
 
     def OnFreeze(self, event):
         """Bold toggle button event handler"""
 
-        post_command_event(self, FrozenMsg)
+        post_command_event(self, self.FrozenMsg)
 
     def OnJustification(self, event):
         """Justification toggle button event handler"""
 
-        post_command_event(self, JustificationMsg)
+        post_command_event(self, self.JustificationMsg)
 
     def OnAlignment(self, event):
         """Alignment toggle button event handler"""
 
-        post_command_event(self, AlignmentMsg)
+        post_command_event(self, self.AlignmentMsg)
+
+    def OnMerge(self, event):
+        """Merge button event handler"""
+
+        post_command_event(self, self.MergeMsg)
 
     def OnRotate(self, event):
         """Rotation spin control event handler"""
 
         angle = self.rotation_spinctrl.GetValue()
 
-        post_command_event(self, TextRotationMsg, angle=angle)
+        post_command_event(self, self.TextRotationMsg, angle=angle)
 
-# end of class AttributesToolbar
+# end of class AttributesToolbar
\ No newline at end of file
