@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright 2011 Martin Manns
+# Copyright Martin Manns
 # Distributed under the terms of the GNU General Public License
 
 # --------------------------------------------------------------------
@@ -25,17 +25,19 @@ _widgets
 
 Provides:
 ---------
-  1. CollapsiblePane: Collapsible pane with basic toggle mechanism
-  2. PythonSTC: Syntax highlighting editor
-  3. ImageComboBox: Base class for image combo boxes
-  4. PenStyleComboBox: ComboBox for border pen style selection
-  5. PenWidthComboBox: ComboBox for border pen width selection
-  5. FontChoiceCombobox: ComboBox for font selection
-  6. BorderEditChoice: ComboBox for border selection
-  7. BitmapToggleButton: Button that toggles through a list of bitmaps
-  8. EntryLine: The line for entering cell code
-  9. StatusBar: Main window statusbar
-  10. TableChoiceIntCtrl: IntCtrl for choosing the current grid table
+  1. PythonSTC: Syntax highlighting editor
+  2. ImageComboBox: Base class for image combo boxes
+  3. PenWidthComboBox: ComboBox for pen width selection
+  4. PenStyleComboBox: ComboBox for pen style selection
+  5. MatplotlibStyleChoice: Base class for matplotlib chart style choices
+  6. LineStyleComboBox: ChoiceBox for selection matplotlib line styles
+  7. MarkerStyleComboBox: ChoiceBox for selection matplotlib marker styles
+  8. FontChoiceCombobox: ComboBox for font selection
+  9. BorderEditChoice: ComboBox for border selection
+ 10. BitmapToggleButton: Button that toggles through a list of bitmaps
+ 11. EntryLine: The line for entering cell code
+ 12. StatusBar: Main window statusbar
+ 13. TableChoiceIntCtrl: IntCtrl for choosing the current grid table
 
 """
 
@@ -44,48 +46,18 @@ import keyword
 import wx
 import wx.grid
 import wx.combo
-import wx.stc  as  stc
+import wx.stc as stc
 from wx.lib.intctrl import IntCtrl, EVT_INT
 
-from _events import *
+import src.lib.i18n as i18n
+
+from _events import post_command_event, EntryLineEventMixin, GridCellEventMixin
+from _events import StatusBarEventMixin, GridEventMixin, GridActionEventMixin
 
 from icons import icons
 
-class CollapsiblePane(wx.CollapsiblePane):
-    """Collapsible pane with basic toggle mechanism
-
-    Parameters:
-    -----------
-    panename: string
-    \tLabel for the collapsible pane
-
-    """
-
-    def __init__(self, *args, **kwds):
-        self.label_show = "Show "
-        self.label_hide = "Hide "
-        panename = kwds.pop('panename')
-        self.__dict__['label'] = panename
-        wx.CollapsiblePane.__init__(self, *args, **kwds)
-        self.SetLabel(self.label_show + panename)
-        self.Bind(wx.EVT_COLLAPSIBLEPANE_CHANGED, self.restore_pane, self)
-
-    def OnToggle(self, event):
-        """"Toggle event handler"""
-
-        self.Collapse(self.IsExpanded())
-        self.restore_pane()
-
-    def restore_pane(self, event=None):
-        """Restores the layout of the content and changes teh labels"""
-        self.GetParent().Layout()
-        # and also change the labels
-        if self.IsExpanded():
-            self.SetLabel(self.label_hide + self.__dict__['label'])
-        else:
-            self.SetLabel(self.label_show + self.__dict__['label'])
-
-# end of class CollapsiblePane
+#use ugettext instead of getttext to avoid unicode errors
+_ = i18n.language.ugettext
 
 
 class PythonSTC(stc.StyledTextCtrl):
@@ -129,7 +101,7 @@ class PythonSTC(stc.StyledTextCtrl):
         self.Bind(stc.EVT_STC_MARGINCLICK, self.OnMarginClick)
 
         # Global default styles for all languages
-        self.StyleSetSpec(stc.STC_STYLE_DEFAULT, \
+        self.StyleSetSpec(stc.STC_STYLE_DEFAULT,
                           "face:%(helv)s,size:%(size)d" % self.faces)
         self.StyleClearAll()  # Reset all to be like the default
 
@@ -162,11 +134,11 @@ class PythonSTC(stc.StyledTextCtrl):
         """
 
         self.faces = {'times': 'Times',
-                      'mono' : 'Courier',
-                      'helv' : wx.SystemSettings.GetFont( \
+                      'mono': 'Courier',
+                      'helv': wx.SystemSettings.GetFont(
                                wx.SYS_DEFAULT_GUI_FONT).GetFaceName(),
                       'other': 'new century schoolbook',
-                      'size' : 10,
+                      'size': 10,
                       'size2': 8,
                      }
 
@@ -175,9 +147,9 @@ class PythonSTC(stc.StyledTextCtrl):
         gray1 = "#404040"
         gray2 = "#808080"
 
-        self.fold_symbol_styles = { \
-          "arrows": \
-          [ \
+        self.fold_symbol_styles = {
+          "arrows":
+          [
             (stc.STC_MARKNUM_FOLDEROPEN, stc.STC_MARK_ARROWDOWN, black, black),
             (stc.STC_MARKNUM_FOLDER, stc.STC_MARK_ARROW, black, black),
             (stc.STC_MARKNUM_FOLDERSUB, stc.STC_MARK_EMPTY, black, black),
@@ -185,9 +157,9 @@ class PythonSTC(stc.StyledTextCtrl):
             (stc.STC_MARKNUM_FOLDEREND, stc.STC_MARK_EMPTY, white, black),
             (stc.STC_MARKNUM_FOLDEROPENMID, stc.STC_MARK_EMPTY, white, black),
             (stc.STC_MARKNUM_FOLDERMIDTAIL, stc.STC_MARK_EMPTY, white, black),
-          ], \
-          "plusminus": \
-          [ \
+          ],
+          "plusminus":
+          [
             (stc.STC_MARKNUM_FOLDEROPEN, stc.STC_MARK_MINUS, white, black),
             (stc.STC_MARKNUM_FOLDER, stc.STC_MARK_PLUS,  white, black),
             (stc.STC_MARKNUM_FOLDERSUB, stc.STC_MARK_EMPTY, white, black),
@@ -195,9 +167,9 @@ class PythonSTC(stc.StyledTextCtrl):
             (stc.STC_MARKNUM_FOLDEREND, stc.STC_MARK_EMPTY, white, black),
             (stc.STC_MARKNUM_FOLDEROPENMID, stc.STC_MARK_EMPTY, white, black),
             (stc.STC_MARKNUM_FOLDERMIDTAIL, stc.STC_MARK_EMPTY, white, black),
-          ], \
+          ],
           "circletree":
-          [ \
+          [
             (stc.STC_MARKNUM_FOLDEROPEN, stc.STC_MARK_CIRCLEMINUS,
                                                             white, gray1),
             (stc.STC_MARKNUM_FOLDER, stc.STC_MARK_CIRCLEPLUS, white, gray1),
@@ -210,9 +182,9 @@ class PythonSTC(stc.StyledTextCtrl):
                                                             white, gray1),
             (stc.STC_MARKNUM_FOLDERMIDTAIL, stc.STC_MARK_TCORNERCURVE,
                                                             white, gray1),
-          ], \
+          ],
           "squaretree":
-          [ \
+          [
             (stc.STC_MARKNUM_FOLDEROPEN, stc.STC_MARK_BOXMINUS, white, gray2),
             (stc.STC_MARKNUM_FOLDER, stc.STC_MARK_BOXPLUS, white, gray2),
             (stc.STC_MARKNUM_FOLDERSUB, stc.STC_MARK_VLINE, white, gray2),
@@ -221,8 +193,9 @@ class PythonSTC(stc.StyledTextCtrl):
                                                             white, gray2),
             (stc.STC_MARKNUM_FOLDEROPENMID, stc.STC_MARK_BOXMINUSCONNECTED,
                                                             white, gray2),
-            (stc.STC_MARKNUM_FOLDERMIDTAIL, stc.STC_MARK_TCORNER, white, gray2),
-          ] \
+            (stc.STC_MARKNUM_FOLDERMIDTAIL, stc.STC_MARK_TCORNER,
+                                                            white, gray2),
+          ]
         }
 
         self.fold_symbol_style = self.fold_symbol_styles["circletree"]
@@ -232,55 +205,54 @@ class PythonSTC(stc.StyledTextCtrl):
         -----------
 
         The lexer defines what each style is used for, we just have to define
-        what each style looks like.  The Python style set is adapted from Scintilla
-        sample property files.
+        what each style looks like.  The Python style set is adapted from
+        Scintilla sample property files.
 
         """
 
-
-        self.text_styles = [ \
-          (stc.STC_STYLE_DEFAULT, "face:%(helv)s,size:%(size)d" % self.faces), \
+        self.text_styles = [
+          (stc.STC_STYLE_DEFAULT, "face:%(helv)s,size:%(size)d" % self.faces),
           (stc.STC_STYLE_LINENUMBER, "back:#C0C0C0,face:%(helv)s,"
-                                     "size:%(size2)d" % self.faces), \
-          (stc.STC_STYLE_CONTROLCHAR, "face:%(other)s" % self.faces), \
-          (stc.STC_STYLE_BRACELIGHT, "fore:#FFFFFF,back:#0000FF,bold"), \
-          (stc.STC_STYLE_BRACEBAD, "fore:#000000,back:#FF0000,bold"), \
+                                     "size:%(size2)d" % self.faces),
+          (stc.STC_STYLE_CONTROLCHAR, "face:%(other)s" % self.faces),
+          (stc.STC_STYLE_BRACELIGHT, "fore:#FFFFFF,back:#0000FF,bold"),
+          (stc.STC_STYLE_BRACEBAD, "fore:#000000,back:#FF0000,bold"),
           # Python styles
           # Default
-          (stc.STC_P_DEFAULT, "fore:#000000,face:%(helv)s,size:%(size)d" % \
-                                                                self.faces), \
+          (stc.STC_P_DEFAULT, "fore:#000000,face:%(helv)s,size:%(size)d" %
+                                                                self.faces),
           # Comments
           (stc.STC_P_COMMENTLINE, "fore:#007F00,face:%(other)s,"
-                                  "size:%(size)d" % self.faces), \
+                                  "size:%(size)d" % self.faces),
           # Number
-          (stc.STC_P_NUMBER, "fore:#007F7F,size:%(size)d" % self.faces), \
+          (stc.STC_P_NUMBER, "fore:#007F7F,size:%(size)d" % self.faces),
           # String
-          (stc.STC_P_STRING, "fore:#7F007F,face:%(helv)s,size:%(size)d" % \
-                                                                self.faces), \
+          (stc.STC_P_STRING, "fore:#7F007F,face:%(helv)s,size:%(size)d" %
+                                                                self.faces),
           # Single quoted string
-          (stc.STC_P_CHARACTER, "fore:#7F007F,face:%(helv)s,size:%(size)d" % \
-                                                                self.faces), \
+          (stc.STC_P_CHARACTER, "fore:#7F007F,face:%(helv)s,size:%(size)d" %
+                                                                self.faces),
           # Keyword
-          (stc.STC_P_WORD, "fore:#00007F,bold,size:%(size)d" % self.faces), \
+          (stc.STC_P_WORD, "fore:#00007F,bold,size:%(size)d" % self.faces),
           # Triple quotes
-          (stc.STC_P_TRIPLE, "fore:#7F0000,size:%(size)d" % self.faces), \
+          (stc.STC_P_TRIPLE, "fore:#7F0000,size:%(size)d" % self.faces),
           # Triple double quotes
-          (stc.STC_P_TRIPLEDOUBLE, "fore:#7F0000,size:%(size)d" % self.faces), \
+          (stc.STC_P_TRIPLEDOUBLE, "fore:#7F0000,size:%(size)d" % self.faces),
           # Class name definition
-          (stc.STC_P_CLASSNAME, "fore:#0000FF,bold,underline,size:%(size)d" % \
-                                                                self.faces), \
+          (stc.STC_P_CLASSNAME, "fore:#0000FF,bold,underline,size:%(size)d" %
+                                                                self.faces),
           # Function or method name definition
-          (stc.STC_P_DEFNAME, "fore:#007F7F,bold,size:%(size)d" % self.faces), \
+          (stc.STC_P_DEFNAME, "fore:#007F7F,bold,size:%(size)d" % self.faces),
           # Operators
-          (stc.STC_P_OPERATOR, "bold,size:%(size)d" % self.faces), \
+          (stc.STC_P_OPERATOR, "bold,size:%(size)d" % self.faces),
           # Identifiers
-          (stc.STC_P_IDENTIFIER, "fore:#000000,face:%(helv)s,size:%(size)d" % \
-                                                                self.faces), \
+          (stc.STC_P_IDENTIFIER, "fore:#000000,face:%(helv)s,size:%(size)d" %
+                                                                self.faces),
           # Comment-blocks
-          (stc.STC_P_COMMENTBLOCK, "fore:#7F7F7F,size:%(size)d" % self.faces), \
+          (stc.STC_P_COMMENTBLOCK, "fore:#7F7F7F,size:%(size)d" % self.faces),
           # End of line where string is not closed
           (stc.STC_P_STRINGEOL, "fore:#000000,face:%(mono)s,"
-                                "back:#E0C0E0,eol,size:%(size)d" % self.faces), \
+                                "back:#E0C0E0,eol,size:%(size)d" % self.faces),
         ]
 
     def OnUpdateUI(self, evt):
@@ -313,7 +285,7 @@ class PythonSTC(stc.StyledTextCtrl):
         if brace_at_caret >= 0:
             brace_opposite = self.BraceMatch(brace_at_caret)
 
-        if brace_at_caret != -1  and brace_opposite == -1:
+        if brace_at_caret != -1 and brace_opposite == -1:
             self.BraceBadLight(brace_at_caret)
         else:
             self.BraceHighlight(brace_at_caret, brace_opposite)
@@ -370,7 +342,7 @@ class PythonSTC(stc.StyledTextCtrl):
                     self.SetFoldExpanded(line_num, False)
 
                     if last_child > line_num:
-                        self.HideLines(line_num+1, last_child)
+                        self.HideLines(line_num + 1, last_child)
 
             line_num = line_num + 1
 
@@ -395,11 +367,11 @@ class PythonSTC(stc.StyledTextCtrl):
             if level & stc.STC_FOLDLEVELHEADERFLAG:
                 if force:
                     self.SetFoldExpanded(line, vislevels - 1)
-                    line = self.expand(line, do_expand, force, vislevels-1)
+                    line = self.expand(line, do_expand, force, vislevels - 1)
 
                 else:
                     expandsub = do_expand and self.GetFoldExpanded(line)
-                    line = self.expand(line, expandsub, force, vislevels-1)
+                    line = self.expand(line, expandsub, force, vislevels - 1)
             else:
                 line += 1
 
@@ -440,70 +412,6 @@ class ImageComboBox(wx.combo.OwnerDrawnComboBox):
         dc.SetPen(wx.Pen(bg_color))
         dc.DrawRectangleRect(rect)
 
-class PenStyleComboBox(ImageComboBox):
-    """Combo box for choosing line styles for cell borders
-
-    Stolen from demo.py
-
-    """
-
-    pen_styles = [wx.SOLID, wx.TRANSPARENT, wx.DOT, wx.LONG_DASH, wx.SHORT_DASH,
-                  wx.DOT_DASH, wx.BDIAGONAL_HATCH, wx.CROSSDIAG_HATCH,
-                  wx.FDIAGONAL_HATCH, wx.CROSS_HATCH, wx.HORIZONTAL_HATCH,
-                  wx.VERTICAL_HATCH]
-
-    def OnDrawItem(self, dc, rect, item, flags):
-
-        if item == wx.NOT_FOUND:
-            return
-
-        r = wx.Rect(*rect)  # make a copy
-        r.Deflate(3, 5)
-
-        if not 0 < item < 12:
-            item = 0
-
-        pen_style = self.pen_styles[item]
-
-        pen = wx.Pen(dc.GetTextForeground(), 3, pen_style)
-        dc.SetPen(pen)
-
-        if flags & wx.combo.ODCB_PAINTING_CONTROL:
-            # for painting the control itself
-            dc.DrawLine(r.x+5, r.y+r.height/2,
-                        r.x+r.width - 5, r.y+r.height/2)
-
-        else:
-            # for painting the items in the popup
-            dc.DrawText(self.GetString(item),
-                        r.x + 3,
-                        (r.y + 0) + ((r.height/2) - dc.GetCharHeight())/2)
-            dc.DrawLine(r.x+5, r.y+((r.height/4)*3)+1,
-                        r.x+r.width - 5, r.y+((r.height/4)*3)+1)
-
-    def OnMeasureItem(self, item):
-        """should return the height needed to display an item in the popup,
-        or -1 for default
-
-        Overridden from OwnerDrawnComboBox
-
-        """
-
-        # Simply demonstrate the ability to have variable-height items
-        if item & 1:
-            return 36
-        else:
-            return 24
-
-    def OnMeasureItemWidth(self, item):
-        """Callback for item width, or -1 for default/undetermined
-
-        Overridden from OwnerDrawnComboBox
-
-        """
-        return -1 # default - will be measured from text width
-
-# end of class PenStyleComboBox
 
 class PenWidthComboBox(ImageComboBox):
     """Combo box for choosing line width for cell borders"""
@@ -524,22 +432,111 @@ class PenWidthComboBox(ImageComboBox):
 
         dc.SetPen(pen)
 
-        if flags & wx.combo.ODCB_PAINTING_CONTROL:
-            # for painting the control itself
-            dc.DrawLine(r.x+5, r.y+r.height/2,
-                        r.x+r.width - 5, r.y+r.height/2)
-
-        else:
-            # for painting the items in the popup
-            dc.DrawLine(r.x + 5, r.y + r.height / 2,
-                        r.x + r.width - 5, r.y + r.height / 2)
+        # Draw the example line in the combobox
+        dc.DrawLine(r.x + 5, r.y + r.height / 2,
+                    r.x + r.width - 5, r.y + r.height / 2)
 
 # end of class PenWidthComboBox
 
+# Chart dialog widgets for matplotlib interaction
+# -----------------------------------------------
+
+
+class MatplotlibStyleChoice(wx.Choice):
+    """Base class for line and marker choice for matplotlib charts"""
+
+    # Style label and code are stored in styles as a list of tuples
+    styles = []
+
+    def __init__(self, *args, **kwargs):
+        kwargs["choices"] = [style[0] for style in self.styles]
+        wx.Choice.__init__(self, *args, **kwargs)
+
+    def get_style_code(self, label):
+        """Returns code for given label string
+
+        Inverse of get_code
+
+        Parameters
+        ----------
+        label: String
+        \tLlabel string, field 0 of style tuple
+
+        """
+
+        for style in self.styles:
+            if style[0] == label:
+                return style[1]
+
+        raise ValueError(_("Label {} is invalid.".format(label)))
+
+    def get_label(self, code):
+        """Returns string label for given code string
+
+        Inverse of get_code
+
+        Parameters
+        ----------
+        code: String
+        \tCode string, field 1 of style tuple
+
+        """
+
+        for style in self.styles:
+            if style[1] == code:
+                return style[0]
+
+        raise ValueError(_("Code {} is invalid.".format(code)))
+
+
+class LineStyleComboBox(MatplotlibStyleChoice):
+    """Combo box for choosing line style for matplotlib charts"""
+
+    styles = [
+        ("Solid line style", "-"),
+        ("Dashed line style", "--"),
+        ("Dash-dot line style", "-."),
+        ("Dotted line style", ":"),
+    ]
+
+
+class MarkerStyleComboBox(MatplotlibStyleChoice):
+    """Choice box for choosing matplotlib chart markers"""
+
+    styles = [
+        ("No marker", ""),
+        ("Point marker", "."),
+        ("Pixel marker", ","),
+        ("Circle marker", "o"),
+        ("Triangle_down marker", "v"),
+        ("Triangle_up marker", "^"),
+        ("Triangle_left marker", "<"),
+        ("Triangle_right marker", ">"),
+        ("Tri_down marker", "1"),
+        ("Tri_up marker", "2"),
+        ("Tri_left marker", "3"),
+        ("Tri_right marker", "4"),
+        ("Square marker", "s"),
+        ("Pentagon marker", "p"),
+        ("Star marker", "*"),
+        ("Hexagon1 marker", "h"),
+        ("hexagon2 marker", "H"),
+        ("Plus marker", "+"),
+        ("X marker", "x"),
+        ("Diamond marker", "D"),
+        ("Thin_diamond marker", "d"),
+        ("Vline marker", "|"),
+        ("Hline marker", "_"),
+    ]
+
+
+# End of chart dialog widgets for matplotlib interaction
+# ------------------------------------------------------
+
+
 class FontChoiceCombobox(ImageComboBox):
     """Combo box for choosing fonts"""
 
-
     def OnDrawItem(self, dc, rect, item, flags):
 
         if item == wx.NOT_FOUND:
@@ -549,18 +546,20 @@ class FontChoiceCombobox(ImageComboBox):
         __rect.Deflate(3, 5)
 
         font_string = self.GetString(item)
-        font = wx.Font(wx.DEFAULT, wx.DEFAULT, wx.NORMAL, wx.NORMAL, \
+        font = wx.Font(wx.DEFAULT, wx.DEFAULT, wx.NORMAL, wx.NORMAL,
                        False, font_string)
-        font.SetPointSize(font.GetPointSize()-2)
+        font.SetPointSize(font.GetPointSize() - 2)
         dc.SetFont(font)
         text_width, text_height = dc.GetTextExtent(font_string)
         text_x = __rect.x
         text_y = __rect.y + int((__rect.height - text_height) / 2.0)
+
+        # Draw the example text in the combobox
         dc.DrawText(font_string, text_x, text_y)
 
-
 # end of class FontChoiceCombobox
 
+
 class BorderEditChoice(ImageComboBox):
     """Combo box for selecting the cell borders that shall be changed"""
 
@@ -577,6 +576,7 @@ class BorderEditChoice(ImageComboBox):
         icon = wx.EmptyIcon()
         icon.CopyFromBitmap(bmp)
 
+        # Draw the border icon in the combobox
         dc.DrawIcon(icon, r.x, r.y)
 
     def OnMeasureItem(self, item):
@@ -593,6 +593,7 @@ class BorderEditChoice(ImageComboBox):
 
 # end of class BorderEditChoice
 
+
 class BitmapToggleButton(wx.BitmapButton):
     """Toggle button that goes through a list of bitmaps
 
@@ -644,22 +645,21 @@ class BitmapToggleButton(wx.BitmapButton):
 # end of class BitmapToggleButton
 
 
-class EntryLine(wx.TextCtrl):
+class EntryLine(wx.TextCtrl, EntryLineEventMixin, GridCellEventMixin):
     """"The line for entering cell code"""
 
     def __init__(self, parent, id=-1, *args, **kwargs):
-        wx.TextCtrl.__init__(self, parent, id)
+        wx.TextCtrl.__init__(self, parent, id, *args, **kwargs)
 
         self.parent = parent
         self.ignore_changes = False
 
-        parent.Bind(EVT_ENTRYLINE_MSG, self.OnContentChange)
-        ##parent.Bind(EVT_ENTRYLINE_SELECTION_MSG, self.OnSelectionMsg)
+        parent.Bind(self.EVT_ENTRYLINE_MSG, self.OnContentChange)
 
         self.SetToolTip(wx.ToolTip("Enter Python expression here."))
 
         self.Bind(wx.EVT_TEXT, self.OnText)
-        self.Bind(wx.EVT_CHAR, self.EvtChar)
+        self.Bind(wx.EVT_CHAR, self.OnChar)
 
     def OnContentChange(self, event):
         """Event handler for updating the content"""
@@ -669,22 +669,17 @@ class EntryLine(wx.TextCtrl):
         else:
             self.SetValue(event.text)
 
-    def OnSelectionMsg(self, event):
-        """Event handler for setting selection"""
-
-        self.SetSelection(event.start, event.stop)
+        event.Skip()
 
     def OnText(self, event):
         """Text event method evals the cell and updates the grid"""
 
-        if self.ignore_changes:
-            return
-
-        post_command_event(self, CodeEntryMsg, code=event.GetString())
+        if not self.ignore_changes:
+            post_command_event(self, self.CodeEntryMsg, code=event.GetString())
 
         event.Skip()
 
-    def EvtChar(self, event):
+    def OnChar(self, event):
         """Key event method
 
          * Forces grid update on <Enter> key
@@ -692,23 +687,25 @@ class EntryLine(wx.TextCtrl):
 
         """
 
-        if self.ignore_changes:
-            return
+        if not self.ignore_changes:
 
-        # Handle special keys
+            # Handle special keys
+            keycode = event.GetKeyCode()
 
-        keycode = event.GetKeyCode()
+            if keycode == 13:
+                # <Enter> pressed --> Focus on grid
+                self.parent.grid.SetFocus()
 
-        if keycode == 13:
-            # <Return> pressed --> Focus on grid
-            self.parent.grid.SetFocus()
+                # Ignore <Ctrl> + <Enter> and Quote content
+                if event.ControlDown():
+                    self.SetValue('"' + self.GetValue() + '"')
 
         event.Skip()
 
 # end of class EntryLine
 
 
-class StatusBar(wx.StatusBar):
+class StatusBar(wx.StatusBar, StatusBarEventMixin):
     """Main window statusbar"""
 
     def __init__(self, parent):
@@ -716,7 +713,7 @@ class StatusBar(wx.StatusBar):
 
         self.SetToolTip(wx.ToolTip("Watch the status bar."))
 
-        parent.Bind(EVT_STATUSBAR_MSG, self.OnMessage)
+        parent.Bind(self.EVT_STATUSBAR_MSG, self.OnMessage)
 
     def OnMessage(self, event):
         """Statusbar message event handler"""
@@ -726,7 +723,7 @@ class StatusBar(wx.StatusBar):
 # end of class StatusBar
 
 
-class TableChoiceIntCtrl(IntCtrl):
+class TableChoiceIntCtrl(IntCtrl, GridEventMixin, GridActionEventMixin):
     """ComboBox for choosing the current grid table"""
 
     def __init__(self, parent, no_tabs):
@@ -738,15 +735,16 @@ class TableChoiceIntCtrl(IntCtrl):
         self.SetMin(0)
         self.SetMax(no_tabs - 1)
 
-        self.SetToolTip(wx.ToolTip( \
-            "Enter grid table number or use mouse wheel to change tables."))
+        tipmsg = _("For switching tables enter the table number or "
+                   "use the mouse wheel.")
+        self.SetToolTip(wx.ToolTip(tipmsg))
 
         # State for preventing to post GridActionTableSwitchMsg
         self.switching = False
 
         self.Bind(EVT_INT, self.OnInt)
         self.Bind(wx.EVT_MOUSEWHEEL, self.OnMouseWheel)
-        self.parent.Bind(EVT_COMMAND_RESIZE_GRID, self.OnResizeGrid)
+        self.parent.Bind(self.EVT_CMD_RESIZE_GRID, self.OnResizeGrid)
 
     def change_max(self, no_tabs):
         """Updates to a new number of tables
@@ -760,6 +758,8 @@ class TableChoiceIntCtrl(IntCtrl):
 
         """
 
+        self.no_tabs = no_tabs
+
         if self.GetValue() >= no_tabs:
             self.SetValue(no_tabs - 1)
 
@@ -775,9 +775,15 @@ class TableChoiceIntCtrl(IntCtrl):
     def OnInt(self, event):
         """IntCtrl event method that updates the current table"""
 
+        self.SetMax(self.no_tabs - 1)
+        if event.GetValue() > self.GetMax():
+            print event.GetValue(), self.GetMax()
+            self.SetValue(self.GetMax())
+            return
+
         if not self.switching:
             self.switching = True
-            post_command_event(self, GridActionTableSwitchMsg,
+            post_command_event(self, self.GridActionTableSwitchMsg,
                                newtable=event.GetValue())
             wx.Yield()
             self.switching = False
@@ -785,6 +791,8 @@ class TableChoiceIntCtrl(IntCtrl):
     def OnMouseWheel(self, event):
         """Mouse wheel event handler"""
 
+        self.SetMax(self.no_tabs - 1)
+
         if event.GetWheelRotation() > 0:
             new_table = self.GetValue() + 1
         else:
@@ -800,5 +808,4 @@ class TableChoiceIntCtrl(IntCtrl):
 
         event.Skip()
 
-
-# end of class TableChoiceIntCtrl
+# end of class TableChoiceIntCtrl
\ No newline at end of file
