@@ -35,8 +35,11 @@ Layer 0: KeyValueStore
 """
 
 import ast
+import base64
+import bz2
 from copy import copy
 import cStringIO
+import datetime
 from itertools import imap, ifilter, product
 import re
 import sys
@@ -51,8 +54,11 @@ from src.config import config
 from src.lib.typechecks import is_slice_like, is_string_like, is_generator_like
 from src.lib.selection import Selection
 
+import src.lib.charts as charts
+
 from unredo import UnRedo
 
+
 class KeyValueStore(dict):
     """Key-Value store in memory. Currently a dict with default value None.
 
@@ -67,7 +73,8 @@ class KeyValueStore(dict):
 
 # End of class KeyValueStore
 
-# ------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
+
 
 class CellAttributes(list):
     """Stores cell formatting attributes in a list of 3 - tuples
@@ -103,6 +110,7 @@ class CellAttributes(list):
         "vertical_align": "top",
         "justification": "left",
         "frozen": False,
+        "merge_area": None,
     }
 
     # Cache for __getattr__ maps key to tuple of len and attr_dict
@@ -120,6 +128,7 @@ class CellAttributes(list):
         self.unredo.mark()
 
         self.append(value)
+        self._attr_cache.clear()
 
     def __getitem__(self, key):
         """Returns attribute dict for a single key"""
@@ -133,7 +142,7 @@ class CellAttributes(list):
             if cache_len == len(self):
                 return cache_dict
 
-        row, col, tab  = key
+        row, col, tab = key
 
         result_dict = copy(self.default_cell_attributes)
 
@@ -148,6 +157,7 @@ class CellAttributes(list):
 
 # End of class CellAttributes
 
+
 class ParserMixin(object):
     """Provides parser methods for DictGrid"""
 
@@ -197,10 +207,8 @@ class ParserMixin(object):
                 # Even cols are values
                 attrs[key] = ast.literal_eval(ele)
 
-
         self.cell_attributes.append((selection, tab, attrs))
 
-
     def parse_to_height(self, line):
         """Parses line and inserts row hight"""
 
@@ -208,8 +216,11 @@ class ParserMixin(object):
         row, tab, height = self._split_tidy(line)
         key = self._get_key(row, tab)
 
-        self.row_heights[key] = float(height)
+        try:
+            self.row_heights[key] = float(height)
 
+        except ValueError:
+            pass
 
     def parse_to_width(self, line):
         """Parses line and inserts column width"""
@@ -218,7 +229,11 @@ class ParserMixin(object):
         col, tab, width = self._split_tidy(line)
         key = self._get_key(col, tab)
 
-        self.col_widths[key] = float(width)
+        try:
+            self.col_widths[key] = float(width)
+
+        except ValueError:
+            pass
 
     def parse_to_macro(self, line):
         """Appends line to macro"""
@@ -256,8 +271,6 @@ class StringGeneratorMixin(object):
 
             yield key_str + u"\t" + code_str + u"\n"
 
-
-
     def attributes_to_strings(self):
         """Yields a string that represents the cell attributes for saving
 
@@ -287,7 +300,6 @@ class StringGeneratorMixin(object):
 
             yield u"\t".join(line_list) + u"\n"
 
-
     def heights_to_strings(self):
         """Yields a string that represents the row heights for saving
 
@@ -303,10 +315,10 @@ class StringGeneratorMixin(object):
         yield u"[row_heights]\n"
 
         for row, tab in self.row_heights:
-            height_strings = map(repr, [row, tab, self.row_heights[(row, tab)]])
+            height = self.row_heights[(row, tab)]
+            height_strings = map(repr, [row, tab, height])
             yield u"\t".join(height_strings) + u"\n"
 
-
     def widths_to_strings(self):
         """Yields a string that represents the column widths for saving
 
@@ -322,7 +334,8 @@ class StringGeneratorMixin(object):
         yield u"[col_widths]\n"
 
         for col, tab in self.col_widths:
-            width_strings = map(repr, [col, tab, self.col_widths[(col, tab)]])
+            width = self.col_widths[(col, tab)]
+            width_strings = map(repr, [col, tab, width])
             yield u"\t".join(width_strings) + u"\n"
 
     def macros_to_strings(self):
@@ -371,7 +384,7 @@ class DictGrid(KeyValueStore, ParserMixin, StringGeneratorMixin):
 
         self.macros = u""
 
-        self.row_heights = {} # Keys have the format (row, table)
+        self.row_heights = {}  # Keys have the format (row, table)
         self.col_widths = {}  # Keys have the format (col, table)
 
     def __getitem__(self, key):
@@ -380,14 +393,15 @@ class DictGrid(KeyValueStore, ParserMixin, StringGeneratorMixin):
 
         for axis, key_ele in enumerate(key):
             if shape[axis] <= key_ele or key_ele < -shape[axis]:
-                raise IndexError, "Grid index " + \
-                      str(key) + " outside grid shape " + str(shape)
+                msg = "Grid index {} outside grid shape {}.".format(key, shape)
+                raise IndexError(msg)
 
         return KeyValueStore.__getitem__(self, key)
 
 # End of class DictGrid
 
-# ------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
+
 
 class DataArray(object):
     """DataArray provides enhanced grid read/write access.
@@ -559,15 +573,14 @@ class DataArray(object):
 
             elif is_string_like(key_ele):
                 # We have something string-like here
-
-                raise NotImplementedError, \
-                      "Cell string based access not implemented"
+                msg = "Cell string based access not implemented"
+                raise NotImplementedError(msg)
 
         # key_ele should be a single cell
 
         return self.dict_grid[key]
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key, value, mark_unredo=True):
         """Accepts index and slice keys"""
 
         single_keys_per_dim = []
@@ -612,8 +625,10 @@ class DataArray(object):
 
                     unredo_mark = True
 
-                    undo_operation = (self.__setitem__, [key, old_value])
-                    redo_operation = (self.__setitem__, [key, value])
+                    undo_operation = (self.__setitem__,
+                                      [key, old_value, mark_unredo])
+                    redo_operation = (self.__setitem__,
+                                      [key, value, mark_unredo])
 
                     self.unredo.append(undo_operation, redo_operation)
 
@@ -628,7 +643,7 @@ class DataArray(object):
                 except (KeyError, TypeError):
                     pass
 
-        if unredo_mark:
+        if mark_unredo and unredo_mark:
             self.unredo.mark()
 
     def cell_array_generator(self, key):
@@ -685,9 +700,6 @@ class DataArray(object):
 
         assert axis in [0, 1, 2]
 
-        # Save cell_attributes for undo
-        old_cell_attributes = copy(self.cell_attributes)
-
         if axis < 2:
             # Adjust selections
             for selection, _, _ in self.cell_attributes:
@@ -702,7 +714,8 @@ class DataArray(object):
 
             for pos, tab in cell_sizes:
                 if pos > insertion_point:
-                    new_sizes[(pos+no_to_insert, tab)] = cell_sizes[(pos, tab)]
+                    new_sizes[(pos + no_to_insert, tab)] = \
+                        cell_sizes[(pos, tab)]
                     cell_sizes[(pos, tab)] = None
                 else:
                     new_sizes[(pos, tab)] = cell_sizes[(pos, tab)]
@@ -713,7 +726,7 @@ class DataArray(object):
             # Adjust tabs
             new_tabs = []
             for _, old_tab, _ in self.cell_attributes:
-                new_tabs.append(old_tab + no_to_insert \
+                new_tabs.append(old_tab + no_to_insert
                                 if old_tab > insertion_point else old_tab)
 
             for i, new_tab in new_tabs:
@@ -722,7 +735,7 @@ class DataArray(object):
             self.cell_attributes._attr_cache.clear()
 
         else:
-            raise ValueError, "axis must be in [0, 1, 2]"
+            raise ValueError("Axis must be in [0, 1, 2]")
 
         # Make undoable
 
@@ -749,11 +762,11 @@ class DataArray(object):
         """
 
         if not 0 <= axis <= len(self.shape):
-            raise ValueError, "Axis not in grid dimensions"
+            raise ValueError("Axis not in grid dimensions")
 
         if insertion_point > self.shape[axis] or \
            insertion_point <= -self.shape[axis]:
-            raise IndexError, "Insertion point not in grid"
+            raise IndexError("Insertion point not in grid")
 
         new_keys = {}
 
@@ -771,23 +784,22 @@ class DataArray(object):
 
         self._adjust_cell_attributes(insertion_point, no_to_insert, axis)
 
-
     def delete(self, deletion_point, no_to_delete, axis):
-        """Deletes no_to_delete rows/cols/tabs/... starting with deletion_point
+        """Deletes no_to_delete rows/cols/... starting with deletion_point
 
         Axis specifies number of dimension, i.e. 0 == row, 1 == col, ...
 
         """
 
         if no_to_delete < 0:
-            raise ValueError, "Cannot delete negative number of rows/cols/..."
+            raise ValueError("Cannot delete negative number of rows/cols/...")
 
         if not 0 <= axis <= len(self.shape):
-            raise ValueError, "Axis not in grid dimensions"
+            raise ValueError("Axis not in grid dimensions")
 
         if deletion_point > self.shape[axis] or \
            deletion_point <= -self.shape[axis]:
-            raise IndexError, "Deletion point not in grid"
+            raise IndexError("Deletion point not in grid")
 
         # Store all keys that are moved here because dict iteration is unsorted
         new_key_values = {}
@@ -862,7 +874,8 @@ class DataArray(object):
 
 # End of class DataArray
 
-# ------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
+
 
 class CodeArray(DataArray):
     """CodeArray provides objects when accessing cells via __getitem__
@@ -884,19 +897,19 @@ class CodeArray(DataArray):
     # Cache for frozen objects
     frozen_cache = {}
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key, value, mark_unredo=True):
         """Sets cell code and resets result cache"""
 
         # Prevent unchanged cells from being recalculated on cursor movement
 
         repr_key = repr(key)
 
-        unchanged = (repr_key in self.result_cache and \
+        unchanged = (repr_key in self.result_cache and
                      value == self(key)) or \
-                    ((value is None or value == "") and \
+                    ((value is None or value == "") and
                      repr_key not in self.result_cache)
 
-        DataArray.__setitem__(self, key, value)
+        DataArray.__setitem__(self, key, value, mark_unredo=mark_unredo)
 
         if not unchanged:
             # Reset result cache
@@ -909,7 +922,14 @@ class CodeArray(DataArray):
         if all(type(k) is not SliceType for k in key):
             frozen_res = self.cell_attributes[key]["frozen"]
             if frozen_res:
-                return self.frozen_cache[repr(key)]
+                if repr(key) in self.frozen_cache:
+                    return self.frozen_cache[repr(key)]
+                else:
+                    # Frozen cache is empty.
+                    # Maybe we have a reload without the frozen cache
+                    result = self._eval_cell(key, self(key))
+                    self.frozen_cache[repr(key)] = result
+                    return result
 
         # Normal cell handling
 
@@ -917,7 +937,7 @@ class CodeArray(DataArray):
             return self.result_cache[repr(key)]
 
         elif self(key) is not None:
-            result = self._eval_cell(key)
+            result = self._eval_cell(key, self(key))
             self.result_cache[repr(key)] = result
 
             return result
@@ -957,16 +977,35 @@ class CodeArray(DataArray):
                (not max(op in code[0] for op in self.operators)) and \
                code[0].count("(") == code[0].count(")")
 
-    def _eval_cell(self, key):
+    def _get_updated_environment(self, env_dict=None):
+        """Returns globals environment with 'magic' variable
+
+        Parameters
+        ----------
+        env_dict: Dict, defaults to {'S': self}
+        \tDict that maps global variable name to value
+
+        """
+
+        if env_dict is None:
+            env_dict = {'S': self}
+
+        env = globals().copy()
+        env.update(env_dict)
+
+        return env
+
+    def _eval_cell(self, key, code):
         """Evaluates one cell"""
 
         # Set up environment for evaluation
-        env = globals().copy()
-        env.update( {'X':key[0], 'Y':key[1], 'Z':key[2],
-                     'R':key[0], 'C':key[1], 'T':key[2],
-                     'S':self } )
 
-        code = self(key)
+        env_dict = {'X': key[0], 'Y': key[1], 'Z': key[2], 'bz2': bz2,
+                    'base64': base64, 'charts': charts,
+                    'R': key[0], 'C': key[1], 'T': key[2], 'S': self}
+        env = self._get_updated_environment(env_dict=env_dict)
+
+        _old_code = self(key)
 
         # Return cell value if in safe mode
 
@@ -1002,21 +1041,50 @@ class CodeArray(DataArray):
 
         except AttributeError, err:
             # Attribute Error includes RunTimeError
-            result = err
+            result = AttributeError(err)
 
         except Exception, err:
             result = Exception(err)
 
         # Change back cell value for evaluation from other cells
-        self.dict_grid[key] = code
+        self.dict_grid[key] = _old_code
 
         if glob_var is not None:
             globals().update({glob_var: result})
 
         return result
 
+    def reload_modules(self):
+        """Reloads modules that are available in cells"""
+
+        import src.lib.charts as charts
+        modules = [charts, bz2, base64, re, ast, sys, wx, numpy, datetime]
+
+        for module in modules:
+            reload(module)
+
+    def clear_globals(self):
+        """Clears all newly assigned globals"""
+
+        base_keys = ['cStringIO', 'IntType', 'KeyValueStore', 'UnRedo',
+                     'is_generator_like', 'StringGeneratorMixin',
+                     'is_string_like', 'ParserMixin', 'bz2', 'base64',
+                     '__package__', 're', 'config', '__doc__', 'SliceType',
+                     'CellAttributes', 'product', 'ast', '__builtins__',
+                     '__file__', 'charts', 'sys', 'is_slice_like', '__name__',
+                     'copy', 'imap', 'wx', 'ifilter', 'Selection', 'DictGrid',
+                     'numpy', 'CodeArray', 'DataArray', 'datetime']
+
+        for key in globals().keys():
+            if key not in base_keys:
+                globals().pop(key)
+
     def execute_macros(self):
-        """Executes all macros and returns result string if not safe_mode"""
+        """Executes all macros and returns result string
+
+        Executes macros only when not in safe_mode
+
+        """
 
         if self.safe_mode:
             return "Safe mode activated. Code not executed."
@@ -1024,6 +1092,9 @@ class CodeArray(DataArray):
         # Windows exec does not like Windows newline
         self.macros = self.macros.replace('\r\n', '\n')
 
+        # Set up environment for evaluation
+        globals().update(self._get_updated_environment())
+
         # Create file-like string to capture output
         code_out = cStringIO.StringIO()
         code_err = cStringIO.StringIO()
@@ -1047,6 +1118,12 @@ class CodeArray(DataArray):
         code_out.close()
         code_err.close()
 
+        # Reset result cache
+        self.result_cache.clear()
+
+        # Reset frozen cache
+        self.frozen_cache.clear()
+
         return outstring
 
     def _sorted_keys(self, keys, startkey, reverse=False):
@@ -1089,7 +1166,7 @@ class CodeArray(DataArray):
 
         """
 
-        if type(datastring) is IntType: # Empty cell
+        if type(datastring) is IntType:  # Empty cell
             return None
 
         if flags is None:
@@ -1111,7 +1188,7 @@ class CodeArray(DataArray):
                 matchstring = r'\b' + findstring + r'+\b'
                 for match in re.finditer(matchstring, datastring):
                     pos = match.start()
-                    break # find 1st occurrance
+                    break  # find 1st occurrance
             else:
                 pos = datastring.find(findstring)
 
@@ -1149,6 +1226,4 @@ class CodeArray(DataArray):
                self._string_match(res_str, find_string, flags) is not None:
                 return key
 
-# End of class CodeArray
-
-
+# End of class CodeArray
\ No newline at end of file
