@@ -21,6 +21,7 @@
 
 """
 
+========
 pyspread
 ========
 
@@ -36,24 +37,28 @@ Provides
 
 """
 
+import sys
+from sysvars import get_program_path
+import optparse
+
+import lib.i18n as i18n
+
+import wx
+
+#use ugettext instead of getttext to avoid unicode errors
+_ = i18n.language.ugettext
+
+sys.setrecursionlimit(10000)
+sys.path.insert(0, get_program_path())
+
 # Patch for using with PyScripter thanks to Colin J. Williams
 # If wx exists in sys,modules, we dont need to import wx version.
 # wx is already imported if the PyScripter wx engine is used.
 
-from sys import path, modules
-from sysvars import get_program_path
-
-import optparse
-
-path.insert(0, get_program_path())
-
 try:
-    modules['wx']
+    sys.modules['wx']
 except KeyError:
-    # End of patch
-
     # Select wx version 2.8 if possible
-
     try:
         import wxversion
         wxversion.select(['2.8', '2.9'])
@@ -65,7 +70,7 @@ from wx import App
 from wx import InitAllImageHandlers
 
 
-from src.gui._events import post_command_event, GridActionOpenMsg
+from src.gui._events import post_command_event, GridActionEventMixin
 
 DEBUG = False
 
@@ -83,20 +88,22 @@ class Commandlineparser(object):
 
     def __init__(self):
         from src.config import config
-        usage = "usage: %prog [options] [filename]"
-        version = "%prog " + unicode(config["version"])
+        self.config = config
+
+        usage = _("usage: %prog [options] [filename]")
+        version = _("%prog {}").format(config["version"])
         self.parser = optparse.OptionParser(usage=usage, version=version)
 
-        grid_shape = ( \
+        grid_shape = (
             config["grid_rows"],
             config["grid_columns"],
             config["grid_tables"],
         )
 
         self.parser.add_option("-d", "--dimensions", type="int", nargs=3,
-            dest="dimensions", default=grid_shape, \
-            help="Dimensions of empty grid (works only without filename) "
-                 "rows, cols, tables [default: %default]")
+            dest="dimensions", default=grid_shape,
+            help=_("Dimensions of empty grid (works only without filename) "
+                   "rows, cols, tables [default: %default]"))
 
     def parse(self):
         """
@@ -109,23 +116,27 @@ class Commandlineparser(object):
         """
         options, args = self.parser.parse_args()
 
+        # If one dimension is 0 then the grid has no cells
         if min(options.dimensions) < 1:
-            raise ValueError, "The number of cells in each dimension " + \
-                              "has to be greater than 0"
+            print _("Cell dimension must be > 0.")
+            sys.exit()
 
+        # No MDI yet, pyspread can be started several times though
         if len(args) > 1:
-            raise ValueError, "Only one file may be opened at a time"
-        elif len(args) == 1:
+            print _("Only one file may be opened at a time.")
+            sys.exit()
+
+        filename = None
+        if len(args) == 1:
+            # A filename is provided and hence opened
             filename = args[0]
-        else:
-            filename = None
 
         return options, filename
 
 # end of class Commandlineparser
 
 
-class MainApplication(App):
+class MainApplication(App, GridActionEventMixin):
     """Main application class for pyspread."""
 
     dimensions = (1, 1, 1)  # Will be overridden anyways
@@ -162,7 +173,7 @@ class MainApplication(App):
 
         # Load filename if provided
         if self.filepath is not None:
-            post_command_event(self.main_window, GridActionOpenMsg,
+            post_command_event(self.main_window, self.GridActionOpenMsg,
                                attr={"filepath": self.filepath})
             self.main_window.filepath = self.filepath
 
@@ -187,27 +198,28 @@ class MainApplication(App):
         self.options, self.filepath = cmdp.parse()
 
         if self.filename is None:
-            self.dimensions = self.options.dimensions
-
-
-def __main__():
-    """Compatibility hack"""
-
-    pass
+            rows, columns, tables = self.options.dimensions
+            cmdp.config["grid_rows"] = str(rows)
+            cmdp.config["grid_columns"] = str(columns)
+            cmdp.config["grid_tables"] = str(tables)
 
 
 def main():
     """Parses command line and starts pyspread"""
 
     # Initialize main application
-    app = MainApplication(0)
+    app = MainApplication(redirect=False)
 
     app.MainLoop()
 
 
 if __name__ == "__main__":
-    if DEBUG:
-        import cProfile
-        cProfile.run('main()')
+    if 'unicode' not in wx.PlatformInfo:
+        print "You need a unicode build of wxPython to run this application."
+
     else:
-        main()
\ No newline at end of file
+        if DEBUG:
+            import cProfile
+            cProfile.run('main()')
+        else:
+            main()
\ No newline at end of file
